#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Walapa Muangjeen
#===============================================================================

#version:
#2.0.264 - version update

import os
import sys
import codecs
import logging
import common as cf

logger = logging.getLogger('.'.join([unicode(os.path.splitext(os.path.basename(sys.argv[0]))[0]),'manager','filtergraph',__name__]))
skipclose = True

class aligner(object):

	cfg = {
		'tmbuildname': None,
		'extractcopy': False,
		'tmroot': '%(tmroot)s',
		'forceinfoupdate': '%(forceinfoupdate)s',
		'version': '2.0.264',
		}
	tmbuildname = None
	extractcopy = False
	tmroot = 'bitext'
	forceinfoupdate = False
	paircount = 0
	isopen = False
	p = object
	errors = []

	def open(self,parent,cfg):
		self.tmbuildname = cfg['tmbuildname']
		self.extractcopy = cfg['extractcopy']
		self.tmroot = cfg['tmroot'] if cfg['tmroot'] and not cfg['tmroot'].startswith('%(') and not cfg['tmroot'].endswith(')s') else self.tmroot
		self.forceinfoupdate = self.forceinfoupdate if not cfg['forceinfoupdate'] or (cfg['forceinfoupdate'].startswith('%(') and cfg['forceinfoupdate'].endswith(')s')) else cf.zbool(cfg['forceinfoupdate'])
		self.paircount = 0
		self.filecount = 0
		self.srctokens = 0
		self.tgttokens = 0

		src = os.sep.join([self.p.rootfolder,'BUILDS','tm',self.tmbuildname,os.extsep.join([self.tmroot,self.p.languagepairs[0][0]])])
		tgt = os.sep.join([self.p.rootfolder,'BUILDS','tm',self.tmbuildname,os.extsep.join([self.tmroot,__name__.split(',')[1]])])

		try:
			os.makedirs(os.path.dirname(tgt))
		except OSError,e:
			if not e.errno == 17:
				logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
				raise OSError(e)

		if os.path.exists(src): os.unlink(src)
		if os.path.exists(tgt): os.unlink(tgt)
		if os.path.exists('%s.%s.manifest'%(tgt,self.p.languagepairs[0][0])): os.unlink('%s.%s.manifest'%(tgt,self.p.languagepairs[0][0]))
		if os.path.exists(os.sep.join([os.path.dirname(tgt),'info'])): os.unlink(os.sep.join([os.path.dirname(tgt),'info']))

		self.f = codecs.open(src,'a','utf8')
		self.e = codecs.open(tgt,'a','utf8')
		self.manifest = []

	def run(self,ktgt):
		'''Consolidate data files to build Moses training data'''

		global skipclose
		try:

			ksrc = list(ktgt)
			ksrc[cf.rdrlang] = ktgt[cf.srclang]
			ksrc = tuple(ksrc)
			if not len(self.p.cfoutput[ksrc]['tempbuff']) == len(self.p.cfoutput[ktgt]['tempbuff']):
				self.errors = [__name__,'alignment','\"%s\"=%d, \"%s\"=%d'%(ktgt[cf.srclang],len(self.p.cfoutput[ksrc]['tempbuff']),ktgt[cf.rdrlang],len(self.p.cfoutput[ktgt]['tempbuff']))]
				logger.error('%s\t%s',*self.errors[-1][1:])
				return -2

			self.f.write('%s\n'%('\n'.join(self.p.cfoutput[ksrc]['tempbuff'])))
			self.e.write('%s\n'%('\n'.join(self.p.cfoutput[ktgt]['tempbuff'])))
			self.manifest.append('\t'.join(self.p.cfinput[ktgt]['filelist']))

			self.paircount += len(self.p.cfoutput[ktgt]['tempbuff'])
			self.filecount += 1
			self.srctokens += sum([len(line.split()) for line in self.p.cfoutput[ksrc]['tempbuff']])
			self.tgttokens += sum([len(line.split()) for line in self.p.cfoutput[ktgt]['tempbuff']])

			skipclose = False

		except KeyboardInterrupt:
			self.e.close()
			self.f.close()
			os.unlink(self.e.name)
			os.unlink(self.f.name)
			skipclose = True
			raise KeyboardInterrupt()

	def flush(self,ktgt):
		if self.extractcopy: return
		for k in self.p.cfoutput.keys():
			self.p.cfoutput[k]['tempbuff'] = [''] * len(self.p.cfoutput[k]['tempbuff'])

	def close(self):
		'''close filter'''
		if skipclose: return

		self.f.close()
		self.e.close()

		with codecs.open('%s.%s.manifest'%(self.e.name,self.p.languagepairs[0][0]),'w','utf8') as f:
			f.write('%s: %s\n'%(self.tmbuildname,cf.timestamp()))
			f.write('file pairs: %s\n'%(self.filecount))
			f.write('phrase pairs: %s\n'%(self.paircount))
			f.write('%s (source) tokens: %s\n'%(os.path.splitext(self.f.name)[1].strip('.'),self.srctokens))
			f.write('%s (target) tokens: %s\n'%(os.path.splitext(self.e.name)[1].strip('.'),self.tgttokens))
			f.write('%s\n'%('\n'.join(self.manifest)))

		if not os.path.getsize(self.e.name) or not os.path.getsize(self.f.name) or not os.path.getsize('%s.%s.manifest'%(self.e.name,self.p.languagepairs[0][0])):
			os.unlink(self.e.name)
			os.unlink(self.f.name)
			os.unlink('%s.%s.manifest'%(self.e.name,self.p.languagepairs[0][0]))

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import filter\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0]
	)

licensetxt=u'''DoMY™ CE v2.0
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stdout.write(usage().encode('utf8')+'\n')
