#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#2.0.264 - version update

import os
import sys
import string
import logging
import common as cf





logger = logging.getLogger('.'.join([unicode(os.path.splitext(os.path.basename(sys.argv[0]))[0]),'manager','filtergraph',__name__]))

class aligner(object):

	cfg = {
		'maxtoken': 65,
		'mintoken': 1,
		'ratio': '5:1',
		'version': '2.0.264',
		}
	maxtoken = 65
	mintoken = 1
	max_word_length = 1000
	ratio = '5:1'
	lowerlimit = 0
	upperlimit = 0
	isopen = False
	p = object
	errors = []

	def open(self,parent,cfg):
		try:
			self.mintoken = 1 if abs(int(cfg['mintoken'])) == 0 else abs(int(cfg['mintoken']))
		except:
			self.mintoken = 1
		try:
			self.maxtoken = 65 if abs(int(cfg['maxtoken'])) == 0 else abs(int(cfg['maxtoken']))
		except:
			self.maxtoken = 65
		self.maxtoken = 100 if self.maxtoken == 0 or self.maxtoken > 100 else self.maxtoken
		if self.maxtoken and self.mintoken >= self.maxtoken:
			self.errors.append([__name__,'invalid','\"[%s]\" \"mintoken=%d\" greater than \"maxtoken=%d\"'%(__name__,self.mintoken,self.maxtoken)])
			logger.error('%s\t%s',*self.errors[-1][1:])
		try:
			ratio = [ float(item.strip()) for item in cfg['ratio'].split(':') ]
			test = 1 / sum(ratio)
		except:
			self.errors.append([__name__,'invalid','[%s] \"ratio=\" - %s'%(__name__,str(e))])
			logger.error('%s\t%s',*self.errors[-1][1:])
		else:
			if len(ratio) == 1: ratio.append(1.0)
			if ratio[0] < ratio[1]:
				self.lowerlimit = 0.1 if (ratio[0] / sum(ratio)) < 0.1 else ratio[0] / sum(ratio)
				self.upperlimit = 0.9 if (ratio[1] / sum(ratio)) > 0.9 else ratio[1] / sum(ratio)
			else:
				self.lowerlimit = 0.1 if (ratio[1] / sum(ratio)) < 0.1 else ratio[1] / sum(ratio)
				self.upperlimit = 0.9 if (ratio[0] / sum(ratio)) > 0.9 else ratio[0] / sum(ratio)

	def run(self, ktgt):
		'''run filter'''
		# source key
		ksrc = list(ktgt)
		ksrc[cf.rdrlang] = ktgt[cf.srclang]
		ksrc = tuple(ksrc)
		# tgt langauge model key
		ktgtlm = list(ktgt)
		ktgtlm[cf.kind] = ktgtlm[cf.kind].replace(u'tm',u'lm',1)
		ktgtlm = tuple(ktgtlm)
		# src language model key
		ksrclm = list(ksrc)
		ksrclm[cf.kind] = ksrclm[cf.kind].replace(u'tm',u'lm',1)
		ksrclm = tuple(ksrclm)
		#move unequal sentence pairs to cferror
		if not len(self.p.cfoutput[ktgt]['tempbuff']) == len(self.p.cfoutput[ksrc]['tempbuff']):
			self.p.cferror[ktgtlm]['tempbuff'] = self.p.cfoutput[ktgt]['tempbuff']
			self.p.cferror[ksrclm]['tempbuff'] = self.p.cfoutput[ksrc]['tempbuff']
			self.p.cfoutput[ktgt]['tempbuff'] = []
			self.p.cfoutput[ksrc]['tempbuff'] = []
			logger.error('%s\t%s',*['mismatch','\"[%s]\" \"Source and target files are different lengths: %s'%(__name__,os.sep.join(self.p.filelist))])
			return
		# drop empty pairs
		# move element with corresponding empty elements to cferror (lm)
		# move pairs with too short or too long elements to cferror (tm)
		# move pairs that violate the GIZA++ sentence ratio limit to cferror (tm)
		# move pairs that violate maximum word length of 1000 characters
		for linenum in reversed(range(len(self.p.cfoutput[ktgt]['tempbuff']))):
			# count target/source tokens
			tkncnttgt = len(self.p.cfoutput[ktgt]['tempbuff'][linenum].strip().split()) if self.p.cfoutput[ktgt]['tempbuff'][linenum].strip() else 0
			tkncntsrc = len(self.p.cfoutput[ksrc]['tempbuff'][linenum].strip().split()) if self.p.cfoutput[ksrc]['tempbuff'][linenum].strip() else 0
			# find maximum word length
			maxwdlentgt = len(max(self.p.cfoutput[ktgt]['tempbuff'][linenum].split(),key=len)) if self.p.cfoutput[ktgt]['tempbuff'][linenum] else 0
			maxwdlensrc = len(max(self.p.cfoutput[ksrc]['tempbuff'][linenum].split(),key=len)) if self.p.cfoutput[ksrc]['tempbuff'][linenum] else 0
			# measure if in/out of ratio
			ratio = 1.0 if not sum([tkncnttgt,tkncntsrc]) else float(tkncnttgt) / sum([tkncnttgt,tkncntsrc])
			f_move = not(self.lowerlimit <= ratio <= self.upperlimit)
			f_move = not(self.mintoken <= tkncnttgt <= self.maxtoken) or f_move
			f_move = not(self.mintoken <= tkncntsrc <= self.maxtoken) or f_move
			f_move = maxwdlentgt >= self.max_word_length or f_move
			f_move = maxwdlensrc >= self.max_word_length or f_move

			if f_move:
				tgttxt = self.p.cfoutput[ktgt]['tempbuff'].pop(linenum).strip()
				srctxt = self.p.cfoutput[ksrc]['tempbuff'].pop(linenum).strip()
				tgtlen = len(tgttxt.strip(string.punctuation+' \t'))
				srclen = len(srctxt.strip(string.punctuation+' \t'))
				if tgtlen and srclen:
					self.p.cferror[ksrc]['tempbuff'].append(srctxt)
					self.p.cferror[ktgt]['tempbuff'].append(tgttxt)
				elif srclen and not tgtlen:
					self.p.cferror[ksrclm]['tempbuff'].append(srctxt)
				elif tgtlen and not srclen:
					self.p.cferror[ktgtlm]['tempbuff'].append(tgttxt)
		for k in self.p.cferror:
			self.p.cferror[k]['tempbuff'].reverse()

	def flush(self, ktgt):
		return

	def close(self):
		return

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import filter\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0]
	)

licensetxt=u'''DoMY™ CE v2.0
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stdout.write(usage().encode('utf8')+'\n')
