#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#2.5.287 - updated gzip to use smaller chunks

import os
import sys
import subprocess
import time
import shutil
import logging
import common as cf
from glob import glob
from flock import flock
import gzip

logger = logging.getLogger('.'.join([os.path.splitext(os.path.basename(sys.argv[0]))[0],'manager','filtergraph',__name__]))
skipclose = True

class filter(object):

	cfg = {
		'args': {},
		'exeoptions': '',
		'kw': {
			'bufsize': 0,
			'executable': '4,compile-lm',
			'stdin': 'PIPE',
			'stdout': 'PIPE',
			'stderr': 'PIPE',
			'preexec_fn': None,
			'close_fds': False,
			'shell': False,
			'cwd': os.sep+'tmp',
			'env': 'LC_ALL=C',
			'universal_newlines': False,
			'startupinfo': None,
			'creationflags': 0,
			},
		'script': '',
		'version': '2.5.287',

		'encoding': 'utf8',
		'recaser': '%(recaser)s',
		'arpa': False,
		'forceinfoupdate': '%(forceinfoupdate)s',
		'lmbuildname': None,
		'e': None,
		'lmgrams': 0,
		'tmpdir': '',
		}
	kw = {}
	encoding = 'utf8'
	recaser = False
	arpa = False
	forceinfoupdate = False
	lmbuildname = None
	e = None
	lmtype = __name__.split(',')[-1].split('-')[-1].lower()
	lmgrams = 5
	subproc = {}
	tmpdir = ''
	deltemp = True
	inputs = []
	logfile = ''
	outputs = []
	isopen = False
	p = object
	errors = []

	def __del__(self):
		if not 'popen' in self.subproc: return
		while self.subproc['popen'].poll() is None:
			self.subproc['popen'].terminate()
			time.sleep(0.2)
		if self.deltemp and os.path.exists(self.tmpdir):
			shutil.rmtree(self.tmpdir)
		for tmpfile in glob(os.sep.join(['','tmp','dskbuff--*'])):
			os.unlink(tmpfile)

	def open(self,parent,cfg):

		self.kw = dict(cfg['kw'])
		self.kw['stdin'] = subprocess.PIPE if self.kw['stdin'] == 'PIPE' else self.kw['stdin']
		self.kw['stdout'] = subprocess.PIPE if self.kw['stdout'] == 'PIPE' else self.kw['stdout']
		self.kw['stderr'] = subprocess.STDOUT if cfg['kw']['stderr'] == 'PIPE' else cfg['kw']['stderr']

		# find program
		self.kw['executable'] = self.p.decodepath(self.kw['executable'],True)
		if not self.kw['executable']:
			self.errors.append([__name__,'missing','\"[%s]\" \"%s\" program not found'%(__name__,cfg['executable'])])
			logger.error('%s\t%s',*self.errors[-1][1:])

		# common options
		self.encoding = 'utf8' if 'utf8' in cfg['encoding'].lower().replace('-','') else cfg['encoding']

		self.lmtype = __name__.split(',')[-1].split('-')[-1].lower()
		if not self.lmtype.upper() in os.environ:
			self.errors.append([__name__,'missing','\"[%s]\" \"%s\" environment variable'%(__name__,self.lmtype.upper())])
			logger.error('%s\t%s',*self.errors[-1][1:])
		self.e = cfg['e'] if not cfg['e'].startswith('%(') and not cfg['e'].endswith(')s') else None
		if not self.e:
			self.errors.append([__name__,'missing','\"[%s]\" \"e=\"'%(__name__)])
			logger.error('%s\t%s',*self.errors[-1][1:])
		self.lmbuildname = cfg['lmbuildname'] if not cfg['lmbuildname'].startswith('%(') and not cfg['lmbuildname'].endswith(')s') else None
		if not self.lmbuildname:
			self.errors.append([__name__,'missing','\"[%s]\" \"lmbuildname=\"'%(__name__)])
			logger.error('%s\t%s',*self.errors[-1][1:])
		self.lmgrams = cfg['lmgrams'] if cfg['lmgrams'] else self.lmgrams

		self.recaser = self.recaser if not cfg['recaser'] or (cfg['recaser'].startswith('%(') and cfg['recaser'].endswith(')s')) else cf.zbool(cfg['recaser'])
		self.forceinfoupdate = self.forceinfoupdate if not cfg['forceinfoupdate'] or (cfg['forceinfoupdate'].startswith('%(') and cfg['forceinfoupdate'].endswith(')s')) else cf.zbool(cfg['forceinfoupdate'])

		# type-specific options
		self.arpa = cfg['arpa']

		# other options
		self.tmpdir = cfg['tmpdir'] if cfg['tmpdir'] else '%s.tmp'%(os.sep.join([self.p.tempdir,__name__]))

		# get input and output and file names
		if self.recaser:
			in_dir = os.sep.join([
				self.p.rootfolder,
				'ENGINES',
				'recasers',
				'%s-t=%s-l=%s-T=%s-n=%s-a=%s-g=%s'%('recaser',self.e,self.lmbuildname,self.lmtype,self.lmgrams,self.alignertype,self.tmgrams),
				])
		else:
			in_dir = os.sep.join([
				self.p.rootfolder,
				'ENGINES',
				'lms',
				'%s-t=%s-l=%s-T=%s-n=%s'%('lm',self.e,self.lmbuildname,self.lmtype,self.lmgrams,),
				])
		in_base = '%s.%s.%s'%(self.lmtype,self.e,'gz')

		self.inputs.append(os.sep.join([in_dir,in_base])
		self.outputs.append(self.inputs[0].replace('%s.%s.gz'%(self.lmtype,self.e),'irstlm_arpa.%s.gz'%(self.e) if self.arpa else '%s.%s.mm'%(self.lmtype,self.e)))

	def run(self,k):
		global skipclose
		skipclose = False

	def flush(self,k):
		return

	def close(self):
		if skipclose: return

		lockfile = os.sep.join(['','var','tmp','domy','train.lock'])
		if not os.path.exists(os.path.dirname(lockfile)): os.makedirs(os.path.dirname(lockfile))
		lock = flock(lockfile,False).acquire()
		if not lock or not lock.ownlock():
			self.errors.append([__name__,'failed','train lock, another training session is running'])
			logger.error('%s\t%s',*self.errors[-1][1:])
			return
		with open(lockfile) as f:
			lockcontent = f.readline().rstrip()

		# test input and output files
		infodata = cf.updatefingerprint(self.inputs,self.outputs,self.forceinfoupdate)
		if 'bypass' in infodata:
			# output file(s) exists.
			logger.info('%s\t%s',*['skipping','outputs exist and verified'])
			sys.stderr.write('\n%s - Skipping. [%s] Outputs exist and verified\n'%(cf.timestamp(),__name__))
			lock.release()
			return
		elif 'errors' in infodata:
			# report/return self.errors
			for error in infodata['errors']:
				self.errors.append([__name__,'error',error.strip()])
				logger.error('%s\t%s',*self.errors[-1][1:])
			lock.release()
			return
		sys.stderr.flush()

		logger.info('%s\t%s',*['start','compile-lm IRSTLM - %s'%('arpa' if self.arpa else 'binarize')])
		sys.stderr.write('\nStart -%s %s - compile-lm IRSTLM-%s [%s]\n'%(' (recaser)' if self.recaser else '',self.e,'arpa' if self.arpa else 'binarize',__name__))

		# make tmpdir
		try:
			os.makedirs(self.tmpdir)
		except OSError,e:
			if e.errno == 17:
				self.deltemp = False
			else:
				logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
				raise OSError(e)

		# make output folder
		try:
			os.makedirs(os.path.dirname(self.outputs[0]))
		except OSError,e:
			if not e.errno == 17:
				logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
				raise OSError(e)

		# create command line args list
		args = [
			'',
			'--text','yes' if self.arpa else 'no',
			self.inputs[0],
			self.outputs[0].replace('%s_arpa.%s.gz'%(self.lmtype,self.e),'%s_arpa.%s'%(self.lmtype,self.e)),
			]

		# run subprocess
		try:
			self.subproc['popen'] = subprocess.Popen(args,**self.kw)
		except Exception,e:
			self.errors.append([__name__,'%s %s - %s'%(os.path.basename(self.kw['executable']),' '.join(args),str(e))])
			logger.error('%s\t%s',*self.errors[-1][1:])
			lock.release()
			return
		logger.info('%s\t%d - %s %s',*['run',self.subproc['popen'].pid,os.path.basename(self.kw['executable']),' '.join(args)])
		sys.stderr.write('run (%s %s) compile-lm IRSTLM - %s\n'%(self.e,'recaser' if self.recaser else 'lm','arpa' if self.arpa else 'binarize'))
		try:
			for line in iter(self.subproc['popen'].stdout.readline, ''):
				if not line.strip(): continue
				try:
					logger.info('%s\t%d, %s, %s: %s',*[os.path.basename(self.kw['executable']),self.subproc['popen'].pid,self.e,'recaser' if self.recaser else 'lm',line.rstrip().decode(self.encoding),])
					sys.stderr.write('    (%s %s) %s\n'%(self.e,'recaser' if self.recaser else 'lm',line.rstrip().decode(self.encoding)))
				except:
					pass
		except KeyboardInterrupt:
			# trap KeyboardInterrupt to delete dskbuff temp files
			for tmpfile in glob(os.sep.join(['','tmp','dskbuff--*'])):
				os.unlink(tmpfile)
			while self.subproc['popen'].poll() is None:
				self.subproc['popen'].terminate()
				time.sleep(0.2)
			raise KeyboardInterrupt()

		for tmpfile in glob(os.sep.join(['','tmp','dskbuff--*'])):
			os.unlink(tmpfile)
		# compress arpa file to self.outputs
		if self.arpa: self.__compressarpa()
		if self.deltemp: shutil.rmtree(self.tmpdir)


		while self.subproc['popen'].poll() is None:
			time.sleep(0.2)

		if self.subproc['popen'].returncode:
			self.errors.append([__name__,'failed','compile-lm IRSTLM - %s'%('arpa' if self.arpa else 'binarize')])
			logger.error('%s\t%s',*self.errors[-1][1:])
		else:
			# update infocontent and save
			infodata = cf.updatefingerprint([],self.outputs,False,infodata)
			if 'errors' in infodata:
				# report/return self.errors
				for error in infodata['errors']:
					self.errors.append([__name__,'error',error.strip()])
					logger.error('%s\t%s',*self.errors[-1][1:])
			else:
				logger.info('%s\t%s',*['success','compile-lm IRSTLM - %s'%('arpa' if self.arpa else 'binarize')])
				sys.stderr.write('Success - %s - %s - compile-lm IRSTLM - %s\n'%(self.e,'recaser' if self.recaser else 'lm','arpa' if self.arpa else 'binarize'))

		sys.stderr.flush()
		lock.release()

	def __compressarpa(self):
		if not  os.path.exists(self.outputs[0].replace('%s_arpa.%s.gz'%(self.lmtype,self.e),'%s_arpa.%s'%(self.lmtype,self.e))): return
		logger.info('%s\t%s',*['run','compress IRSTLM - %s'%('arpa' if self.arpa else 'binarize')])
		sys.stderr.write('    (%s %s) compress IRSTLM - %s\n'%(self.e,'recaser' if self.recaser else 'lm','arpa' if self.arpa else 'binarize'))
		f_out = gzip.open(self.outputs[0],'wb')
		f_in = open(self.outputs[0].replace('%s_arpa.%s.gz'%(self.lmtype,self.e),'%s_arpa.%s'%(self.lmtype,self.e)),'rb')
		while True:
			data = f_in.read(65536)
			if not data: break
			f_out.write(data)
		f_out.close()
		f_in.close()
		os.unlink(self.outputs[0].replace('%s_arpa.%s.gz'%(self.lmtype,self.e),'%s_arpa.%s'%(self.lmtype,self.e)))

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import filter\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0]
	)

licensetxt=u'''DoMY™ CE v2.5
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stdout.write(usage().encode('utf8')+'\n')
