#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#2.5.329 - update display to clarify t-table display

import os
import sys
import subprocess
import time
import shutil
import logging
import common as cf
import glob
import flock
import gzip

logger = logging.getLogger('.'.join([os.path.splitext(os.path.basename(sys.argv[0]))[0],'manager','filtergraph',__name__]))
skipclose = True

class filter(object):

	cfg = {
		'args': {},
		'exeoptions': '',
		'kw': {
			'bufsize': 0,
			'executable': '4,processPhraseTable',
			'stdin': 'PIPE',
			'stdout': 'PIPE',
			'stderr': 'PIPE',
			'preexec_fn': None,
			'close_fds': False,
			'shell': False,
			'cwd': os.sep+'tmp',
			'env': 'LC_ALL=C',
			'universal_newlines': False,
			'startupinfo': None,
			'creationflags': 0,
			},
		'script': '',
		'encoding': 'utf8',
		'model-dir': '',
		'forceinfoupdate': '%(forceinfoupdate)s',
		'hierarchical': False,
		'alignertype': 'giza',
		'tmbuildname': '',
		'e': '',
		'f': '',
		'tmgrams': 7,
		'version': '2.5.329',

		'tmpdir': '',
		'inputs': {},
		'logfile': '',
		'outputs': {},
		}
	kw = {}
	encoding = 'utf8'
	subproc = {}
	model_dir = ''
	forceinfoupdate = False
	hierarchical = False
	alignertype = 'giza'
	tmbuildname = None
	e = None
	f = None
	tmgrams = 7
	tmpdir = ''
	inputs = {}
	logfile = ''
	outputs = {}
	isopen = False
	p = object
	errors = []

	def __del__(self):
		if self.tmpdir and os.path.exists(self.tmpdir): shutil.rmtree(self.tmpdir)
		try:
			while self.subproc['popen'].poll() is None:
				self.subproc['popen'].terminate()
				time.sleep(0.2)
		except:
			pass

	def open(self,parent,cfg):
		self.kw = dict(cfg['kw'])
		self.kw['stdin'] = subprocess.PIPE if self.kw['stdin'] == 'PIPE' else self.kw['stdin']
		self.kw['stdout'] = subprocess.PIPE if self.kw['stdout'] == 'PIPE' else self.kw['stdout']
		self.kw['stderr'] = subprocess.PIPE if self.kw['stderr'] == 'PIPE' else self.kw['stderr']

		# find program
		self.kw['executable'] = self.p.decodepath(self.kw['executable'],True)
		if not self.kw['executable']:
			self.errors.append([__name__,'missing','\"[%s]\" \"%s\" program not found'%(__name__,self.kw['executable'])])
			logger.error('%s\t%s',*self.errors[-1][1:])

		# common options
		self.encoding = 'utf8' if 'utf8' in cfg['encoding'].lower().replace('-','') else cfg['encoding']
		self.forceinfoupdate = self.forceinfoupdate if not cfg['forceinfoupdate'] or (cfg['forceinfoupdate'].startswith('%(') and cfg['forceinfoupdate'].endswith(')s')) else cf.zbool(cfg['forceinfoupdate'])
		self.tmpdir = cfg['tmpdir'] if cfg['tmpdir'] else os.sep.join([self.p.tempdir,__name__])
		self.hierarchical = cfg['hierarchical']

		self.model_dir = cfg['model-dir'].replace('%(rootfolder)s',self.p.rootfolder) if cfg['model-dir'] else self.model_dir

		if not self.model_dir:

			if cfg['tmbuildname']:
				self.tmbuildname = str(cfg['tmbuildname'])
			else:
				self.errors.append([__name__,'missing','\"[%s]\" tmbuildname must be set'%(__name__,)])
				logger.warn('%s\t%s',*self.errors[-1][1:])

			if 'giza' in cfg['alignertype'].lower():
				self.alignertype = 'giza'
			elif 'berk' in cfg['alignertype'].lower():
				self.alignertype = 'berk'
			else:
				self.errors.append([__name__,'invalid','\"[%s]\" alignertype must be set to \'giza\', or \'berk\''%(__name__,)])
				logger.error('%s\t%s',*self.errors[-1][1:])

			if not cfg['f'] or cfg['f'] == '%(f)s' or cfg['f'] == '%(srclang)s':
				self.errors.append([__name__,'missing','\"[%s]\" f='%(__name__,)])
				logger.error('%s\t%s',*self.errors[-1][1:])
			if not cfg['e'] or cfg['e'] == '%(e)s' or cfg['e'] == '%(tgtlang)s':
				self.errors.append([__name__,'missing','\"[%s]\" e='%(__name__,)])
				logger.error('%s\t%s',*self.errors[-1][1:])
			self.e = cfg['e']
			self.f = cfg['f']

			if cfg['tmgrams'] > 9:
				logger.warn('%s\t%s',*['warning','\"[%s]\" tmgrams=%d is very large and will consume significant resources'%(__name__,cfg['tmgrams'])])
			self.tmgrams = cfg['tmgrams']

			# get input and output filelist and file name
			self.model_dir = os.sep.join([self.p.rootfolder,'ENGINES','tables','%s-s=%s-t=%s-p=%s-a=%s-g=%d'%('rule' if self.hierarchical else 'phrase',self.f,self.e,self.tmbuildname,self.alignertype,self.tmgrams)])

	def run(self,k):
		global skipclose
		skipclose = False

	def flush(self,k):
		return

	def close(self):
		if skipclose: return

		lockfile = os.sep.join(['','var','tmp','domy','train.lock'])
		if not os.path.exists(os.path.dirname(lockfile)): os.makedirs(os.path.dirname(lockfile))
		lock = flock.flock(lockfile,False).acquire()
		if not lock or not lock.ownlock():
			self.errors.append([__name__,'failed','train lock, another training session is running'])
			logger.error('%s\t%s',*self.errors[-1][1:])
			return
		with open(lockfile) as f:
			lockcontent = f.readline().rstrip()

		for path in glob.glob(os.sep.join([self.model_dir,'%s*.gz'%('rule-table*' if self.hierarchical else 'phrase-table*')])):
			self.inputs[path] = [path]
			self.outputs[path] = [
				self.inputs[path][0].replace('.gz','.binphr.idx'),
				self.inputs[path][0].replace('.gz','.binphr.srctree'),
				self.inputs[path][0].replace('.gz','.binphr.srcvoc'),
				self.inputs[path][0].replace('.gz','.binphr.tgtdata'),
				self.inputs[path][0].replace('.gz','.binphr.tgtvoc'),
				]

		if not self.inputs:
			self.errors.append([__name__,'missing','\"[%s]\" \"%s\" input not found'%(__name__,os.sep.join([self.model_dir,'%s*.gz'%('rule-table' if self.hierarchical else 'phrase-table')]))])
			logger.error('%s\t%s',*self.errors[-1][1:])

		for k in self.inputs:

			# test input and output files
			infodata = cf.updatefingerprint(self.inputs[k],self.outputs[k],self.forceinfoupdate)
			if 'bypass' in infodata:
				# output file(s) exists.
				logger.info('%s\t%s',*['skipping','outputs exist and verified'])
				sys.stderr.write('\n%s - Skipping. [%s] Outputs exist and verified\n'%(cf.timestamp(),__name__))
				sys.stderr.flush()
				continue
			elif 'errors' in infodata:
				# report/return self.errors
				for error in infodata['errors']:
					self.errors.append([__name__,'error',error.strip()])
					logger.error('%s\t%s',*self.errors[-1][1:])
				break

			logger.info('%s\t%s',*['start','processPhraseTable'])
			sys.stderr.write('\nStart - processPhraseTable [%s]\n'%(__name__))

			# make tmpdir
			try:
				os.makedirs(self.tmpdir)
			except OSError,e:
				if not e.errno == 17:
					logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
					raise OSError(e)
			else:
				unziped = os.sep.join([self.tmpdir,os.path.basename(self.inputs[k][0].replace('.gz','.raw'))])
				xsorted = os.sep.join([self.tmpdir,os.path.basename(self.inputs[k][0].replace('.gz',''))])

			# 1) Unzip the tarball
			if self.inputs[k][0].endswith('.gz'):
				logger.info('%s\t%s',*['expand',self.inputs[k][0]])
				sys.stderr.write('Expand... ')
				f_in = gzip.open(self.inputs[k][0],'rb')
				f_out = open(unziped, 'wb')
				while True:
					data = f_in.read(65536)
					if not data: break
					f_out.write(data)
				f_out.close()
				f_in.close()

			else:
				shutil.copyfile(self.inputs[k][0],unziped)

			# 2) sort contents
			if not os.path.exists(unziped) or os.path.getsize(unziped) < 35:

				self.errors.append([__name__,'failed','gunzip %s to %s'%(self.inputs[k][0],unziped)])
				logger.error('%s\t%s',*self.errors[-1][1:])
				if os.path.exists(unziped): os.unlink(unziped)

			else:

				logger.info('%s\t%s',*['sort',unziped])
				sys.stderr.write('Sort...\n')

				args = ['', '-o', xsorted, unziped]
				kw = dict(self.kw)
				kw['executable'] = 'sort'
				try:
					self.subproc['popen'] = subprocess.Popen(args,**kw)
				except Exception,e:
					self.errors.append([__name__,'failed','%s %s - %s'%(os.path.basename(kw['executable']),' '.join(args),str(e))])
					logger.error('%s\t%s',*self.errors[-1][1:])
				else:
					stdout,stderr = self.subproc['popen'].communicate('\n')
				finally:
					if os.path.exists(unziped):
						logger.info('%s\t%s',*['delete',unziped])
						os.unlink(unziped)

			# 3) processPhraseTable -ttable 0 0 self.inputs[k] -nscores "self.tmgrams" -out "self.outputs[k]"
			if not os.path.exists(xsorted) or os.path.getsize(xsorted) < 35:

				self.errors.append([__name__,'failed','sort %s to %s'%(unziped,xsorted)])
				logger.error('%s\t%s',*self.errors[-1][1:])
				if os.path.exists(xsorted): os.unlink(xsorted)

			else:

				logger.info('%s\t%s',*['binarize',xsorted])

				# make output folder
				for out in self.outputs[k]:
					try:
						os.makedirs(os.path.dirname(out))
					except OSError,e:
						if not e.errno == 17:
							logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
							raise OSError(e)

				args = [
					'',
					'-ttable','0','0',xsorted,
					'-nscores',str(self.tmgrams),
					'-out',self.inputs[k][0].replace('.gz',''),
					]
				try:
					self.subproc['popen'] = subprocess.Popen(args,**self.kw)
				except Exception,e:
					self.errors.append([__name__,'failed','%s %s - %s'%(os.path.basename(self.kw['executable']),' '.join(args),str(e))])
					logger.error('%s\t%s',*self.errors[-1][1:])
				else:
					logger.info('%s\t%d - %s %s',*['run',self.subproc['popen'].pid,os.path.basename(self.kw['executable']),' '.join(args)])
					for line in iter(self.subproc['popen'].stderr.readline, ''):
						if not line.strip(): continue
						# note: progress dots "." generated per "distinct source phrase"
						try:
							logger.info('%s\t%d, %s',*[os.path.basename(self.kw['executable']),self.subproc['popen'].pid,line.rstrip()])
							if line.split()[0] == 'processing':
								sys.stderr.write('processing ptree for [%s]\n'%(os.path.basename(self.model_dir)))
							else:
								sys.stderr.write('%s\n'%(line.replace('distinct source phrases:','\ndistinct source phrases:').replace('distinct first words of source phrases:','\ndistinct first words of source phrases:').replace('number of phrase pairs (line count):','\nnumber of phrase pairs (line count):').rstrip().encode(self.encoding)))
							sys.stderr.flush()
						except:
							pass

				finally:
					if os.path.exists(xsorted):
						logger.info('%s\t%s',*['delete',xsorted])
						os.unlink(xsorted)
					shutil.rmtree(self.tmpdir)

					while self.subproc['popen'].poll() is None:
						time.sleep(0.2)

			if self.subproc['popen'].returncode:
				self.errors.append([__name__,'failed','processPhraseTable - return code: %s'%(self.subproc['popen'].returncode)])
				logger.error('%s\t%s',*self.errors[-1][1:])
				break
			else:
				# update infodata and save
				infodata = cf.updatefingerprint([],self.outputs[k],False,infodata)
				if 'errors' in infodata:
					# report/return self.errors
					for error in infodata['errors']:
						self.errors.append([__name__,'error',error.strip()])
						logger.error('%s\t%s',*self.errors[-1][1:])
					break
				else:
					logger.info('%s\t%s',*['success','processPhraseTable'])
					sys.stderr.write('Success - processPhraseTable\n')

		sys.stderr.flush()
		lock.release()

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import filter\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0]
	)

licensetxt=u'''DoMY™ CE v2.5
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stdout.write(usage().encode('utf8')+'\n')
