#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#2.0.264 - version update

import os
import sys
import codecs
import logging
import common as cf

logger = logging.getLogger('.'.join([unicode(os.path.splitext(os.path.basename(sys.argv[0]))[0]),'manager','filtergraph',__name__]))
skipclose = True

class filter(object):

	cfg = {
		'lmbuildname': None,
		'lmroot': 'monotext',
		'remove1gram': False,
		'removedupes': False,
		'forceinfoupdate': '%(forceinfoupdate)s',
		'version': '2.0.264',
		}
	lmbuildname = None
	lmroot = 'monotext'
	filecount = 0
	linecount = 0
	skip = {}
	remove1gram = False
	removedupes = False
	forceinfoupdate = False
	isopen = False
	p = object
	errors = []

	def open(self,parent,cfg):
		self.lmbuildname = cfg['lmbuildname']
		self.lmroot = cfg['lmroot'] if cfg['lmroot'] and not cfg['lmroot'].startswith('%(') and not cfg['lmroot'].endswith(')s') else self.lmroot
		self.remove1gram = cfg['remove1gram']
		self.removedupes = cfg['removedupes']
		self.forceinfoupdate = self.forceinfoupdate if not cfg['forceinfoupdate'] or (cfg['forceinfoupdate'].startswith('%(') and cfg['forceinfoupdate'].endswith(')s')) else cf.zbool(cfg['forceinfoupdate'])
		self.linecount = 0
		self.filecount = 0
		self.tgttokens = 0

		tgt = os.sep.join([self.p.rootfolder,'BUILDS','lm',self.lmbuildname,os.extsep.join([self.lmroot,__name__.split(',')[1]])])

		try:
			os.makedirs(os.path.dirname(tgt))
		except OSError,e:
			if not e.errno == 17:
				logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
				raise OSError(e)

		if os.path.exists(tgt): os.unlink(tgt)
		if os.path.exists('%s.manifest'%(tgt)): os.unlink('%s.manifest'%(tgt))
		if os.path.exists(os.sep.join([os.path.dirname(tgt),'info'])): os.unlink(os.sep.join([os.path.dirname(tgt),'info']))

		self.e = codecs.open(tgt,'a','utf8')
		self.manifest = []
		self.limit = 1 if self.remove1gram else 0

	def run(self,k):
		'''Consolidate data files to build LM training data'''

		global skipclose
		try:

			if tuple([__name__]+self.p.filelist) in self.skip: return -2
			self.skip[tuple([__name__]+self.p.filelist)] = None
			if not sum([len(line) for line in self.p.cfoutput[k]['tempbuff'] if line.replace('<s> ','').replace(' </s>','').strip()]): return

			# removes one-gram phrases
			if self.removedupes:
				self.e.write('%s\n'%('\n'.join(_f5([line for line in self.p.cfoutput[k]['tempbuff'] if len(line.replace('<s> ','').replace(' </s>','').strip().split()) > self.limit]))))
			else:
				self.e.write('%s\n'%('\n'.join([line for line in self.p.cfoutput[k]['tempbuff'] if len(line.replace('<s> ','').replace(' </s>','').strip().split()) > self.limit])))
			self.manifest.append('\t'.join(self.p.cfinput[k]['filelist']))

			self.linecount += len(self.p.cfoutput[k]['tempbuff'])
			self.filecount += 1
			self.tgttokens += sum([len(line.split()) for line in self.p.cfoutput[k]['tempbuff']])

			skipclose = False

		except KeyboardInterrupt:
			self.e.close()
			os.unlink(self.e.name)
			skipclose = True
			raise KeyboardInterrupt()

	def flush(self,k):
		return

	def close(self):
		'''close filter'''
		if skipclose: return

		self.e.close()

		with codecs.open('%s.manifest'%(self.e.name),'w','utf8') as f:
			f.write('%s: %s\n'%(self.lmbuildname,cf.timestamp()))
			f.write('file pairs: %s\n'%(self.filecount))
			f.write('phrase pairs: %s\n'%(self.linecount))
			f.write('%s tokens: %s\n'%(os.path.splitext(self.e.name)[1].strip('.'),self.tgttokens))
			f.write('%s\n'%('\n'.join(self.manifest)))

		if not os.path.getsize(self.e.name) or not os.path.getsize('%s.manifest'%(self.e.name)):
			os.unlink(self.e.name)
			os.unlink('%s.manifest'%(self.e.name))

def _f5(seq,idfun=None): 
	# order preserving dedupe
	if idfun is None:
		def idfun(x): return x
	seen = {}
	result = []
	for item in seq:
		marker = idfun(item)
		if marker in seen: continue
		seen[marker] = 1
		result.append(item)
	return result

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import filter\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0]
	)

licensetxt=u'''DoMY™ CE v2.0
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stdout.write(usage().encode('utf8')+'\r')
