#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#2.5.328 - bugfix catch broken pipes in open() hangs on error

import os
import sys
import subprocess
import common as cf
import logging


logger = logging.getLogger('.'.join([os.path.splitext(os.path.basename(sys.argv[0]))[0],'manager','filtergraph',__name__]))
skipclose = True


class filter(object):

	cfg = {
		'args': {},
		'args1': '-b',
		'args2': '-q',
		'args3': '-l',
		'args4': '',
		'encoding': 'utf8',
		'exeoptions': '-w',
		'kw': {
			'bufsize': 0,
			'executable': '4,perl',
			'stdin': 'PIPE',
			'stdout': 'PIPE',
			'stderr': 'PIPE',
			'preexec_fn': None,
			'close_fds': False,
			'shell': False,
			'cwd': None,
			'env': None,
			'universal_newlines': False,
			'startupinfo': None,
			'creationflags': 0,
			},
		'script': '4,detokenizer.perl',

		'inputfile': '',
		'outputfile': '',
		'lang': '',
		'headch': '',
		'tailch': '',

		'version': '2.5.310',
		}
	args = []
	encoding = 'utf8'
	kw = {}
	subproc = None
	script = '4,detokenizer.perl'
	inputfile = ''
	outputfile = ''
	headch = ' '
	tailch = ' '
	lang = __name__.split(',')[1].split('_')[0]
	isopen = False
	p = object
	errors = []

	def __del__(self):
		try:
			self.subproc.stdin.close()
			self.subproc.stdout.close()
			self.subproc.stderr.close()
		except:
			pass
		else:
			while self.subproc.poll() is None:
				self.subproc.terminate()

	def open(self,parent,cfg):
		self.args = dict(cfg['args'])
		self.kw = dict(cfg['kw'])
		if cfg['script']:
			self.script = self.p.decodepath('%s,%s'%(__name__.split(',')[1],cfg['script']),True)
			if not self.script:
				self.script = self.p.decodepath('%s,5,%s'%(__name__.split(',')[1],os.sep.join([os.path.dirname(os.path.dirname(os.path.realpath(sys.argv[0]))),'mosesdecoder','scripts','tokenizer','tokenizer.perl',])),True)
				if not self.script:
					self.errors.append([__name__,'missing',cfg['script']])
					logger.error('%s\t%s',*self.errors[-1][1:])
			self.args[0] = str(self.script)
			self.kw['cwd'] = str(os.path.dirname(os.path.realpath(self.script)))
		else:
			self.script = None
		# convert args dict to args list
		self.args = [str(self.args[k]) for k in sorted(self.args.keys())]
		if 'executable' in cfg['kw'] and cfg['kw']['executable']:
			self.kw['executable'] = self.p.decodepath('%s,%s'%(__name__.split(',')[1],cfg['kw']['executable']),True)
			if not self.kw['executable']:
				self.errors.append([__name__,'missing',cfg['kw']['executable']])
				logger.error('%s\t%s',*self.errors[-1][1:])
			self.args.insert(0,cfg['exeoptions'] if cfg['exeoptions'] else '')
		else:
			self.kw['executable'] = None
		self.kw['stdin'] = subprocess.PIPE if cfg['kw']['stdin'] == 'PIPE' else cfg['kw']['stdin']
		self.kw['stdout'] = subprocess.PIPE if cfg['kw']['stdout'] == 'PIPE' else cfg['kw']['stdout']
		self.kw['stderr'] = subprocess.PIPE if cfg['kw']['stderr'] == 'PIPE' else cfg['kw']['stderr']

		self.encoding = 'utf8' if 'utf8' in cfg['encoding'].lower().replace('-','') else cfg['encoding']
		self.lang = cfg['lang'] if cfg['lang'] else self.lang
		if self.lang == 'alt':
			self.errors.append([__name__,'invalid','[%s] lang= must be set when plugin is in [alt] toolchain. - %s'%(__name__,cfg['inputfile'])])
			logger.warn('%s\t%s',*self.errors[-1][1:])

		self.inputfile = cfg['inputfile'].replace('%(rootfolder)s',self.p.rootfolder) if cfg['inputfile'] else self.inputfile
		self.outputfile = cfg['outputfile'].replace('%(rootfolder)s',self.p.rootfolder) if cfg['outputfile'] else self.outputfile
		if (self.inputfile and not self.outputfile) or (not self.inputfile and self.outputfile):
			self.errors.append([__name__,'invalid','[%s] inputfile=%s without outputfile= value'%(__name__,cfg['inputfile'])])
			logger.warn('%s\t%s',*self.errors[-1][1:])

		self.headch = cfg['headch'] if cfg['headch'] else self.headch
		self.tailch = cfg['tailch'] if cfg['tailch'] else self.tailch

		# test and set detokenizer nonbreaking_prefixes
		prefixpath = os.sep.join([os.path.dirname(os.path.dirname(os.path.realpath(self.script))),'share','nonbreaking_prefixes',])
		if os.path.exists(prefixpath):
			if self.lang in [os.path.splitext(fname)[1].lstrip('.') for fname in os.listdir(prefixpath)]:
				self.args[-1] = self.lang
			elif self.inputfile and self.lang == 'alt':
				self.errors.append([__name__,'invalid','[%s] inputfile=%s without lang= value'%(__name__,cfg['inputfile'])])
				logger.warn('%s\t%s',*self.errors[-1][1:])
			else:
				self.args[-1] = 'en'
		else:
			self.errors.append([__name__,'missing',prefixpath])
			logger.warn('%s\t%s',*self.errors[-1][1:])

		if not self.errors:
			try:
				self.subproc = subprocess.Popen(self.args,**self.kw)
				self.subproc.stdin.write('\n')
				test = self.subproc.stdout.readline().rstrip('\r\n').decode(self.encoding)
				if not self.subproc.poll() is None: raise IOError('[Errno 32] Broken pipe')
			except Exception,e:
				self.errors.append([__name__,'failed','\"[%s]\" \"%s %s\" - %s'%(__name__,self.kw['executable'],' '.join(self.args),str(e))])
				logger.warn('%s\t%s',*self.errors[-1][1:])

	def run(self,k):
		global skipclose
		skipclose = not self.inputfile
		if self.inputfile: return

		for i,line in enumerate(self.p.cfoutput[k]['tempbuff']):
			line = line.rstrip('\r\n')
			head = line[len(self.headch)-1] if line and line[len(self.headch)-1] == self.headch else ''
			tail = line[-len(self.tailch)] if line and line[-len(self.tailch)] == self.tailch else ''
			self.subproc.stdin.write('%s\n'%(line.encode(self.encoding)))
			self.p.cfoutput[k]['tempbuff'][i] = '%s%s%s'%(head,self.subproc.stdout.readline().rstrip('\r\n').decode(self.encoding),tail)

	def flush(self,k):
		return

	def close(self):
		if skipclose:
			try:
				self.subproc.stdin.close()
				self.subproc.stdout.close()
				self.subproc.stderr.close()
			except:
				pass
			else:
				while self.subproc.poll() is None:
					self.subproc.terminate()
			return

		import codecs
		from progress_bar import ProgressBar

		if not os.path.exists(self.inputfile):
			self.errors.append([__name__,'missing','[%s] %s'%(__name__,self.inputfile)])
			logger.error('%s\t%s',*self.errors[-1][1:])
			return

		# make output folder
		try:
			os.makedirs(os.path.dirname(self.outputfile))
		except OSError,e:
			if not e.errno == 17:
				logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
				raise OSError(e)

		# open input and output files
		out = self.outputfile
		if out == self.inputfile:
			import tempfile
			fd,out = tempfile.mkstemp(suffix='.tmp', prefix='~', dir=self.p.tempdir)
			os.close(fd)
		try:
			o = codecs.open(out,'w',self.encoding)
			ttl_lines = cf.file_length(self.inputfile)
			i = codecs.open(self.inputfile,'r',self.encoding)
		except:
			raise RuntimeError('Failed to open [%s] input/output files'%(__name__))

		progress = ProgressBar()
		progress.reset(0,ttl_lines,72,mode='fixed',char='#')
		sys.stderr.write('[%s] tokenizing %s phrases\n'%(__name__,ttl_lines))
		sys.stderr.flush()

		try:
			# loop writes output line-by-line
			for line in i:
				line = line.rstrip('\r\n')
				head = line[len(self.headch)-1] if line and line[len(self.headch)-1] == self.headch else ''
				tail = line[-len(self.tailch)] if line and line[-len(self.tailch)] == self.tailch else ''
				self.subproc.stdin.write('%s\n'%(line.encode(self.encoding)))
				o.write('%s%s%s\x0a'%(head,self.subproc.stdout.readline().rstrip('\r\n').decode(self.encoding),tail))
				progress.increment_amount()
				sys.stderr.write(progress.bar)
				sys.stderr.flush()
			progress.update_amount(ttl_lines)
			sys.stderr.write(progress.bar)
			sys.stderr.write('\n')
			sys.stderr.flush()
			self.subproc.stdin.flush()
			# close input and output files
			i.close()
			o.close()

			try:
				self.subproc.stdin.close()
				self.subproc.stdout.close()
				self.subproc.stderr.close()
			except:
				pass
			else:
				while self.subproc.poll() is None:
					self.subproc.terminate()

			if not out == self.outputfile:
				import shutil
				shutil.move(out,self.outputfile)

		except KeyboardInterrupt:
			sys.stderr.write('\n')
			os.unlink(out)
			try:
				self.subproc.stdin.close()
				self.subproc.stdout.close()
				self.subproc.stderr.close()
			except:
				pass
			else:
				while self.subproc.poll() is None:
					self.subproc.terminate()
			raise KeyboardInterrupt()

		if not os.path.exists(self.outputfile):
			self.errors.append([__name__,'missing','[%s] %s'%(__name__,self.outputfile)])
			logger.error('%s\t%s',*self.errors[-1][1:])

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import filter\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0]
	)

licensetxt=u'''DoMY™ CE v2.5
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stdout.write(usage().encode('utf8')+'\n')
