#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#2.5.287 - removed file locking

import os
import sys
import common as cf
import logging
import codecs
import shutil
import tempfile
from progress_bar import ProgressBar

logger = logging.getLogger('.'.join([os.path.splitext(os.path.basename(sys.argv[0]))[0],'manager','filtergraph',__name__]))
skipclose = True

class filter(object):

	cfg = {
		'encoding': 'utf8',
		'inputfile': None,
		'outputfile': None,
		'extractfile': '',
		'extract-size': None,


		'version': '2.5.287',
		}
	encoding = 'utf8'
	inputfile = ''
	outputfile = ''
	extract_size = 0


	isopen = False
	p = object
	errors = []

	def open(self,parent,cfg):
		self.encoding = 'utf8' if 'utf8' in cfg['encoding'].lower().replace('-','') else cfg['encoding']

		if cfg['inputfile']:
			self.inputfile = cfg['inputfile'].replace('%(rootfolder)s',self.p.rootfolder)
		else:
			self.errors.append([__name__,'missing','[%s] inputfile= required'%(__name__)])
			logger.warn('%s\t%s',*self.errors[-1][1:])

		if cfg['outputfile']:
			self.outputfile = cfg['outputfile'].replace('%(rootfolder)s',self.p.rootfolder)
		else:
			self.errors.append([__name__,'missing','[%s] outputfile= required'%(__name__)])
			logger.warn('%s\t%s',*self.errors[-1][1:])

		self.extracts = {}
		for k in [k for k in cfg.keys() if k[:11] == 'extractfile' and (cf.isinteger(k[11:] or not k[11:]))]:
			self.extracts[cfg[k].replace('%(rootfolder)s',self.p.rootfolder)] = None
		if self.extracts:
			self.extracts = self.extracts.keys()

			if self.inputfile in self.extracts:
				self.errors.append([__name__,'invalid','[%s] inputfile= and extractfileN= must be different paths'%(__name__)])
				logger.warn('%s\t%s',*self.errors[-1][1:])

			if self.outputfile in self.extracts:
				self.errors.append([__name__,'invalid','[%s] outputfile= and extractfileN= must be different paths'%(__name__)])
				logger.warn('%s\t%s',*self.errors[-1][1:])

		else:
			self.errors.append([__name__,'missing','[%s] extractfileN= required'%(__name__)])
			logger.warn('%s\t%s',*self.errors[-1][1:])

		self.extract_size = int(cfg['extract-size']) if cf.isinteger(cfg['extract-size']) else self.extract_size
		if not self.extract_size:
			self.errors.append([__name__,'missing','[%s] extract-size=number required'%(__name__)])
			logger.warn('%s\t%s',*self.errors[-1][1:])




	def run(self,k):
		global skipclose
		skipclose = False

	def flush(self,k):
		return

	def close(self):
		if skipclose: return

		if not os.path.exists(self.inputfile):
			self.errors.append([__name__,'missing','[%s] %s'%(__name__,self.inputfile)])
			logger.error('%s\t%s',*self.errors[-1][1:])
			return 249

		# make output folders
		test = self.extracts if self.inputfile == self.outputfile and os.path.exists(self.outputfile) else [self.outputfile]+self.extracts
		for out in test:
			try:
				os.makedirs(os.path.dirname(out))
			except OSError,e:
				if not e.errno == 17:
					logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
					raise OSError(e)

		popsize = cf.file_length(self.inputfile)

		extractlists = cf.samplepopulations(popsize,self.extract_size,len(self.extracts))

		out = self.outputfile
		if out == self.inputfile:
			fd,out = tempfile.mkstemp(suffix='.tmp', prefix='~', dir=self.p.tempdir)
			os.close(fd)
		# open input and output files
		try:
			i = codecs.open(self.inputfile,'r',self.encoding)
			o = codecs.open(out,'w',self.encoding)
			extracts = {}
			for j,fname in enumerate(self.extracts):
				extracts[fname] = {
					'list': extractlists[j],
					'obj': codecs.open(fname,'w',self.encoding)
					}
		except:
			if os.path.exists(out): os.unlink(out)
			for j,fname in enumerate(self.extracts):
				if os.path.exists(fname): os.unlink(fname)
			raise RuntimeError('Failed to open [%s] input/output files'%(__name__))

		progress = ProgressBar()
		progress.reset(0,len(extractlists)*self.extract_size,72,mode='fixed',char='#')
		sys.stderr.write('[%s] extracting %s sets of %s phrases\n'%(__name__,len(self.extracts),self.extract_size))
		sys.stderr.flush()
		try:
			# loop writes output line-by-line
			cnt = 0
			for line in i:
				for fname in extracts:
					if cnt in extracts[fname]['list']:
						extracts[fname]['obj'].write(line)
						line = ''
						progress.increment_amount()
						sys.stderr.write(progress.bar)
						sys.stderr.flush()
				if line: o.write(line)
				cnt += 1
			sys.stderr.write('\n')
			sys.stderr.flush()
			# close input and output files
			i.close()
			o.close()
			for fname in extracts:
				extracts[fname]['obj'].close()

			# put temp files in place
			if not out == self.outputfile:
				shutil.move(out,self.outputfile)

		except KeyboardInterrupt:
			sys.stderr.write('\n')
			if os.path.exists(out): os.unlink(out)
			for j,fname in enumerate(self.extracts):
				if os.path.exists(fname): os.unlink(fname)
			raise KeyboardInterrupt()

		if not os.path.exists(self.outputfile):
			self.errors.append([__name__,'missing','[%s] %s'%(__name__,self.outputfile)])
			logger.error('%s\t%s',*self.errors[-1][1:])

		for j,fname in enumerate(self.extracts):
			if not os.path.exists(fname):
				self.errors.append([__name__,'missing','[%s] %s'%(__name__,fname)])
				logger.error('%s\t%s',*self.errors[-1][1:])

		sys.stderr.flush()

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import filter\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0]
	)

licensetxt=u'''DoMY™ CE v2.5
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stdout.write(usage().encode('utf8')+'\n')
