#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#2.5.319 - delete existing extracted files to prevent accidental reuse, extend
#          support to extract-size=0

import os
import sys
import common as cf
import logging
import codecs
import shutil
import tempfile
from progress_bar import ProgressBar

logger = logging.getLogger('.'.join([os.path.splitext(os.path.basename(sys.argv[0]))[0],'manager','filtergraph',__name__]))
skipclose = True

class filter(object):

	cfg = {
		'encoding': 'utf8',
		'f': '',
		'e': '',
		'inputroot': None,
		'outputroot': None,
		'extract-size': None,


		'version': '2.5.319',
		}
	encoding = 'utf8'
	inputs = []
	outputs = []
	inputs_corpus = {}
	outputs_corpus = {}
	outputs_extract = {}
	extract_size = 0


	isopen = False
	p = object
	errors = []

	def open(self,parent,cfg):
		self.encoding = 'utf8' if 'utf8' in cfg['encoding'].lower().replace('-','') else cfg['encoding']

		langs = {}
		for k in [k for k in cfg.keys() if k[:4] == 'lang' and (cf.isinteger(k[4:]) or not k[4:])]:
			langs[cfg[k]] = None
		langs = langs.keys()

		# Languages
		if cfg['e'] and not cfg['e'].startswith('%(') and not cfg['e'].endswith(')s') and not cfg['e'] in langs:
			langs.append(cfg['e'])
		if cfg['f'] and not cfg['f'].startswith('%(') and not cfg['f'].endswith(')s') and not cfg['f'] in langs:
			langs.append(cfg['f'])

		if not langs:
			self.errors.append([__name__,'missing','\"[%s]\" one or more langN= options must be set'%(__name__,)])
			logger.error('%s\t%s',*self.errors[-1][1:])

		if cfg['inputroot']:
			self.inputs_corpus = {}
			for lang in langs:
				self.inputs.append('%s.%s'%(cfg['inputroot'].replace('%(rootfolder)s',self.p.rootfolder),lang))
				self.inputs_corpus[lang] = '%s.%s'%(cfg['inputroot'].replace('%(rootfolder)s',self.p.rootfolder),lang)
		else:
			self.inputs.append(-1)
			self.errors.append([__name__,'missing','[%s] inputroot= required'%(__name__)])
			logger.warn('%s\t%s',*self.errors[-1][1:])

		if cfg['outputroot']:
			self.outputs_corpus = {}
			for lang in langs:
				if not '%s.%s'%(cfg['outputroot'].replace('%(rootfolder)s',self.p.rootfolder),lang) == '%s.%s'%(cfg['inputroot'].replace('%(rootfolder)s',self.p.rootfolder),lang):
					self.outputs.append('%s.%s'%(cfg['outputroot'].replace('%(rootfolder)s',self.p.rootfolder),lang))
				self.outputs_corpus[lang] = '%s.%s'%(cfg['outputroot'].replace('%(rootfolder)s',self.p.rootfolder),lang)
		else:
			self.outputs.append(0)
			self.errors.append([__name__,'missing','[%s] outputroot= required'%(__name__)])
			logger.warn('%s\t%s',*self.errors[-1][1:])

		for k in [k for k in cfg.keys() if k[:11] == 'extractroot' and (cf.isinteger(k[11:] or not k[11:]))]:
			self.outputs_extract[int(k[11:])] = {}
			for lang in langs:
				self.outputs.append('%s.%s'%(cfg[k].replace('%(rootfolder)s',self.p.rootfolder),lang))
				self.outputs_extract[int(k[11:])][lang] = '%s.%s'%(cfg[k].replace('%(rootfolder)s',self.p.rootfolder),lang)
				if os.path.exists(self.outputs[-1]): os.unlink(self.outputs[-1])

		if sum([self.outputs[2:].count(item) for item in self.inputs]):
			self.errors.append([__name__,'invalid','[%s] inputroot= and outputs-extractN= must be different paths'%(__name__)])
			logger.warn('%s\t%s',*self.errors[-1][1:])

		self.extract_size = int(cfg['extract-size']) if cf.isinteger(cfg['extract-size']) else self.extract_size
		if not self.extract_size > -1:
			self.errors.append([__name__,'missing','[%s] extract-size=number required'%(__name__)])
			logger.warn('%s\t%s',*self.errors[-1][1:])




	def run(self,k):
		global skipclose
		skipclose = False

	def flush(self,k):
		return

	def close(self):
		if skipclose: return

		# test input and output files
		infodata = cf.updatefingerprint(self.inputs,self.outputs,True)
		if 'bypass' in infodata and self.inputs[0] == self.outputs[1]:
			infodata = {'start':'%s'%(cf.timestamp())}
		elif 'bypass' in infodata:
			# output file(s) exists.
			logger.info('%s\t%s',*['skipping','outputs exist and verified'])
			sys.stderr.write('%s - Skipping. [%s] Outputs exist and verified\n'%(cf.timestamp(),__name__))
			return 249
		elif 'errors' in infodata:
			# report/return self.errors
			for error in infodata['errors']:
				self.errors.append([__name__,'error',error.strip()])
				logger.error('%s\t%s',*self.errors[-1][1:])
			return 249
		sys.stderr.flush()

		# make output folders
		for out in self.outputs:
			try:
				os.makedirs(os.path.dirname(out))
			except OSError,e:
				if not e.errno == 17:
					logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
					raise OSError(e)

		popsize = [cf.file_length(item) for item in self.inputs]
		if not sum([item == popsize[0] for item in popsize]) == len(popsize):
			self.errors.append([__name__,'error','input files have different numbers of lines'])
			logger.error('%s\t%s',*self.errors[-1][1:])
			return 249

		if self.extract_size > 0:
			extractlists = cf.samplepopulations(popsize[0],self.extract_size,len(self.outputs_extract))

			# open input and output files
			in_corpus = {}
			out_corpus = {}
			for lang in self.outputs_corpus:
				if self.outputs_corpus[lang] == self.inputs_corpus[lang]:
					fd,out_corpus[lang] = tempfile.mkstemp(suffix='.tmp', prefix='~', dir=self.p.tempdir)
					os.close(fd)
				else:
					out_corpus[lang] = self.outputs_corpus[lang]
			extract_corpus = {}

			try:
				for lang in self.inputs_corpus:
					in_corpus[lang] = codecs.open(self.inputs_corpus[lang],'r',self.encoding)
				for lang in out_corpus:
					out_corpus[lang] = codecs.open(out_corpus[lang],'w',self.encoding)
				for i,k in enumerate(sorted(self.outputs_extract.keys())):
					extract_corpus[k] = {'extractlists': extractlists[i]}
					for lang in self.outputs_extract[k]:
						extract_corpus[k][lang] = codecs.open(self.outputs_extract[k][lang],'w',self.encoding)
			except:
				for out in self.outputs:
					if os.path.exists(out): os.unlink(out)
				raise RuntimeError('Failed to open [%s] input/output files'%(__name__))

			progress = ProgressBar()
			progress.reset(0,len(in_corpus) * len(extractlists) * len(extractlists[0])+1,72,mode='fixed',char='#')
			sys.stderr.write('[%s] extracting %s sets of %s pairs\n'%(__name__,len(extractlists),len(extractlists[0])))
			sys.stderr.flush()
			try:
				# loop writes output line-by-line
				i = 0
				for lang in in_corpus:
					cnt = 0
					for line in in_corpus[lang]:
						for k in sorted(extract_corpus.keys()):
							if cnt in extract_corpus[k]['extractlists']:
								extract_corpus[k][lang].write('%s\n'%(line.rstrip()))
								line = ''
								progress.increment_amount()
								sys.stderr.write(progress.bar)
								sys.stderr.flush()
						if line: out_corpus[lang].write(line)
						cnt += 1
				sys.stderr.write('\nDone. Cleaning up.\n')
				sys.stderr.flush()
				# close input and output files
				for lang in in_corpus:
					in_corpus[lang].close()
				for lang in out_corpus:
					out_corpus[lang].close()
				for k in extract_corpus:
					for lang in [lang for lang in extract_corpus[k] if not lang == 'extractlists']:
						extract_corpus[k][lang].close()

				# put temp files in place
				for lang in out_corpus:
					if not out_corpus[lang] == self.outputs_corpus[lang]:
						shutil.move(out_corpus[lang].name,self.outputs_corpus[lang])

			except KeyboardInterrupt:
				sys.stderr.write('\n')
				for out in self.outputs:
					if os.path.exists(out): os.unlink(out)
				raise KeyboardInterrupt()

		# update infodata and save
		if self.errors:
			logger.info('%s\t%s',*['error','[%s]'%(__name__)])
			sys.stderr.write('Error - [%s]\n'%(__name__))
		else:
			infodata = cf.updatefingerprint(self.inputs,self.outputs,True,infodata)
			if 'errors' in infodata:
				# report/return self.errors
				for error in infodata['errors']:
					self.errors.append([__name__,'error',error.strip()])
					logger.error('%s\t%s',*self.errors[-1][1:])
			else:
				logger.info('%s\t%s',*['success','%s'%(__name__)])
				sys.stderr.write('Success - %s\n'%(__name__))

		sys.stderr.flush()

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import filter\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0]
	)

licensetxt=u'''DoMY™ CE v2.5
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stdout.write(usage().encode('utf8')+'\n')
