#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#2.5.321 - bugfix f=%(source)s for recaser (not necessary)

import os
import sys
import logging
import common as cf
from flock import flock

logger = logging.getLogger('.'.join([os.path.splitext(os.path.basename(sys.argv[0]))[0],'manager','filtergraph',__name__]))
skipclose = True

class filter(object):

	cfg = {
		'encoding': 'utf8',
		'recaser': '%(recaser)s',
		'forceinfoupdate': '%(forceinfoupdate)s',
		'hierarchical': '%(hierarchical)s',
		'usebinary': True,
		'lmlang': '',
		'f': '',
		'e': None,
		'tmbuildname': '',
		'alignertype': 'giza',
		'tmgrams': 7,
		'lmbuildname': None,
		'lmtype': 'irstlm',
		'lmgrams': 5,
		'lmfactor': '0',
		'lmstructure': 'BloomMap',
		'lmodel-files': '',
		'mertroot': '%(mertroot)s',
		'evalroot': '%(evalroot)s',
		'for-mert': False,
		'for-evaluation': False,
		'version': '2.5.321',
		}
	encoding = 'utf8'
	recaser = False
	forceinfoupdate = False
	hierarchical = False
	usebinary = 'True'
	lmlang = ''
	f = ''
	e = None
	tmbuildname = None
	alignertype = 'giza'
	tmgrams = 7
	lmbuildname = None
	lmtype = 'irstlm'
	lmgrams = 5
	lmfactor = '0'
	lmstructure = 'BloomMap'
	lmodel_files = []
	mertroot = 'no_mert'
	evalroot = 'no_eval'
	for_mert = False
	for_evaluation = False
	notrun = True
	isopen = False
	p = object
	errors = []

	def open(self,parent,cfg):

		self.encoding = 'utf8' if 'utf8' in cfg['encoding'].lower().replace('-','') else cfg['encoding']

		self.for_evaluation = cfg['for-evaluation']
		self.for_mert = cfg['for-mert']
		self.recaser = self.recaser if not cfg['recaser'] or (cfg['recaser'].startswith('%(') and cfg['recaser'].endswith(')s')) else cf.zbool(cfg['recaser'])
		self.forceinfoupdate = self.forceinfoupdate if cfg['forceinfoupdate'] == '%(forceinfoupdate)s' else cf.zbool(cfg['forceinfoupdate'])
		self.hierarchical = self.hierarchical if not cfg['hierarchical'] or (cfg['hierarchical'].startswith('%(') and cfg['hierarchical'].endswith(')s')) else cf.zbool(cfg['hierarchical'])
		self.evalroot = cfg['evalroot'] if cfg['evalroot'] and not cfg['evalroot'].startswith('%(') and not cfg['evalroot'].endswith(')s') else self.evalroot
		self.mertroot = cfg['mertroot'] if cfg['mertroot'] and not cfg['mertroot'].startswith('%(') and not cfg['mertroot'].endswith(')s') else self.mertroot
		self.usebinary = cfg['usebinary']

		# test required values
		if self.for_mert and self.mertroot == 'no_mert' and not self.recaser:
			self.errors.append([__name__,'missing','\"[%s]\" mertroot= but must be set'%(__name__,)])
			logger.error('%s\t%s',*self.errors[-1][1:])
		if self.for_evaluation and self.evalroot == 'no_eval' and not self.recaser:
			self.errors.append([__name__,'missing','\"[%s]\" evalroot= but must be set'%(__name__,)])
			logger.error('%s\t%s',*self.errors[-1][1:])

		self.e = cfg['e']
		if (not self.e or (self.e.startswith('%(') and self.e.endswith(')s'))):
			self.errors.append([__name__,'missing','\"[%s]\" e= must be set'%(__name__,)])
			logger.error('%s\t%s',*self.errors[-1][1:])
		self.f = cfg['f']
		if (not self.f or (self.f.startswith('%(') and self.f.endswith(')s'))) and not self.recaser:
			self.errors.append([__name__,'missing','\"[%s]\" f= must be set'%(__name__,)])
			logger.error('%s\t%s',*self.errors[-1][1:])

		# LM values
		self.lmbuildname = cfg['lmbuildname']
		if not self.lmbuildname or (self.lmbuildname.startswith('%(') and self.lmbuildname.endswith(')s')):
			self.errors.append([__name__,'missing','\"[%s]\" lmbuildname must be set'%(__name__,)])
			logger.warn('%s\t%s',*self.errors[-1][1:])
		self.lmtype = cfg['lmtype'].lower() if cfg['lmtype'] and not cfg['lmtype'].startswith('%(') and not cfg['lmtype'].endswith(')s') else self.lmtype.lower()
		if not self.lmtype in ['irstlm','randlm','srilm','irstlmken','srilmken',]:
			self.errors.append([__name__,'invalid','\"[%s]\" lmtype=%s. It must be set to \'irstlm\', \'randlm\', \'srilm\', \'irstlmken\',  or \'srilmken\''%(__name__,self.lmtype)])
			logger.error('%s\t%s',*self.errors[-1][1:])
		self.lmgrams = str(int(cfg['lmgrams'])) if cf.isinteger(cfg['lmgrams']) else self.lmgrams
		self.lmstructure = cfg['lmstructure'] if cfg['lmstructure'] in ['BloomMap','LogFreqBloomFilter'] else self.lmstructure
		self.lmfactor = str(int(cfg['lmfactor'])) if cf.isinteger(cfg['lmfactor']) else  self.lmfactor

		# TM values
		if (not cfg['tmbuildname'] or cfg['tmbuildname'].startswith('%(') or cfg['tmbuildname'].endswith(')s')) and not self.recaser:
			self.errors.append([__name__,'missing','\"[%s]\" tmbuildname= must be set'%(__name__,)])
			logger.warn('%s\t%s',*self.errors[-1][1:])
		self.tmbuildname = cfg['tmbuildname']
		self.alignertype = cfg['alignertype'].lower() if cfg['alignertype'] and not cfg['alignertype'].startswith('%(') and not cfg['alignertype'].endswith(')s') else self.alignertype
		if 'giza' in self.alignertype:
			self.alignertype = 'giza'
		elif 'recaser' in self.alignertype:
			self.alignertype = 'recaser'
		elif 'berk' in self.alignertype:
			self.alignertype = 'berk'
		else:
			self.errors.append([__name__,'invalid','\"[%s]\" alignertype=%s, but must be set to \'giza\', \'berk\', or \'recaser\''%(__name__,cfg['alignertype'])])
			logger.error('%s\t%s',*self.errors[-1][1:])
		self.tmgrams = str(int(cfg['tmgrams'])) if cf.isinteger(cfg['tmgrams']) else self.tmgrams

		# Language model
		if cfg['lmodel-files']:
			self.lmodel_files,tmperrors = cf.parse_lmodel_files(cfg['lmodel-files'],__name__,self.lmodel_files)
			self.errors.extend(tmperrors)
			for error in tmperrors:
				logger.error('%s\t%s',*error[1:])
		lmlang = cfg['lmlang'] if cfg['lmlang'] and not cfg['lmlang'].startswith('%(') and not cfg['lmlang'].endswith(')s') else self.e
		if self.recaser:
			lmfolder = '%s-t=%s-l=%s-T=%s-n=%s-a=%s-g=%s'%('recaser',lmlang,self.lmbuildname,self.lmtype,self.lmgrams,self.alignertype,self.tmgrams)
			kind = 'recasers'
		else:
			lmfolder = '%s-t=%s-l=%s-T=%s-n=%s'%('lm',lmlang,self.lmbuildname,self.lmtype,self.lmgrams)
			kind = 'lms'
		lm_dir = os.sep.join([self.p.rootfolder,'ENGINES',kind,lmfolder,])
		if self.lmtype == 'irstlm':
			basename = os.extsep.join([self.lmtype,lmlang,'mm'])
		elif self.lmtype == 'randlm':
			basename = os.extsep.join([self.lmtype,lmlang,self.lmstructure])
		elif self.lmtype == 'srilm':
			basename = os.extsep.join([self.lmtype,lmlang,'gz'])
		elif self.lmtype == 'irstlmken':
			basename = os.extsep.join([self.lmtype,lmlang,'mm'])
		elif self.lmtype == 'srilmken':
			basename = os.extsep.join([self.lmtype,lmlang,'mm'])
		lmodel = '%s %s %s %s'%(cf.lmbytype[self.lmtype] if self.lmtype in cf.lmbytype else '1', self.lmfactor,self.lmgrams,os.sep.join([lm_dir,basename]))
		if not lmodel in self.lmodel_files: self.lmodel_files.append(lmodel)

		try:
			os.makedirs(cf.getconfigpath())
		except OSError,e:
			if not e.errno == 17:
				logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
				raise OSError(e)

		# moses config file
		if self.recaser:
			recaserfolder = 'recaser-t=%s-l=%s-T=%s-n=%s-a=%s-g=%s'%(self.e,self.lmbuildname,self.lmtype,self.lmgrams,self.alignertype,self.tmgrams,)
			recaserconf = 'recaser-%s-t=%s-l=%s-T=%s-n=%s-a=%s-g=%s.conf'%(self.mertroot,self.e,self.lmbuildname,self.lmtype,self.lmgrams,self.alignertype,self.tmgrams)
			mert_recaser_input  = os.sep.join([self.p.rootfolder,'ENGINES','recasers',recaserfolder,'moses.ini',])
			mert_recaser_output = os.sep.join([self.p.rootfolder,'ENGINES','recasers',recaserfolder,'working','run0.moses.ini',])
			eval_recaser_input  = os.sep.join([self.p.rootfolder,'ENGINES','recasers',recaserfolder,'moses.ini',])
			eval_recaser_output = os.sep.join([self.p.rootfolder,'ENGINES','recasers',recaserfolder,'moses-%s.ini'%(self.mertroot),])
			norm_recaser_input  = os.sep.join([self.p.rootfolder,'ENGINES','recasers',recaserfolder,'moses-%s.ini'%(self.mertroot),])
			norm_recaser_output = os.sep.join([cf.getconfigpath(),recaserconf,])
			if self.for_mert:
				self.inputfile = mert_recaser_input
				self.outputfile = mert_recaser_output
			elif self.for_evaluation:
				self.inputfile = eval_recaser_input
				self.outputfile = eval_recaser_output
			else:
				self.inputfile = norm_recaser_input
				self.outputfile = norm_recaser_output
		else:
			tablesfolder = '%s-s=%s-t=%s-p=%s-a=%s-g=%s'%('rule' if self.hierarchical else 'phrase',self.f,self.e,self.tmbuildname,self.alignertype,self.tmgrams,)
			mertsfolder = 'mert-%s-s=%s-t=%s-p=%s-a=%s-g=%s-l=%s-T=%s-n=%s'%(self.mertroot,self.f,self.e,self.tmbuildname,self.alignertype,self.tmgrams,self.lmbuildname,self.lmtype,self.lmgrams,)
			evalfolder = 'eval-%s-s=%s-t=%s-p=%s-a=%s-g=%s-l=%s-T=%s-n=%s'%(self.mertroot,self.f,self.e,self.tmbuildname,self.alignertype,self.tmgrams,self.lmbuildname,self.lmtype,self.lmgrams,)
			modelconf = 'domy-%s-s=%s-t=%s-p=%s-a=%s-g=%s-l=%s-T=%s-n=%s.conf'%(self.mertroot,self.f,self.e,self.tmbuildname,self.alignertype,self.tmgrams,self.lmbuildname,self.lmtype,self.lmgrams)
			mert_model_input    = os.sep.join([self.p.rootfolder,'ENGINES','tables',tablesfolder,'moses.ini',])
			mert_model_output   = os.sep.join([self.p.rootfolder,'TRAININGS','merts',mertsfolder,'run0.moses.ini',])
			eval_model_input    = os.sep.join([self.p.rootfolder,'TRAININGS','merts',mertsfolder,'moses-%s.ini'%(self.mertroot),])
			eval_model_output   = os.sep.join([self.p.rootfolder,'ENGINES','evals',evalfolder,'moses-%s.ini'%(self.mertroot),])
			norm_model_input    = os.sep.join([self.p.rootfolder,'ENGINES','evals',evalfolder,'moses-%s.ini'%(self.mertroot),])
			norm_model_output   = os.sep.join([cf.getconfigpath(),modelconf,])
			if self.for_mert:
				self.inputfile = mert_model_input
				self.outputfile = mert_model_output
			elif self.for_evaluation:
				self.inputfile = eval_model_input
				self.outputfile = eval_model_output
			else:
				self.inputfile = norm_model_input
				self.outputfile = norm_model_output

	def run(self,k):
		global skipclose
		skipclose = False

	def flush(self,k):
		return

	def close(self):
		if skipclose: return

		if os.name == 'nt':
			lockfile = os.sep.join([os.environ['ALLUSERSPROFILE'],'Application Data','domy','train.lock'])
		else:
			lockfile = os.sep.join(['','var','tmp','domy','train.lock'])
		if not os.path.exists(os.path.dirname(lockfile)): os.makedirs(os.path.dirname(lockfile))
		lock = flock(lockfile,False).acquire()
		if not lock or not lock.ownlock():
			self.errors.append([__name__,'failed','train lock, another training session is running'])
			logger.error('%s\t%s',*self.errors[-1][1:])
			return
		with open(lockfile) as f:
			lockcontent = f.readline().rstrip()

		if self.recaser or self.for_mert:
			if not os.path.exists(self.inputfile):
				self.errors.append([__name__,'missing','[%s] %s'%(__name__,self.inputfile)])
				logger.error('%s\t%s',*self.errors[-1][1:])
				return
		else:
			infodata = cf.updatefingerprint(self.inputfile,self.outputfile,self.forceinfoupdate)
			if 'bypass' in infodata:
				# output file(s) exists.
				logger.info('%s\t%s',*['skipping','outputs exist and verified'])
				sys.stderr.write('%s - Skipping. [%s] Outputs exist and verified\n'%(cf.timestamp(),__name__))
				lock.release()
				return
			elif 'errors' in infodata:
				# report/return self.errors
				for error in infodata['errors']:
					self.errors.append([__name__,'error',error.strip()])
					logger.error('%s\t%s',*self.errors[-1][1:])
				lock.release()
				return
			sys.stderr.flush()

		with open(self.inputfile,'r') as t:
			contents = [line.split(u'¶') for line in u'¶'.join(t.read().splitlines()).replace(u'¶#',u'\n#').replace(u'¶[',u'\n[').splitlines()]

		ttable_files = []
		distortion_files = []
		generation_files = []
		lmodel_files = []
		ttables = []
		distortions = []
		generations = []
		lmodels = []

		# check each new moses config settings & validate components exist
		for i,content in enumerate(contents):

			tmperrors = []
			if line[0].startswith('#'):
				continue

			elif content[0] == '[ttable-file]':
				# test if data formatted correctly
				ttable_files,tmperrors = cf.parse_ttable_files(content[1:],__name__,self.usebinary,[],self.inputfile)
				self.errors.extend(tmperrors)
				for error in tmperrors:
					logger.error('%s\t%s',*error[1:])
				# test if files exists
				if not tmperrors:
					ok,ttables = cf.test_ttable_files(ttable_files)
					if ok:
						contents[i] = content[:1] + ttable_files + [u'']
					else:
						self.errors.append([__name__,'invalid','\"[%s]\" moses config=%s invalid [ttable-file]'%(__name__,self.inputfile)])
						logger.error('%s\t%s',*self.errors[-1][1:])

			elif content[0] == '[distortion-file]':
				distortion_files,tmperrors = cf.parse_distortion_files(content[1:],__name__,self.usebinary,[],self.inputfile)
				self.errors.extend(tmperrors)
				for error in tmperrors:
					logger.error('%s\t%s',*error[1:])
				if not tmperrors:
					ok,distortions = cf.test_distortion_files(distortion_files)
					if ok:
						contents[i] = content[:1] + distortion_files + [u'']
					else:
						self.errors.append([__name__,'invalid','\"[%s]\" moses config=%s invalid [distortion-file]'%(__name__,self.inputfile)])
						logger.error('%s\t%s',*self.errors[-1][1:])

			elif content[0] == '[generation-file]':
				generation_files,tmperrors = cf.parse_generation_files(content[1:],__name__,[],self.inputfile)
				self.errors.extend(tmperrors)
				for error in tmperrors:
					logger.error('%s\t%s',*error[1:])
				if not tmperrors:
					ok,generations = cf.test_generation_files(generation_files)
					if ok:
						contents[i] = content[:1] + generation_files + [u'']
					else:
						self.errors.append([__name__,'invalid','\"[%s]\" moses config=%s invalid [generation-file]'%(__name__,self.inputfile)])
						logger.error('%s\t%s',*self.errors[-1][1:])

			elif content[0] == '[lmodel-file]':
				self.lmodel_files,tmperrors = cf.parse_lmodel_files(content[1:],__name__,self.lmodel_files,self.inputfile)
				self.errors.extend(tmperrors)
				for error in tmperrors:
					logger.error('%s\t%s',*error[1:])
				if not tmperrors:
					ok,lmodels = cf.test_lmodel_files(self.lmodel_files)
					if ok:
						contents[i] = content[:1] + self.lmodel_files + [u'']
					else:
						self.errors.append([__name__,'missing','\"[%s]\" moses config=%s invalid [lmodel-file] and missing lmodel-file= in graph\'s config.ini'%(__name__,self.inputfile)])
						logger.error('%s\t%s',*self.errors[-1][1:])

			elif line[0] == '[weight-d]':			# distortion (reordering) weight
				contents[i] = content
			elif line[0] == '[weight-l]':			# language model weights
				contents[i] = content
			elif line[0] == '[weight-t]':			# translation model weights
				contents[i] = content
			elif line[0] == '[weight-w]':			# word penalty
				contents[i] = content
			elif line[0] == '[weight-generation]':	# generation models
				contents[i] = content
			elif line[0] == '[weight-lex]':			# weight-generation section
				contents[i] = content
			elif line[0] == '[weight-i]':
				contents[i] = content
			elif line[0] == '[distortion-limit]':	# distortion-limit
				contents[i] = content

			self.errors.extend(tmperrors)
			contents[i] = '\n'.join(contents[i])

		# distortion_files and generation_files are mutually exclusive, or not either
		if distortion_files and generation_files:
			self.errors.append([__name__,'invalid','\"moses config requires either [distortion-file] or [generation-file] not both in %s'%(self.inputfile)])
			logger.error('%s\t%s',*self.errors[-1][1:])

		if self.errors: return

		logger.info('%s\t%s',*['start','make-config'])
		sys.stderr.write('\nStart -%s make-config [%s]\n'%(' (recaser)' if self.recaser else '',__name__))

		# make output folder
		try:
			os.makedirs(os.path.dirname(self.outputfile))
		except OSError,e:

			if not e.errno == 17:
				logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
				raise OSError(e)

		# moses config validated and components exist. create new moses config file
		try:
			with open(self.outputfile,'w') as t:
				t.write('%s\x0a'%('\x0a'.join(contents)))
		except:
			if os.path.exists(self.outputfile): os.unlink(self.outputfile)

		if self.recaser or self.for_mert:
			if not os.path.exists(self.outputfile):
				self.errors.append([__name__,'fail','save file failed: %s'%(self.outputfile)])
				logger.error('%s\t%s',*self.errors[-1][1:])
			if not self.errors:
				logger.info('%s\t%s',*['success','make-config'])
				sys.stderr.write('Success - make-config\n')
		elif self.errors:
			self.errors.append([__name__,'fail',__name__])
			logger.error('%s\t%s',*self.errors[-1][1:])
		else:
			infodata = cf.updatefingerprint([],self.outputfile,True,infodata)
			if 'errors' in infodata:
				# report/return self.errors
				sys.stderr.write('\n')
				for error in infodata['errors']:
					self.errors.append([__name__,'error',error.strip()])
					logger.error('%s\t%s',*self.errors[-1][1:])
			else:
				logger.info('%s\t%s',*['success','make-config'])
				sys.stderr.write('Success - make-config\n')

		sys.stderr.flush()
		lock.release()

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import filter\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0]
	)

licensetxt=u'''DoMY™ CE v2.5
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stderr.write(usage().encode('utf8')+'\n')
