#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#2.5.288 - change from .sgm to .xml extensions

import os
import sys
import subprocess
import shutil
import logging
import common as cf
from flock import flock

logger = logging.getLogger('.'.join([os.path.splitext(os.path.basename(sys.argv[0]))[0],'manager','filtergraph',__name__]))
skipclose = True

class filter(object):

	cfg = {
		'encoding': 'utf8',
		'makemteval': '4,makemteval.py',
		'version': '2.5.288',

		'forceinfoupdate': '%(forceinfoupdate)s',
		'f': None,
		'e': None,
		'tmbuildname': None,
		'evalroot': '%(evalroot)s',
		'docid': '',
		'genre': '',
		'language': '',
		'refid': '',
		'setid': '',
		'sysbleu': 0,
		'sysid': '',
		}
	encoding = 'utf8'
	forceinfoupdate = False
	f = None
	e = None
	tmbuildname = None
	evalroot = 'eval'
	makemteval = '4,makemteval.py'
	mtevalcfg = {
		'docid': 'DocID',
		'genre': 'Genre',
		'language': '--',
		'refid': 'RefID',
		'setid': 'SetID',
		'sysbleu': 1.0000,
		'sysid': 'SysID',
		}
	isopen = False
	p = object
	errors = []

	def open(self,parent,cfg):
		# create makemteval object
		self.makemteval = cfg['makemteval'] if cfg['makemteval'] else self.makemteval
		self.makemteval = self.p.decodepath(self.makemteval,True)
		if not self.makemteval:
			self.errors.append([__name__,'missing','\"[%s]\" \"makemteval=%s\"'%(__name__,self.makemteval)])
			logger.error('%s\t%s',*self.errors[-1][1:])
		try:
			import imp
			fp, pathname, description = imp.find_module(os.path.splitext(os.path.basename(self.makemteval))[0],[os.path.dirname(self.makemteval)])
			self.mksgm = imp.load_module(os.path.splitext(os.path.basename(self.makemteval))[0], fp, pathname, description)
		except:
			self.errors.append([__name__,'failed','import \"[%s]\" \"buildname=%s\"'%(__name__,self.makemteval)])
			logger.error('%s\t%s',*self.errors[-1][1:])
		if fp: fp.close()

		self.encoding = 'utf8' if 'utf8' in cfg['encoding'].lower().replace('-','') else cfg['encoding']

		# test required values
		if not cfg['f'] or cfg['f'] == '%(f)s' or cfg['f'] == '%(srclang)s':
			self.errors.append([__name__,'missing','\"[%s]\" f= must be set'%(__name__,)])
			logger.error('%s\t%s',*self.errors[-1][1:])
		if not cfg['e'] or cfg['e'] == '%(e)s' or cfg['e'] == '%(tgtlang)s':
			self.errors.append([__name__,'missing','\"[%s]\" e= must be set'%(__name__,)])
			logger.error('%s\t%s',*self.errors[-1][1:])
		self.f = cfg['f']
		self.e = cfg['e']

		# TM values
		if not cfg['tmbuildname'] or cfg['tmbuildname'] == '%(tmbuildname)s':
			self.errors.append([__name__,'missing','\"[%s]\" tmbuildname= must be set'%(__name__,)])
			logger.warn('%s\t%s',*self.errors[-1][1:])
		self.tmbuildname = cfg['tmbuildname']

		# other values
		self.forceinfoupdate = self.forceinfoupdate if cfg['forceinfoupdate'] == '%(forceinfoupdate)s' else cf.zbool(cfg['forceinfoupdate'])
		self.evalroot = cfg['evalroot'] if cfg['evalroot'] and not cfg['evalroot'].startswith('%(') and not cfg['evalroot'].endswith(')s') else self.evalroot

		self.mtevalcfg['docid'] = cfg['docid'] if cfg['docid'] else self.mtevalcfg['docid']
		self.mtevalcfg['refid'] = cfg['refid'] if cfg['refid'] else self.mtevalcfg['refid']
		self.mtevalcfg['setid'] = cfg['setid'] if cfg['setid'] else self.mtevalcfg['setid']

		self.mtevalcfg['genre'] = cfg['genre'] if cfg['genre'] else self.mtevalcfg['genre']
		self.mtevalcfg['language'] = cfg['language'] if  cfg['language'] else self.mtevalcfg['language']
		self.mtevalcfg['sysbleu'] = cfg['sysbleu'] if cfg['sysbleu'] else self.mtevalcfg['sysbleu']
		self.mtevalcfg['sysid'] = cfg['sysid'] if cfg['sysid'] else self.mtevalcfg['sysid']

		# important: the first item in self.inputs list must be the source language
		self.inputs = [
			os.sep.join([self.p.rootfolder,'BUILDS','tm',self.tmbuildname,os.extsep.join([self.evalroot,self.f])]),
			os.sep.join([self.p.rootfolder,'BUILDS','tm',self.tmbuildname,os.extsep.join([self.evalroot,self.e])]),
			]

		# important: the 'basename' of all output file must conform to "evalroot.settype.lang.ext"
		self.outputs = [
			os.sep.join([self.p.rootfolder,'BUILDS','tm',self.tmbuildname,os.extsep.join([self.evalroot,'ref',self.f,'xml'])]),
			os.sep.join([self.p.rootfolder,'BUILDS','tm',self.tmbuildname,os.extsep.join([self.evalroot,'src',self.f,'xml'])]),
			os.sep.join([self.p.rootfolder,'BUILDS','tm',self.tmbuildname,os.extsep.join([self.evalroot,'tst',self.f,'xml'])]),
			os.sep.join([self.p.rootfolder,'BUILDS','tm',self.tmbuildname,os.extsep.join([self.evalroot,'ref',self.e,'xml'])]),
			os.sep.join([self.p.rootfolder,'BUILDS','tm',self.tmbuildname,os.extsep.join([self.evalroot,'src',self.e,'xml'])]),
			os.sep.join([self.p.rootfolder,'BUILDS','tm',self.tmbuildname,os.extsep.join([self.evalroot,'tst',self.e,'xml'])]),
			]

	def run(self,k):
		global skipclose
		skipclose = False

	def flush(self,k):
		return

	def close(self):
		if skipclose: return

		if os.name == 'nt':
			lockfile = os.sep.join([os.environ['ALLUSERSPROFILE'],'Application Data','domy','train.lock'])
		else:
			lockfile = os.sep.join(['','var','tmp','domy','train.lock'])
		if not os.path.exists(os.path.dirname(lockfile)): os.makedirs(os.path.dirname(lockfile))
		lock = flock(lockfile,False).acquire()
		if not lock or not lock.ownlock():
			self.errors.append([__name__,'failed','train lock, another training session is running'])
			logger.error('%s\t%s',*self.errors[-1][1:])
			return
		with open(lockfile) as f:
			lockcontent = f.readline().rstrip()

		# test input and output files
		infodata = cf.updatefingerprint(self.inputs,self.outputs,self.forceinfoupdate)
		if 'bypass' in infodata:
			# output file(s) exists.
			logger.info('%s\t%s',*['skipping','outputs exist and verified'])
			sys.stderr.write('%s - Skipping. [%s] Outputs exist and verified\n'%(cf.timestamp(),__name__))
			lock.release()
			return
		elif 'errors' in infodata:
			# report/return self.errors
			for error in infodata['errors']:
				self.errors.append([__name__,'error',error.strip()])
				logger.error('%s\t%s',*self.errors[-1][1:])
			lock.release()
			return
		sys.stderr.flush()

		logger.info('%s\t%s',*['start','make-mteval'])
		sys.stderr.write('\nStart - make-mteval [%s]\n'%(__name__))

		# make output folder
		for out in self.outputs:
			try:
				os.makedirs(os.path.dirname(out))
			except OSError,e:
				if not e.errno == 17:
					logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
					raise OSError(e)

		mksgm = self.mksgm.makemteval()
		for out in self.outputs:
			evalroot,settype,lang,ext = os.path.basename(out).split(os.extsep)
			self.mtevalcfg['filein'] = self.inputs[0] if lang == self.f else self.inputs[1]
			self.mtevalcfg['fileout'] = out
			self.mtevalcfg['settype'] = '%sset'%(settype)
			if settype == 'src':
				self.mtevalcfg['srclang'] = self.f if lang == self.f else self.e
				self.mtevalcfg['tstlang'] = self.e if lang == self.f else self.f
			else:
				self.mtevalcfg['srclang'] = self.e if lang == self.f else self.f
				self.mtevalcfg['tstlang'] = self.f if lang == self.f else self.e
			mksgm.writesgm(self.mtevalcfg)
			if not os.path.exists(out):
				self.errors.append([__name__,'missing','%s'%(out)])
				logger.error('%s\t%s',*self.errors[-1][1:])

		del(mksgm)

		# update infodata and save
		if self.errors:

			logger.info('%s\t%s',*['error','make-mteval - failed to create %s'%(self.outputs[0])])
			sys.stderr.write('Error - make-mteval - failed to create %s\n'%(self.outputs[0]))
		else:
			infodata = cf.updatefingerprint([],self.outputs,True,infodata)
			if 'errors' in infodata:
				# report/return self.errors
				for error in infodata['errors']:
					self.errors.append([__name__,'error',error.strip()])
					logger.error('%s\t%s',*self.errors[-1][1:])
			else:
				logger.info('%s\t%s',*['success','make-mteval'])
				sys.stderr.write('Success - make-mteval\n')

		sys.stderr.flush()
		lock.release()

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import filter\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0]
	)

licensetxt=u'''DoMY™ CE v2.5
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stderr.write(usage().encode('utf8')+'\n')
