#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#2.5.324 - version update

import os
import sys
import subprocess
import logging
import time
import common as cf
from progress_bar import ProgressBar
from threading import Thread

logger = logging.getLogger('.'.join([os.path.splitext(os.path.basename(sys.argv[0]))[0],'manager','filtergraph',__name__]))
endsignal = '01AD2372-0AAE-430E-BE15-7C490AFD7412'

class filter(object):

	cfg = {
		'args': {},
		'encoding': 'utf8',
		'engine': '',
		'exeoptions': '',
		'kw': {
			'bufsize': 0,
			'executable': '4,moses',
			'stdin': 'PIPE',
			'stdout': 'PIPE',
			'stderr': 'PIPE',
			'preexec_fn': None,
			'close_fds': False,
			'shell': False,
			'cwd': None,
			'env': None,
			'universal_newlines': False,
			'startupinfo': None,
			'creationflags': 0,
			},
		'script': '',
		'version': '2.5.324',
		}
	args = []
	encoding = 'utf8'
	kw = {}
	script = ''
	subproc = {}
	isopen = False
	p = object
	errors = []

	def open(self,parent,cfg):
		self.args = dict(cfg['args'])
		self.kw = dict(cfg['kw'])
		if cfg['script']:
			self.script = self.p.decodepath(','.join([__name__.split(',')[1]]+cfg['script'].split(',')),True)
			if not self.script:
				self.errors.append([__name__,'missing',self.script])
				logger.error('%s\t%s',*self.errors[-1][1:])
			self.args[0] = self.script
			self.kw['cwd'] = os.path.dirname(self.script)
		else:
			self.script = None
		# convert args dict to args list
		self.args = [str(self.args[k]) for k in sorted(self.args.keys())]
		if cfg['kw']['executable']:
			self.kw['executable'] = self.p.decodepath(','.join([__name__.split(',')[1]]+cfg['kw']['executable'].split(',')),True)
			if not self.kw['executable']:
				self.errors.append([__name__,'missing',cfg['kw']['executable']])
				logger.error('%s\t%s',*self.errors[-1][1:])
			self.args.insert(0,cfg['exeoptions'] if cfg['exeoptions'] else '')
		else:
			self.kw['executable'] = None
		self.kw['stdin'] = subprocess.PIPE if cfg['kw']['stdin'] == 'PIPE' else cfg['kw']['stdin']
		self.kw['stdout'] = subprocess.PIPE if cfg['kw']['stdout'] == 'PIPE' else cfg['kw']['stdout']
		self.kw['stderr'] = subprocess.PIPE if cfg['kw']['stderr'] == 'PIPE' else cfg['kw']['stderr']
		self.encoding = 'utf8' if 'utf8' in cfg['encoding'].lower().replace('-','') else cfg['encoding']

		self.progress = ProgressBar()

		# configure moses engine
		engines = {}
		if cfg['engine']:

				if os.path.exists(cfg['engine']):
					engines[0] = [__name__.split(',')[1],cfg['engine']]
				else:
					self.errors.append([__name__,'missing','\"[%s]\" engine=%s'%(__name__,cfg['engine'])])
					logger.error('%s\t%s',*self.errors[-1][1:])
		else:
			self.errors.append([__name__,'missing','\"[%s]\" engine='%(__name__)])
			logger.error('%s\t%s',*self.errors[-1][1:])
		for i in engines.keys():
			tgtlang,engine = engines[i]
			# set command line for this instance
			self.subproc[i] = {'tgtlang':tgtlang,'args': list(self.args),}
			self.subproc[i]['args'].extend( ['-f', engine] )
			# configure threading
			if 'T=irstlm' in [os.path.splitext(os.path.basename(item))[0].split('-') for item in engine.split(os.sep)][-1]:
				self.subproc[i]['args'].extend(['-threads', '1'])
			else:
				self.subproc[i]['args'].extend(['-threads', '1'])
			# load moses for this instance
			logger.info('%s\t%s',*['moses',"%s - %s %s "%(tgtlang,self.kw['executable'],' '.join(self.subproc[i]['args']))])
			try:
				self.subproc[i]['popen'] = subprocess.Popen(self.subproc[i]['args'],**self.kw)
			except Exception,e:
				self.errors.append([__name__,'failed','%s %s\n%s'%(self.kw['executable'],' '.join(self.subproc[i]['args']),str(e))])
				logger.error('%s\t%s',*self.errors[-1][1:])
			else:
				bufferr = []
				for errline in iter(self.subproc[i]['popen'].stderr.readline,''):
					bufferr.append(errline.rstrip().decode(self.encoding))
					if bufferr[-1].lstrip().startswith('Created input-output object :'): break
				if not self.subproc[i]['popen'].poll() is None:
					sys.stdout.write('\n')
					for i,errline in enumerate([e for e in bufferr if e.rstrip()]):
						if errline.startswith('Usage:'): break
					self.errors.append([__name__,'termination','code: %d - %s %s\n%s'%(self.subproc[i]['popen'].returncode,self.kw['executable'],' '.join(self.subproc[i]['args']),'%s\n'%(bufferr[i - 1].strip()))])
					logger.error('%s\t%s',*self.errors[-1][1:])
				else:
					# create non-blocking thread for stderr
					self.subproc[i]['stderr'] = self.__start_thread(self.__get_stderr,self.subproc[i])

	def __del__(self):
		for i in sorted(self.subproc.keys()):
			tgtlang = self.subproc[i]['tgtlang']
			try:
				self.subproc[i]['popen'].stdin.close()
				self.subproc[i]['popen'].stdout.close()
				self.subproc[i]['popen'].stderr.close()
			except:
				pass
			else:
				while self.subproc[i]['popen'].poll() is None:
					self.subproc[i]['popen'].terminate()

	def run(self,k):
		'''run filter'''
		if not sum([len(line.strip()) for line in self.p.cfoutput[k]['tempbuff']]): return

		for i in sorted(self.subproc.keys()):
			tgtlang = self.subproc[i]['tgtlang']
			# create progress bar for translation
			self.progress.reset(0,len(self.p.cfoutput[k]['tempbuff']),72,mode='fixed',char='#')
			sys.stderr.write('     Recasing %s (%s>%s) phrases in %s\n'%(len(self.p.cfoutput[k]['tempbuff']),k[cf.srclang],tgtlang,k[cf.basename]))
			sys.stderr.write(self.progress.bar)

			try:



				for linenum,line in enumerate(self.p.cfoutput[k]['tempbuff']):
					self.subproc[i]['popen'].stdin.write('%s\n'%(line.encode(self.encoding).strip()))
					self.p.cfoutput[k]['tempbuff'][linenum] = self.subproc[i]['popen'].stdout.readline().strip().decode(self.encoding)
					self.progress.increment_amount()
					sys.stderr.write(self.progress.bar)
					sys.stderr.flush()

				sys.stderr.write('\n')
				sys.stderr.write('     Done recasing %s phrases\n'%(len(self.p.cfoutput[k]['tempbuff'])))
			except KeyboardInterrupt:
				sys.stderr.write('\n')
				for i in sorted(self.subproc.keys()):
					tgtlang = self.subproc[i]['tgtlang']
					while self.subproc[i]['popen'].poll() is None:
						try:
							self.subproc[i]['popen'].terminate()
						except:
							pass
						else:
							time.sleep(0.2)
				raise KeyboardInterrupt()

	def flush(self,k):
		return

	def close(self):
		for i in sorted(self.subproc.keys()):
			tgtlang = self.subproc[i]['tgtlang']
			try:
				self.subproc[i]['popen'].stdin.close()
				self.subproc[i]['popen'].stdout.close()
				self.subproc[i]['popen'].stderr.close()
			except:
				pass
			else:
				while self.subproc[i]['popen'].poll() is None:
					pass

	def __start_thread(self,target,*args):
		t = Thread(target=target, args=args)
		t.daemon = True
		t.start()
		return t

	def __get_stderr(self,popen):
		for line in iter(popen['popen'].stderr.readline, ''):
			pass

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import filter\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0]
	)

licensetxt=u'''DoMY™ CE v2.5
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stdout.write(usage().encode('utf8')+'\n')
