#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#2.5.326 - new tab value report

import os
import sys
import subprocess
import time
import shutil
import logging
import common as cf
import codecs
from glob import glob
from flock import flock

logger = logging.getLogger('.'.join([os.path.splitext(os.path.basename(sys.argv[0]))[0],'manager','filtergraph',__name__]))
skipclose = True

class filter(object):

	cfg = {
		'args': {},
		'exeoptions': '-w',
		'kw': {
			'bufsize': 0,
			'executable': '4,perl',
			'stdin': 'PIPE',
			'stdout': 'PIPE',
			'stderr': 'PIPE',
			'preexec_fn': None,
			'close_fds': False,
			'shell': False,
			'cwd': None,
			'env': 'LC_ALL=C',
			'universal_newlines': False,
			'startupinfo': None,
			'creationflags': 0,
			},
		'script': '4,mteval-v12.pl',
		'encoding': 'utf8',
		'version': '2.5.324',

		'forceinfoupdate': '%(forceinfoupdate)s',
		'f': None,
		'e': None,
		'tmbuildname': None,
		'alignertype': 'giza',
		'tmgrams': 7,
		'lmbuildname': None,
		'lmtype': 'irstlm',
		'lmgrams': 5,
		'evalroot': '%(evalroot)s',
		'mertroot': '%(mertroot)s',
		'makemteval': '',
		'docid': '',
		'genre': '',
		'language': '',
		'refid': '',
		'setid': '',
		'sysbleu': 0,
		'sysid': '',
		'casing': True,
		'bleu-only': False,
		'nist-only': False,
		'detailed': 0,
		'enclose-non-ascii': False,
		}
	args = []
	script = '4,mteval-v12.pl'
	kw = {}
	encoding = 'utf8'
	subproc = {}
	forceinfoupdate = False
	f = None
	e = None
	tmbuildname = None
	alignertype = 'giza'
	tmgrams = 7
	lmbuildname = None
	lmtype = 'irstlm'
	lmgrams = 5
	evalroot = 'no_eval'
	mertroot = 'no_mert'
	makemteval = '4,makemteval.py'
	mtevalcfg = {
		'docid': 'DocID',
		'genre': 'Genre',
		'language': '--',
		'refid': 'RefID',
		'setid': 'SetID',
		'sysbleu': 1.0000,
		'sysid': 'SysID',
		}
	isopen = False
	p = object
	errors = []

	def open(self,parent,cfg):
		self.args = dict(cfg['args'])
		self.kw = dict(cfg['kw'])
		if cfg['script']:
			self.script = self.p.decodepath(','.join([__name__.split(',')[1]]+cfg['script'].split(',')),True)
			if not self.script:
				self.errors.append([__name__,'missing',self.script])
				logger.error('%s\t%s',*self.errors[-1][1:])
			self.args[0] = self.script
			self.kw['cwd'] = os.path.dirname(self.script)
		else:
			self.script = None
		# convert args dict to args list
		self.args = [str(self.args[k]) for k in sorted(self.args.keys())]
		if cfg['kw']['executable']:
			self.kw['executable'] = self.p.decodepath(','.join([__name__.split(',')[1]]+cfg['kw']['executable'].split(',')),True)
			if not self.kw['executable']:
				self.errors.append([__name__,'missing',self.kw['executable']])
				logger.error('%s\t%s',*self.errors[-1][1:])
			self.args.insert(0,cfg['exeoptions'] if cfg['exeoptions'] else '')
		else:
			self.kw['executable'] = None
		self.kw['stdin'] = subprocess.PIPE if cfg['kw']['stdin'] == 'PIPE' else cfg['kw']['stdin']
		self.kw['stdout'] = subprocess.PIPE if cfg['kw']['stdout'] == 'PIPE' else cfg['kw']['stdout']
		self.kw['stderr'] = subprocess.STDOUT if cfg['kw']['stderr'] == 'PIPE' else cfg['kw']['stderr']
		self.encoding = 'utf8' if 'utf8' in cfg['encoding'].lower().replace('-','') else cfg['encoding']

		# test required values
		if not cfg['f'] or cfg['f'] == '%(f)s' or cfg['f'] == '%(srclang)s':
			self.errors.append([__name__,'missing','\"[%s]\" f= must be set'%(__name__,)])
			logger.error('%s\t%s',*self.errors[-1][1:])
		if not cfg['e'] or cfg['e'] == '%(e)s' or cfg['e'] == '%(tgtlang)s':
			self.errors.append([__name__,'missing','\"[%s]\" e= must be set'%(__name__,)])
			logger.error('%s\t%s',*self.errors[-1][1:])
		self.f = cfg['f']
		self.e = cfg['e']

		# TM values
		if not cfg['tmbuildname'] or cfg['tmbuildname'] == '%(tmbuildname)s':
			self.errors.append([__name__,'missing','\"[%s]\" tmbuildname= must be set'%(__name__,)])
			logger.error('%s\t%s',*self.errors[-1][1:])
		self.tmbuildname = cfg['tmbuildname']

		self.alignertype = self.alignertype if cfg['alignertype'] == '%(alignertype)s' else cfg['alignertype'].lower()
		if 'giza' in self.alignertype:
			self.alignertype = 'giza'
		elif 'berk' in self.alignertype:
			self.alignertype = 'berk'
		else:
			self.errors.append([__name__,'invalid','\"[%s]\" alignertype= must be set to \'giza\', or \'berk\''%(__name__,)])
			logger.error('%s\t%s',*self.errors[-1][1:])

		self.tmgrams = cfg['tmgrams'] if cfg['tmgrams'] else self.tmgrams

		# LM values
		if not cfg['lmbuildname'] or (cfg['lmbuildname'].startswith('%(') and cfg['lmbuildname'].endswith(')s')):
			self.errors.append([__name__,'missing','\"[%s]\" lmbuildname must be set'%(__name__,)])
			logger.warn('%s\t%s',*self.errors[-1][1:])
		self.lmbuildname = cfg['lmbuildname']

		self.lmtype = self.lmtype.lower() if cfg['lmtype'] == '%(lmtype)s' else cfg['lmtype'].lower()
		if not self.lmtype in ['irstlm','randlm','srilm','irstlmken','srilmken',]:
			self.errors.append([__name__,'invalid','\"[%s]\" lmtype=%s. It must be set to \'irstlm\', \'randlm\', \'srilm\', \'irstlmken\',  or \'srilmken\''%(__name__,self.lmtype)])
			logger.error('%s\t%s',*self.errors[-1][1:])

		self.lmgrams = cfg['lmgrams'] if cfg['lmgrams'] else self.lmgrams

		# other values
		self.forceinfoupdate = self.forceinfoupdate if not cfg['forceinfoupdate'] or (cfg['forceinfoupdate'].startswith('%(') and cfg['forceinfoupdate'].endswith(')s')) else cf.zbool(cfg['forceinfoupdate'])
		self.evalroot = cfg['evalroot'] if cfg['evalroot'] and not cfg['evalroot'].startswith('%(') and not cfg['evalroot'].endswith(')s') else self.evalroot
		self.mertroot = cfg['mertroot'] if cfg['mertroot'] and not cfg['mertroot'].startswith('%(') and not cfg['mertroot'].endswith(')s') else self.mertroot

		# create makemteval object
		self.makemteval = cfg['makemteval'] if cfg['makemteval'] else self.makemteval
		self.makemteval = self.p.decodepath(self.makemteval,True)
		if not self.makemteval:
			self.errors.append([__name__,'missing','\"[%s]\" \"makemteval=%s\"'%(__name__,self.makemteval)])
			logger.error('%s\t%s',*self.errors[-1][1:])
		try:
			import imp
			fp, pathname, description = imp.find_module(os.path.splitext(os.path.basename(self.makemteval))[0],[os.path.dirname(self.makemteval)])
			self.mksgm = imp.load_module(os.path.splitext(os.path.basename(self.makemteval))[0], fp, pathname, description)
		except:
			self.errors.append([__name__,'failed','import \"[%s]\" \"buildname=%s\"'%(__name__,self.makemteval)])
			logger.error('%s\t%s',*self.errors[-1][1:])
		if fp: fp.close()

		if cfg['casing']: 
			self.args.append('-c')
		if cfg['bleu-only']:
			self.args.append('-b')
			if cf.isinteger(cfg['detailed']) and 0 < cfg['detailed'] < 4:
				self.args.extend(['-d',str(cfg['detailed'])])
		elif cfg['nist-only']:
			self.args.append('-n')
			if cf.isinteger(cfg['detailed']) and 0 < cfg['detailed'] < 4:
				self.args.extend(['-d',str(cfg['detailed'])])
		if cfg['enclose-non-ascii']:
			self.args.append('-e')

		self.mtevalcfg['genre'] = cfg['genre'] if cfg['genre'] else self.mtevalcfg['genre']
		self.mtevalcfg['language'] = cfg['language'] if  cfg['language'] else self.mtevalcfg['language']
		self.mtevalcfg['sysid'] = cfg['sysid'] if cfg['sysid'] else '%s-s=%s-t=%s-p=%s-a=%s-g=%s-l=%s-T=%s-n=%s'%(self.mertroot,self.f,self.e,self.tmbuildname,self.alignertype,self.tmgrams,self.lmbuildname,self.lmtype,self.lmgrams,)
		if os.path.exists(os.sep.join([self.p.rootfolder,self.p.writer.roottype,self.p.writer.stage,'eval-%s'%(self.mtevalcfg['sysid']),'moses-%s.ini'%(self.mertroot),])):
			self.mtevalcfg['sysbleu'] = [line.split() for line in open(os.sep.join([self.p.rootfolder,self.p.writer.roottype,self.p.writer.stage,'eval-%s'%(self.mtevalcfg['sysid']),'moses-%s.ini'%(self.mertroot),]),'r').read().splitlines() if line.startswith('# BLEU ')][0][2]
		self.mtevalcfg['sysbleu'] = cfg['sysbleu'] if cfg['sysbleu'] else self.mtevalcfg['sysbleu']

		self.count = 0
		# set input files
		self.inputs = {}
		# set output file
		self.outputs = {}

	def run(self,k):
		global skipclose

		if k and not k[cf.srclang] == k[cf.rdrlang]: return
		basename = os.extsep.join([k[cf.basename],'txt']) if k else os.extsep.join([self.evalroot,'txt'])

		self.inputs[self.count] = [
			os.sep.join([self.p.rootfolder,self.p.writer.roottype,self.p.writer.stage,'eval-%s'%(self.mtevalcfg['sysid']),'tm',self.f,self.f,basename]),
			os.sep.join([self.p.rootfolder,self.p.writer.roottype,self.p.writer.stage,'eval-%s'%(self.mtevalcfg['sysid']),'tm',self.f,self.e,basename]),
			os.sep.join([self.p.rootfolder,self.p.writer.roottype,self.p.writer.stage,'eval-%s'%(self.mtevalcfg['sysid']),'tm',self.f,'%s_domy'%(self.e),basename]),
			]

		self.outputs[self.count] = [
			os.sep.join([self.p.rootfolder,self.p.writer.roottype,self.p.writer.stage,'eval-%s'%(self.mtevalcfg['sysid']),'mteval-v12.pl-%s.txt'%(os.path.splitext(basename)[0])]),
			]

		if '-d 2' in ' '.join(self.args):
			self.outputs[self.count].append(os.sep.join([self.p.rootfolder,self.p.writer.roottype,self.p.writer.stage,'eval-%s'%(self.mtevalcfg['sysid']),'mteval-v12.pl-%s.tab'%(os.path.splitext(basename)[0])]))

		skipclose = False
		self.count += 1

	def flush(self,k):
		return

	def close(self):
		if skipclose: return

		lockfile = os.sep.join(['','var','tmp','domy','train.lock'])
		if not os.path.exists(os.path.dirname(lockfile)): os.makedirs(os.path.dirname(lockfile))
		lock = flock(lockfile,False).acquire()
		if not lock or not lock.ownlock():
			self.errors.append([__name__,'failed','train lock, another training session is running'])
			logger.error('%s\t%s',*self.errors[-1][1:])
			return
		with open(lockfile) as f:
			lockcontent = f.readline().rstrip()

		for i in sorted(self.inputs.keys()):

			if self.errors: continue

			# test input and output files
			infodata = cf.updatefingerprint(self.inputs[i],self.outputs[i],self.forceinfoupdate)
			if 'bypass' in infodata:
				# output file(s) exists.
				logger.info('%s\t%s',*['skipping','outputs exist and verified'])
				sys.stderr.write('%s - Skipping. [%s] Outputs exist and verified\n'%(cf.timestamp(),__name__))
				sys.stderr.write('\n\nPrevious report:\n')
				with open(self.outputs[i][0],'r') as f:
					sys.stderr.write(f.read())
				lock.release()
				return
			elif 'errors' in infodata:
				# report/return self.errors
				for error in infodata['errors']:
					self.errors.append([__name__,'error',error.strip()])
					logger.error('%s\t%s',*self.errors[-1][1:])
				lock.release()
				return
			sys.stderr.flush()

			logger.info('%s\t%s',*['start','mteval-v12.pl'])
			sys.stderr.write('\nStart - mteval-v12.pl [%s]\n'%(__name__))

			# make output folder
			for out in self.outputs[i]:
				try:
					os.makedirs(os.path.dirname(out))
				except OSError,e:
					if not e.errno == 17:
						logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
						raise OSError(e)

			# create makemteval object
			mksgm = self.mksgm.makemteval()

			self.mtevalcfg['docid'] = os.path.splitext(os.path.basename(self.inputs[i][0]))[0]
			self.mtevalcfg['refid'] = os.path.splitext(os.path.basename(self.inputs[i][1]))[0]
			self.mtevalcfg['setid'] = os.path.splitext(os.path.basename(self.inputs[i][2]))[0]

			# create srcset
			self.mtevalcfg['filein'] = self.inputs[i][0]
			self.mtevalcfg['fileout'] = self.inputs[i][0].replace('.txt','.xml')
			self.mtevalcfg['settype'] = 'srcset'
			self.mtevalcfg['srclang'] = self.f
			self.mtevalcfg['tstlang'] = self.e
			if mksgm.writesgm(self.mtevalcfg):
				self.errors.append([__name__,'error','%s'%('makemteval failed on \'srcset\'')])
				logger.error('%s\t%s',*self.errors[-1][1:])

			# create refset
			self.mtevalcfg['filein'] = self.inputs[i][1]
			self.mtevalcfg['fileout'] = self.inputs[i][1].replace('.txt','.xml')
			self.mtevalcfg['settype'] = 'refset'
			self.mtevalcfg['srclang'] = self.f
			self.mtevalcfg['tstlang'] = self.e
			if mksgm.writesgm(self.mtevalcfg):
				self.errors.append([__name__,'error','%s'%('makemteval failed on \'refset\'')])
				logger.error('%s\t%s',*self.errors[-1][1:])

			# create tstset
			self.mtevalcfg['filein'] = self.inputs[i][2]
			self.mtevalcfg['fileout'] = self.inputs[i][2].replace('.txt','.xml')
			self.mtevalcfg['settype'] = 'tstset'
			self.mtevalcfg['srclang'] = self.f
			self.mtevalcfg['tstlang'] = self.e
			if mksgm.writesgm(self.mtevalcfg):
				self.errors.append([__name__,'error','%s'%('makemteval failed on \'tstset\'')])
				logger.error('%s\t%s',*self.errors[-1][1:])

			if not self.errors:
				# build command line
				args = list(self.args)
				args.extend([
					'-s',self.inputs[i][0].replace('.txt','.xml'),
					'-r',self.inputs[i][1].replace('.txt','.xml'),
					'-t',self.inputs[i][2].replace('.txt','.xml'),
					])

				# run mteval-v12.pl
				try:
					self.subproc['popen'] = subprocess.Popen(args,**self.kw)
				except Exception,e:
					self.errors.append([__name__,'failed','%s %s - %s'%(os.path.basename(self.kw['executable']),' '.join(args),str(e))])
					logger.error('%s\t%s',*self.errors[-1][1:])
					lock.release()
					return

				logger.info('%s\t%d - %s %s',*['run',self.subproc['popen'].pid,os.path.basename(self.kw['executable']),' '.join(args)])

				# wait to load
				outbuff = []

				if '-d 2' in ' '.join(self.args):
					tabbuff = [
						'System ID:\t%s'%(self.mtevalcfg['sysid']),
						'Test set:\t%s'%(self.mtevalcfg['setid']),
						'Total Lines:',
						'Identical:',
						'BLEU=1.0:',
						'',
						'Line #\tBLEU (4-gram)\tidentical\tSource\tReference\tTest',
						]
					with codecs.open(self.inputs[i][0],'r','utf-8') as s:
						src = s.read().splitlines()
					with codecs.open(self.inputs[i][1],'r','utf-8') as r:
						ref = r.read().splitlines()
					with codecs.open(self.inputs[i][2],'r','utf-8') as t:
						tst = t.read().splitlines()

				identical = 0
				bleu1 = 0
				for line in iter(self.subproc['popen'].stdout.readline,''):
					if line.lstrip().startswith('length ratio:') or line.lstrip().startswith('BLEU score = '): continue
					if line.lstrip().startswith('BLEU score using 4-grams = '):
						if '-d 2' in ' '.join(self.args):
							line = line.split()
							if ref[int(line[11])-1] == tst[int(line[11])-1]: identical +=1
							if line[5] == '1.0000': bleu1 += 1
							tabbuff.append('%s\t%s\t%s\t%s\t%s\t%s'%(line[11],line[5],ref[int(line[11])-1] == tst[int(line[11])-1],src[int(line[11])-1],ref[int(line[11])-1],tst[int(line[11])-1]))
						continue

					outbuff.append(line.rstrip())
					sys.stderr.write('%s\n'%(line.rstrip()))
				sys.stderr.write('\n')
				sys.stderr.flush()

				# save output report
				try:
					with open(self.outputs[i][0],'w') as f:
						f.write('%s\x0d\x0a'%('\x0d\x0a'.join(outbuff)))
				except:
					if os.path.exists(self.outputs[i][0]): os.unlink(self.outputs[i][0])
					self.errors.append([__name__,'missing','%s'%(self.outputs[i][0])])
					logger.error('%s\t%s',*self.errors[-1][1:])

				# save output tab table
				if '-d 2' in ' '.join(self.args):
					tabbuff[2] = '%s\t%s'%(tabbuff[2],len(tabbuff)-7)
					tabbuff[3] = '%s\t%s\t(%s%%)'%(tabbuff[3],identical,round((float(identical)/(len(tabbuff)-7))*100,1))
					tabbuff[4] = '%s\t%s\t(%s%%)'%(tabbuff[4],bleu1,round((float(bleu1)/(len(tabbuff)-7))*100,1))
					try:
						with codecs.open(self.outputs[i][1],'w','utf-8') as f:
							f.write('%s\x0d\x0a'%('\x0d\x0a'.join(tabbuff)))
					except:
						if os.path.exists(self.outputs[i][1]): os.unlink(self.outputs[i][1])
						self.errors.append([__name__,'missing','%s'%(self.outputs[i][1])])
						logger.error('%s\t%s',*self.errors[-1][1:])

				while self.subproc['popen'].poll() is None:
					time.sleep(0.2)
				if self.subproc['popen'].returncode:
					self.errors.append([__name__,'%s\tmteval-v12.pl returncode = %s'%('failed',self.subproc['popen'].returncode)])
					logger.error('%s\t%s',*self.errors[-1][1:])

			# update infodata and save
			if self.errors:
				logger.info('%s\t%s',*['error','mteval-v12.pl - return code = %s'%(self.subproc['popen'].returncode)])
				sys.stderr.write('Error - mteval-v12.pl - return code = %s\n'%(self.subproc['popen'].returncode))
			else:
				infodata = cf.updatefingerprint([],self.outputs[i],True,infodata)
				if 'errors' in infodata:
					# report/return self.errors
					for error in infodata['errors']:
						self.errors.append([__name__,'error',error.strip()])
						logger.error('%s\t%s',*self.errors[-1][1:])
				else:
					logger.info('%s\t%s',*['success','mteval-v12.pl'])
					sys.stderr.write('Success - mteval-v12.pl\n')

		sys.stderr.flush()
		lock.release()

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import filter\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0]
	)

licensetxt=u'''DoMY™ CE v2.5
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stderr.write(usage().encode('utf8')+'\n')
