#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#2.5.321 - changed _eval to _domy

import os
import sys
import subprocess
import logging
import time
import common as cf
import shutil
from threading import Thread
from flock import flock
from progress_bar import ProgressBar
import codecs

logger = logging.getLogger('.'.join([os.path.splitext(os.path.basename(sys.argv[0]))[0],'manager','filtergraph',__name__]))
skipclose = True
endsignal = '01AD2372-0AAE-430E-BE15-7C490AFD7412'

class filter(object):

	cfg = {
		'args': {},
		'encoding': 'utf8',
		'exeoptions': '',

		'kw': {
			'bufsize': 0,
			'executable': '4,moses',
			'stdin': 'PIPE',
			'stdout': 'PIPE',
			'stderr': 'PIPE',
			'preexec_fn': None,
			'close_fds': False,
			'shell': False,
			'cwd': None,
			'env': None,
			'universal_newlines': False,
			'startupinfo': None,
			'creationflags': 0,
			},
		'script': '',
		'version': '2.5.321',

		'forceinfoupdate': '%(forceinfoupdate)s',
		'f': None,
		'e': None,
		'tmbuildname': None,
		'alignertype': 'giza',
		'tmgrams': 7,
		'lmbuildname': None,
		'lmtype': 'irstlm',
		'lmgrams': 5,
		'evalroot': '%(evalroot)s',
		'mertroot': '%(mertroot)s',
		}
	args = []
	encoding = 'utf8'
	kw = {}
	script = ''
	forceinfoupdate = False
	f = None
	e = None
	tmbuildname = None
	alignertype = 'giza'
	tmgrams = 7
	lmbuildname = None
	lmtype = 'irstlm'
	lmgrams = 5
	evalroot = 'no_eval'
	mertroot = 'no_mert'
	isopen = False
	p = object
	errors = []

	def open(self,parent,cfg):
		self.args = dict(cfg['args'])
		self.kw = dict(cfg['kw'])
		if cfg['script']:
			self.script = self.p.decodepath(','.join([__name__.split(',')[1]]+cfg['script'].split(',')),True)
			if not self.script:
				self.errors.append([__name__,'missing',self.script])
				logger.error('%s\t%s',*self.errors[-1][1:])
			self.args[0] = self.script
			self.kw['cwd'] = os.path.dirname(self.script)
		else:
			self.script = None
		# convert args dict to args list
		self.args = [str(self.args[k]) for k in sorted(self.args.keys())]
		if cfg['kw']['executable']:
			self.kw['executable'] = self.p.decodepath(','.join([__name__.split(',')[1]]+cfg['kw']['executable'].split(',')),True)
			if not self.kw['executable']:
				self.errors.append([__name__,'missing',self.kw['executable']])
				logger.error('%s\t%s',*self.errors[-1][1:])
			self.args.insert(0,cfg['exeoptions'] if cfg['exeoptions'] else '')
		else:
			self.kw['executable'] = None
		self.kw['stdin'] = subprocess.PIPE if cfg['kw']['stdin'] == 'PIPE' else cfg['kw']['stdin']
		self.kw['stdout'] = subprocess.PIPE if cfg['kw']['stdout'] == 'PIPE' else cfg['kw']['stdout']
		self.kw['stderr'] = subprocess.PIPE if cfg['kw']['stderr'] == 'PIPE' else cfg['kw']['stderr']
		self.encoding = 'utf8' if 'utf8' in cfg['encoding'].lower().replace('-','') else cfg['encoding']

		# test required values
		if not cfg['f'] or cfg['f'] == '%(f)s' or cfg['f'] == '%(srclang)s':
			self.errors.append([__name__,'missing','\"[%s]\" f= must be set'%(__name__,)])
			logger.error('%s\t%s',*self.errors[-1][1:])
		if not cfg['e'] or cfg['e'] == '%(e)s' or cfg['e'] == '%(tgtlang)s':
			self.errors.append([__name__,'missing','\"[%s]\" e= must be set'%(__name__,)])
			logger.error('%s\t%s',*self.errors[-1][1:])
		self.f = cfg['f']
		self.e = cfg['e']

		# TM values
		if not cfg['tmbuildname'] or cfg['tmbuildname'] == '%(tmbuildname)s':
			self.errors.append([__name__,'missing','\"[%s]\" tmbuildname= must be set'%(__name__,)])
			logger.warn('%s\t%s',*self.errors[-1][1:])
		self.tmbuildname = cfg['tmbuildname']

		self.alignertype = self.alignertype if cfg['alignertype'] == '%(alignertype)s' else cfg['alignertype'].lower()
		if 'giza' in self.alignertype:
			self.alignertype = 'giza'
		elif 'berk' in self.alignertype:
			self.alignertype = 'berk'
		else:
			self.errors.append([__name__,'invalid','\"[%s]\" alignertype= must be set to \'giza\', or \'berk\''%(__name__,)])
			logger.error('%s\t%s',*self.errors[-1][1:])

		self.tmgrams = cfg['tmgrams'] if cfg['tmgrams'] else self.tmgrams

		# LM values
		if not cfg['lmbuildname'] or (cfg['lmbuildname'].startswith('%(') and cfg['lmbuildname'].endswith(')s')):
			self.errors.append([__name__,'missing','\"[%s]\" lmbuildname must be set'%(__name__,)])
			logger.warn('%s\t%s',*self.errors[-1][1:])
		self.lmbuildname = cfg['lmbuildname']

		self.lmtype = self.lmtype.lower() if cfg['lmtype'] == '%(lmtype)s' else cfg['lmtype'].lower()
		if not self.lmtype in ['irstlm','randlm','srilm','irstlmken','srilmken',]:
			self.errors.append([__name__,'invalid','\"[%s]\" lmtype=%s. It must be set to \'irstlm\', \'randlm\', \'srilm\', \'irstlmken\', or \'srilmken\''%(__name__,self.lmtype)])
			logger.error('%s\t%s',*self.errors[-1][1:])

		self.lmgrams = cfg['lmgrams'] if cfg['lmgrams'] else self.lmgrams

		# other values
		self.forceinfoupdate = self.forceinfoupdate if not cfg['forceinfoupdate'] or (cfg['forceinfoupdate'].startswith('%(') and cfg['forceinfoupdate'].endswith(')s')) else cf.zbool(cfg['forceinfoupdate'])
		self.evalroot = cfg['evalroot'] if cfg['evalroot'] and not cfg['evalroot'].startswith('%(') and not cfg['evalroot'].endswith(')s') else self.evalroot
		self.mertroot = cfg['mertroot'] if cfg['mertroot'] and not cfg['mertroot'].startswith('%(') and not cfg['mertroot'].endswith(')s') else self.mertroot

		# configure threading
		self.cpu_count = 1 if self.lmtype == 'irstlm' else 1

		evalfolder = 'eval-%s-s=%s-t=%s-p=%s-a=%s-g=%s-l=%s-T=%s-n=%s'%(self.mertroot,self.f,self.e,self.tmbuildname,self.alignertype,self.tmgrams,self.lmbuildname,self.lmtype,self.lmgrams,)
		self.inputs = [
			os.sep.join([self.p.rootfolder,'BUILDS','tm',self.tmbuildname,os.extsep.join([self.evalroot,self.f])]),
			os.sep.join([self.p.rootfolder,'BUILDS','tm',self.tmbuildname,os.extsep.join([self.evalroot,self.e])]),
			os.sep.join([self.p.rootfolder,'ENGINES','evals',evalfolder,'moses-%s.ini'%(self.mertroot),]),
			]

		self.outputs = [
			os.sep.join([self.p.rootfolder,'ENGINES','evals',evalfolder,'tm',self.f,self.f,os.extsep.join([self.evalroot,'txt'])]),
			os.sep.join([self.p.rootfolder,'ENGINES','evals',evalfolder,'tm',self.f,self.e,os.extsep.join([self.evalroot,'txt'])]),
			os.sep.join([self.p.rootfolder,'ENGINES','evals',evalfolder,'tm',self.f,'%s_domy'%(self.e),os.extsep.join([self.evalroot,'txt'])]),
			]

	def __del__(self):
		try:
			self.subproc.stdin.close()
			self.subproc.stdout.close()
			self.subproc.stderr.close()
		except:
			pass
		else:
			while self.subproc.poll() is None:
				self.subproc.terminate()

	def run(self,k):
		global skipclose
		skipclose = False

	def flush(self,k):
		return

	def close(self):
		if skipclose: return

		lockfile = os.sep.join(['','var','tmp','domy','train.lock'])
		if not os.path.exists(os.path.dirname(lockfile)): os.makedirs(os.path.dirname(lockfile))
		lock = flock(lockfile,False).acquire()
		if not lock or not lock.ownlock():
			self.errors.append([__name__,'failed','train lock, another training session is running'])
			logger.error('%s\t%s',*self.errors[-1][1:])
			return
		with open(lockfile) as f:
			lockcontent = f.readline().rstrip()

		# test input and output files
		infodata = cf.updatefingerprint(self.inputs,self.outputs,self.forceinfoupdate)
		if 'bypass' in infodata:
			# output file(s) exists.
			logger.info('%s\t%s',*['skipping','outputs exist and verified'])
			sys.stderr.write('%s - Skipping. [%s] Outputs exist and verified\n'%(cf.timestamp(),__name__))
			lock.release()
			return
		elif 'errors' in infodata:
			# report/return self.errors
			for error in infodata['errors']:
				self.errors.append([__name__,'error',error.strip()])
				logger.error('%s\t%s',*self.errors[-1][1:])
			lock.release()
			return
		sys.stderr.flush()

		logger.info('%s\t%s',*['start','train-generate-eval'])
		sys.stderr.write('\nStart - train-generate-eval [%s]\n'%(__name__))

		# set command line for this instance
		args = list(self.args)
		args.extend(['-f', self.inputs[2]])
		args.extend(['-v', str(1)])
		args.extend(['-threads', str(self.cpu_count)])

		# load moses for this instance
		try:
			self.subproc = subprocess.Popen(args,**self.kw)
		except Exception,e:
			self.errors.append([__name__,'failed','%s %s - %s'%(os.path.basename(self.kw['executable']),' '.join(args),str(e))])
			logger.error('%s\t%s',*self.errors[-1][1:])
			lock.release()
			return

		logger.info('%s\t%s',*['run',"%s %s "%(self.kw['executable'],' '.join(args))])

		# wait to load
		bufferr = []
		for errline in iter(self.subproc.stderr.readline, ''):
			bufferr.append(errline.rstrip().decode(self.encoding))
			if bufferr[-1].lstrip().startswith('Created input-output object :'): break

		# test for self.errors
		if not self.subproc.poll() is None:
			sys.stdout.write('\n')
			for i,errline in enumerate([e.rstrip() for e in bufferr if e.rstrip()]):
				if errline.startswith('Usage:'): break
				sys.stdout.write('%s\n'%(errline.replace('(per moses.ini or switch)','(per config file or switch)').encode(self.encoding)))
			self.errors.append([__name__,'termination','code: %d - %s %s\n%s'%(self.subproc.returncode,self.kw['executable'],' '.join(self.subproc[tgtlang]['args']),'%s\n'%(bufferr[i - 1]))])
			logger.error('%s\t%s',*self.errors[-1][1:])
			lock.release()
			return

		# create threads for non-blocking stderr
		errthread = _start_thread(_get_stderr,self.subproc)

		ttl_lines = cf.file_length(self.inputs[0])
		progress = ProgressBar()
		progress.reset(0,ttl_lines,72,mode='fixed',char='#')
		sys.stderr.write("generate %s test phrases for evaluation\n"%(ttl_lines))
		sys.stderr.write(progress.bar)
		sys.stderr.flush()

		tstbuff = []




		with codecs.open(self.inputs[0],'r',self.encoding) as f:
			srcbuff = f.read().splitlines()

		for i,line in enumerate(srcbuff):
			self.subproc.stdin.write('%s\n'%(line.encode(self.encoding).strip()))
			tstbuff.append(self.subproc.stdout.readline().strip().decode(self.encoding))
			progress.increment_amount()
			sys.stderr.write(progress.bar)
			sys.stderr.flush()











		sys.stderr.write('\n')
		sys.stderr.write('     Done with %s phrases\n'%(ttl_lines))
		sys.stderr.flush()

		# terminate moses
		try:
			self.subproc.stdin.close()
			self.subproc.stdout.close()
			self.subproc.stderr.close()
		except:
			pass
		else:
			while self.subproc.poll() is None:
				self.subproc.terminate()

		# make output folder
		for out in self.outputs:
			try:
				os.makedirs(os.path.dirname(out))
			except OSError,e:
				if not e.errno == 17:
					logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
					raise OSError(e)

		# save translated output as text file
		try:
			with codecs.open(self.outputs[2],'w',self.encoding) as f:
				f.write('%s\n'%('\n'.join(tstbuff)))
		except:
			if os.path.exists(self.outputs[2]): os.unlink(self.outputs[2])
			self.errors.append([__name__,'missing','%s'%(self.outputs[2])])
			logger.error('%s\t%s',*self.errors[-1][1:])
		else:
			shutil.copyfile(self.inputs[0],self.outputs[0])
			shutil.copyfile(self.inputs[1],self.outputs[1])

		# clean up
		if self.errors:
			if os.path.exists(self.outputs[0]): os.unlink(self.outputs[0])
			if os.path.exists(self.outputs[1]): os.unlink(self.outputs[1])
			self.errors.append([__name__,'error','train-generate-eval'])
			logger.error('%s\t%s',*self.errors[-1][1:])
		else:
			infodata = cf.updatefingerprint([],self.outputs,True,infodata)
			if 'errors' in infodata:
				# report/return self.errors
				for error in infodata['errors']:
					self.errors.append([__name__,'error',error.strip()])
					logger.error('%s\t%s',*self.errors[-1][1:])
			else:
				logger.info('%s\t%s',*['success','train-generate-eval'])
				sys.stderr.write('Success - train-generate-eval\n')

		sys.stderr.flush()
		lock.release()

def _start_thread(target,*args):
	t = Thread(target=target, args=args)
	t.daemon = True
	t.start()
	return t

def _get_stderr(popen):
	for line in iter(popen.stderr.readline,''):
		pass

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import filter\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0]
	)

licensetxt=u'''DoMY™ CE v2.5
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stdout.write(usage().encode('utf8')+'\n')
