#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#2.5.321 - update to support inputfile config option
#          don't delete intermediary irstlm for recasers

import os
import sys
import subprocess
import time
import shutil
import logging
import common as cf
from flock import flock
from glob import glob

logger = logging.getLogger('.'.join([os.path.splitext(os.path.basename(sys.argv[0]))[0],'manager','filtergraph',__name__]))
skipclose = True
lmbytype = {
	'srilm': '0',
	'irstlm': '1',
	'randlm': '5',


	}

class filter(object):

	cfg = {
		'args': {},
		'exeoptions': '',
		'kw': {
			'bufsize': 0,
			'executable': '',
			'stdin': 'PIPE',
			'stdout': 'PIPE',
			'stderr': 'PIPE',
			'preexec_fn': None,
			'close_fds': False,
			'shell': False,
			'cwd': os.sep+'tmp',
			'env': 'LC_ALL=C',
			'universal_newlines': False,
			'startupinfo': None,
			'creationflags': 0,
			},
		'script': '',
		'version': '2.5.321',

		'encoding': 'utf8',
		'recaser': '%(recaser)s',
		'forceinfoupdate': '%(forceinfoupdate)s',
		'lmtype': 'irstlm',
		'lmroot': '%(lmroot)s',
		'lmbuildname': None,
		'e': None,
		'lmgrams': 5,
		'tmgrams': 7,
		'alignertype': 'recaser',
		'tmpdir': '',
		'inputfile': '',
		'inputs': [],
		'logfile': '',
		'outputs': [],
#		IRSTLM
		'boundaries': False,
		'dictionary': '',
		'parts': 0,
		'prune': False,
		'smoothing': 'witten-bell',
		'uniform': False,
		'perl': '4,perl',
		'dict': '4,dict',
		'split-dict.pl': '4,split-dict.pl',
		'ngt': '4,ngt',
		'build-sublm.pl': '4,build-sublm.pl',
		'merge-sublm.pl': '4,merge-sublm.pl',
		'compile-lm': '4,compile-lm',
#		RANDLM
		'lmstructure': '',
		'falsepos': 8,
		'values': 8,
#		SRILM
		'discount': 'kndiscount',
		'boundaries': False,
		'interpolate': True,
		'memuse': True,







		}
	kw = {}
	encoding = 'utf8'
	subprocs = {}
	tmpdir = ''
	deltemp = True
	inputs = []
	logfile = ''
	outputs = []
	recaser = False
	lmtype = 'irstlm'
	lmroot = 'monotext'
	lmbuildname = None
	e = None
	lmgrams = 5
	tmgrams = 7
	alignertype = 'recaser'
#	IRSTLM
	boundaries = False
	dictionary = ''
	forceinfoupdate = False
	parts = 0
	prune = False
	smoothing = 'witten-bell'
	uniform = False
	progs = ['dict','split-dict.pl','ngt','build-sublm.pl','merge-sublm.pl','compile-lm']
	perl = '4,perl'
#	RANDLM
	lmstructure = ''
	falsepos = 8
	values = 8
#	SRILM
	discount = 'kndiscount'
	boundaries = False
	interpolate = True
	memuse = True







	isopen = False
	p = object
	errors = []

	def __del__(self):
		if self.deltemp and os.path.exists(self.tmpdir):
			shutil.rmtree(self.tmpdir)

	def open(self,parent,cfg):

		self.kw = dict(cfg['kw'])
		self.kw['stdin'] = subprocess.PIPE if cfg['kw']['stdin'] == 'PIPE' else self.kw['stdin']
		self.kw['stdout'] = subprocess.PIPE if cfg['kw']['stdout'] == 'PIPE' else self.kw['stdout']
		self.kw['stderr'] = subprocess.STDOUT if cfg['kw']['stderr'] == 'PIPE' else self.kw['stderr']

		# common options
		self.encoding = 'utf8' if 'utf8' in cfg['encoding'].lower().replace('-','') else cfg['encoding']
		self.e = cfg['e'] if not cfg['e'].startswith('%(') and not cfg['e'].endswith(')s') else None
		if not self.e:
			self.errors.append([__name__,'missing','\"[%s]\" \"e=\"'%(__name__)])
			logger.error('%s\t%s',*self.errors[-1][1:])
		self.lmbuildname = cfg['lmbuildname'] if not cfg['lmbuildname'].startswith('%(') and not cfg['lmbuildname'].endswith(')s') else None
		if not self.lmbuildname:
			self.errors.append([__name__,'missing','\"[%s]\" \"lmbuildname=\"'%(__name__)])
			logger.error('%s\t%s',*self.errors[-1][1:])
		self.lmgrams = cfg['lmgrams'] if cf.isinteger(cfg['lmgrams']) else self.lmgrams

		self.recaser = self.recaser if not cfg['recaser'] or (cfg['recaser'].startswith('%(') and cfg['recaser'].endswith(')s')) else cf.zbool(cfg['recaser'])
		self.forceinfoupdate = self.forceinfoupdate if not cfg['forceinfoupdate'] or (cfg['forceinfoupdate'].startswith('%(') and cfg['forceinfoupdate'].endswith(')s')) else cf.zbool(cfg['forceinfoupdate'])
		self.lmroot = cfg['lmroot'] if cfg['lmroot'] and not cfg['lmroot'].startswith('%(') and not cfg['lmroot'].endswith(')s') else 'recasetext' if self.recaser else 'monotext'

		# other options
		self.tmpdir = cfg['tmpdir'] if cfg['tmpdir'] else '%s.tmp'%(os.sep.join([self.p.tempdir,__name__]))

		self.lmtype = cfg['lmtype'].lower() if cfg['lmtype'] and not cfg['lmtype'].startswith('%(') and not cfg['lmtype'].endswith(')s') else self.lmtype.lower()
		if not self.lmtype in lmbytype.keys():
			self.errors.append([__name__,'invalid','\"[%s]\" lmtype=%s must be set to \'irstlm\', \'randlm\', \'srilm\''%(__name__,self.lmtype)])
			logger.error('%s\t%s',*self.errors[-1][1:])
		lmtype = cf.iniFileGet(self.p.config_setup,self.lmtype.lower().replace('ken',''),'installdir')
		if lmtype: os.environ[self.lmtype.upper().replace('KEN','')] = lmtype
		if not self.lmtype.upper().replace('KEN','') in os.environ:
			self.errors.append([__name__,'missing','\"[%s]\" \"%s\" environment variable'%(__name__,self.lmtype.upper())])
			logger.error('%s\t%s',*self.errors[-1][1:])

		if self.recaser:
			# TM values
			self.alignertype = cfg['alignertype'].lower() if cfg['alignertype'] and not cfg['alignertype'].startswith('%(') and not cfg['alignertype'].endswith(')s') else self.alignertype
			if 'giza' in self.alignertype:
				self.alignertype = 'giza'
			elif 'recaser' in self.alignertype:
				self.alignertype = 'recaser'


			else:
				self.errors.append([__name__,'invalid','\"[%s]\" alignertype=%s, but must be set to \'giza\', or \'recaser\''%(__name__,cfg['alignertype'])])
				logger.error('%s\t%s',*self.errors[-1][1:])
			self.tmgrams = str(int(cfg['tmgrams'])) if cf.isinteger(cfg['tmgrams']) else self.tmgrams

		# type-specific options
		if self.lmtype in ['irstlm']:

			# find programs
			self.cat = self.p.decodepath('4,cat',True)
			if not self.cat:
				self.errors.append([__name__,'missing','\"[%s]\" \"%s\" program not found'%(__name__,'cat')])
				logger.error('%s\t%s',*self.errors[-1][1:])

			self.gzip = self.p.decodepath('4,gzip',True)
			if not self.gzip:
				self.errors.append([__name__,'missing','\"[%s]\" \"%s\" program not found'%(__name__,'gzip')])
				logger.error('%s\t%s',*self.errors[-1][1:])

			self.gunzip = self.p.decodepath('4,gunzip',True)
			if not self.gunzip:
				self.errors.append([__name__,'missing','\"[%s]\" \"%s\" program not found'%(__name__,'gunzip')])
				logger.error('%s\t%s',*self.errors[-1][1:])

			self.perl = self.p.decodepath(cfg['perl'],True)
			if not self.perl:
				self.errors.append([__name__,'missing','\"[%s]\" \"%s\" program not found'%(__name__,cfg['perl'])])
				logger.error('%s\t%s',*self.errors[-1][1:])

			# test scripts installed
			for prog in self.progs:
				self.subprocs[prog] = {}
				self.subprocs[prog]['popen'] = None
				self.subprocs[prog]['kw'] = dict(self.kw)
				self.subprocs[prog]['path'] = self.p.decodepath(cfg[prog],True)
				if not self.subprocs[prog]['path']:
					self.errors.append([__name__,'missing','\"[%s]\" \"%s\" program not found'%(__name__,cfg[prog])])
					logger.error('%s\t%s',*self.errors[-1][1:])

		elif self.lmtype in ['srilm']:

			# find program
			self.path_ngram_count = self.p.decodepath('4,ngram-count',True)
			if not self.path_ngram_count:
				self.errors.append([__name__,'missing','\"[%s]\" \"%s\" program not found'%(__name__,'4,ngram-count')])
				logger.error('%s\t%s',*self.errors[-1][1:])

		elif self.lmtype in ['randlm']:

			# find program
			self.path_buildlm = self.p.decodepath('4,buildlm',True)
			if not self.path_buildlm:
				self.errors.append([__name__,'missing','\"[%s]\" \"%s\" program not found'%(__name__,'4,buildlm')])
				logger.error('%s\t%s',*self.errors[-1][1:])

		# IRSTLM
		self.parts = cfg['parts'] if cf.isinteger(cfg['parts']) else 1
		self.dictionary = os.path.abspath(os.path.expanduser(cfg['dictionary'])) if cfg['dictionary'] else ''
		if cfg['smoothing'].lower() in ('kneser-ney','improved-kneser-ney','witten-bell'):
			self.smoothing = cfg['smoothing'].lower()
		else:
			self.smoothing = 'witten-bell'
		self.boundaries = cfg['boundaries']
		self.prune = cfg['prune']
		self.uniform = cfg['uniform']
		self.verbose = self.p.verbose >= logging.INFO

		# SRILM
		if cfg['discount'].lower().strip('-') in ('kndiscount','wbdiscount'):
			self.discount = '-'+cfg['discount'].lower().strip('-')
		else:
			self.discount = '-kndiscount'
		self.boundaries = cfg['boundaries']
		self.interpolate = cfg['interpolate']
		self.memuse = cfg['memuse']

		# RANDLM
		self.lmstructure = cfg['lmstructure'] if cfg['lmstructure'] else self.lmstructure
		if not self.lmstructure in ('BloomMap','LogFreqBloomFilter'): self.lmstructure = 'BloomMap'
		self.falsepos = cfg['falsepos']
		self.values = cfg['values']









		# get input and output and file names
		inputfile = cfg['inputfile'].replace('%(rootfolder)s',self.p.rootfolder) if cfg['inputfile'] else os.sep.join([self.p.rootfolder,'BUILDS','lm',self.lmbuildname,os.extsep.join([self.lmroot,self.e])])
		self.inputs.append(inputfile)

		if self.recaser:
			lm_dir = os.sep.join([
				self.p.rootfolder,
				'ENGINES',
				'recasers',
				'%s-t=%s-l=%s-T=%s-n=%s-a=%s-g=%s'%('recaser',self.e,self.lmbuildname,self.lmtype,self.lmgrams,self.alignertype,self.tmgrams),
				])
		else:
			lm_dir = os.sep.join([
				self.p.rootfolder,
				'ENGINES',
				'lms',
				'%s-t=%s-l=%s-T=%s-n=%s'%('lm',self.e,self.lmbuildname,self.lmtype.replace('irstlmken','irstlm').replace('srilmken','srilm'),self.lmgrams,),
				])
		baseroot = '%s.%s.%s'%(self.lmtype.replace('irstlmken','irstlm').replace('srilmken','srilm'),self.e,self.lmstructure if self.lmtype == 'randlm' else 'gz')

		self.outputs.append(os.sep.join([lm_dir,baseroot]))
		if self.lmtype in ['irstlm','irstlmken']:
			self.outputs.append(self.outputs[0].replace('%s.%s.gz'%(self.lmtype.replace('irstlmken','irstlm'),self.e),'irstlm_arpa.%s.gz'%(self.e)))
			self.outputs.append(self.outputs[0].replace('%s.%s.gz'%(self.lmtype.replace('irstlmken','irstlm'),self.e),'irstlm.%s.mm'%(self.e)))





























	def run(self,k):
		global skipclose
		skipclose = False

	def flush(self,k):
		return

	def close(self):
		if skipclose: return

		lockfile = os.sep.join(['','var','tmp','domy','train.lock'])
		if not os.path.exists(os.path.dirname(lockfile)): os.makedirs(os.path.dirname(lockfile))
		lock = flock(lockfile,False).acquire()
		if not lock or not lock.ownlock():
			self.errors.append([__name__,'failed','train lock, another training session is running'])
			logger.error('%s\t%s',*self.errors[-1][1:])
			return
		with open(lockfile) as f:
			lockcontent = f.readline().rstrip()


		if self.recaser:
			if not os.path.exists(self.inputs[0]):
				self.errors.append([__name__,'missing','[%s] %s'%(__name__,self.inputs[0])])
				logger.error('%s\t%s',*self.errors[-1][1:])
				return

		else:

			# test input and output files
			infodata = cf.updatefingerprint(self.inputs,self.outputs,self.forceinfoupdate)


			if 'bypass' in infodata:
				# output file(s) exists.
				logger.info('%s\t%s',*['skipping','outputs exist and verified'])
				sys.stderr.write('%s - Skipping. [%s] Outputs exist and verified\n'%(cf.timestamp(),__name__))
				lock.release()
				return
			elif 'errors' in infodata:
				# report/return self.errors
				for error in infodata['errors']:
					self.errors.append([__name__,'error',error.strip()])
					logger.error('%s\t%s',*self.errors[-1][1:])
				lock.release()
				return
			sys.stderr.flush()

		logger.info('%s\t%s',*['start','%s'%(self.lmtype.upper())])
		sys.stderr.write('\nStart -%s %s [%s]\n'%(' (recaser)' if self.recaser else '',self.lmtype.upper(),__name__))

		# make tmpdir
		try:
			os.makedirs(self.tmpdir)
		except OSError,e:
			if e.errno == 17:
				self.deltemp = False
			else:
				logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
				raise OSError(e)

		# test if lm corpus has <s> & </s> tags & if not, add
		inp = _teststartend(self.inputs[0],self.encoding,self.tmpdir)

		if self.lmtype in ['irstlm']: self.build_lm(inp)

		elif self.lmtype in ['srilm']: self.ngram_count(inp)

		elif self.lmtype in ['randlm']: self.buildlm(inp)

		if not self.inputs[0] == inp: os.unlink(inp) # delete temporary <s> file



		if self.deltemp: shutil.rmtree(self.tmpdir)

		if self.recaser:
			for j,fname in enumerate(self.outputs):
				if not os.path.exists(fname):
					self.errors.append([__name__,'missing','[%s] %s'%(__name__,fname)])
					logger.error('%s\t%s',*self.errors[-1][1:])
		elif self.errors:
			self.errors.append([__name__,'failed','%s'%(self.lmtype.upper())])
			logger.error('%s\t%s',*self.errors[-1][1:])
		else:
			# update infocontent and save
			if self.lmtype in ['irstlm']:
				infodata['misc'] = {}
				infodata['misc']['IRSTLM - boundaries'] = self.boundaries
				infodata['misc']['IRSTLM - dictionary'] = self.dictionary if self.dictionary else False
				infodata['misc']['IRSTLM - order'] = self.lmgrams
				infodata['misc']['IRSTLM - parts'] = self.parts
				infodata['misc']['IRSTLM - prune'] = self.prune
				infodata['misc']['IRSTLM - smoothing'] = self.smoothing
				infodata['misc']['IRSTLM - uniform'] = self.uniform
			elif self.lmtype in ['srilm']:
				infodata['misc'] = {}
				infodata['misc']['order'] = self.lmgrams
				infodata['misc']['interpolate'] = self.interpolate
				infodata['misc']['memuse'] = self.memuse
				infodata['misc']['boundaries'] = self.boundaries
			elif self.lmtype in ['randlm']:
				infodata['misc'] = {}
				infodata['misc']['order'] = self.lmgrams
				infodata['misc']['struct'] = self.lmstructure
				infodata['misc']['falsepos'] = self.falsepos
				infodata['misc']['values'] = self.values








			infodata = cf.updatefingerprint([],self.outputs,True,infodata)
			if 'errors' in infodata:
				# report/return self.errors
				for error in infodata['errors']:
					self.errors.append([__name__,'error',error.strip()])
					logger.error('%s\t%s',*self.errors[-1][1:])
			else:
				logger.info('%s\t%s',*['success','%s'%(self.lmtype.upper())])
				sys.stderr.write('Success -%s %s\n'%(' (recaser)' if self.recaser else '',self.lmtype.upper()))

		sys.stderr.flush()
		lock.release()

	def build_lm(self,inp):

		# calculate split parts based on memory size or user override
		self.parts = self.parts if self.parts else (os.path.getsize(self.inputs[0]) / (cf.getfreeram()/(4*cf.cpu_count()))) + 1

		# build corpus dictionary
		self.subprocs['dict']['msg'] = 'dict - Extracting dictionary from training corpus'
		self.subprocs['dict']['kw']['executable'] = self.subprocs['dict']['path']
		self.subprocs['dict']['args'] = [
			'',
			'-sort=%s'%('no'),
			'-f=%s'%('n' if self.uniform else 'y'),
			'-i=%s'%(inp),
			'-o=%s%sdictionary'%(self.tmpdir,os.sep),
			]
		returncode = self.__runcmd('dict',self.subprocs['dict']['args'])
		if not returncode == 0 or not os.path.exists('%s%sdictionary'%(self.tmpdir,os.sep)):
			self.errors.append([__name__,'failed',self.subprocs['dict']['msg']])
			logger.error('%s\t%s',*self.errors[-1][1:])
			return

		# split corpus dictionary
		self.subprocs['split-dict.pl']['msg'] = 'split-dict.pl - Splitting dictionary'
		self.subprocs['split-dict.pl']['kw']['executable'] = self.perl
		self.subprocs['split-dict.pl']['args'] = [
			'',
			self.subprocs['split-dict.pl']['path'],
			'--input', '%s%sdictionary'%(self.tmpdir,os.sep),
			'--output','%s%sdict.'%(self.tmpdir,os.sep),
			'--parts', '%s'%(self.parts),
			]
		returncode = self.__runcmd('split-dict.pl',self.subprocs['split-dict.pl']['args'])
		if os.path.exists('%s%sdictionary'%(self.tmpdir,os.sep)): os.unlink('%s%sdictionary'%(self.tmpdir,os.sep))
		if not returncode == 0 or not self.parts == len(glob(os.sep.join([self.tmpdir,'dict.???']))):
			self.errors.append([__name__,'failed',self.subprocs['dict']['msg']])
			logger.error('%s\t%s',*self.errors[-1][1:])
			return

		# prepare ngt for loop, add custom dictionary
		self.subprocs['ngt']['kw']['executable'] = self.subprocs['ngt']['path']
		self.subprocs['ngt']['args'] = [
			'',
			'-i=%s'%(inp),
			'-n=%d'%(self.lmgrams),
			'-gooout=%s'%('y'),
			]
		if self.dictionary: self.subprocs['ngt']['args'].append('%s'%(self.dictionary))

		# prepare build-sublm.pl for loop
		self.subprocs['build-sublm.pl']['kw']['executable'] = self.perl
		self.subprocs['build-sublm.pl']['args'] = [
			'',
			self.subprocs['build-sublm.pl']['path'],
			'--size', self.lmgrams,
			]
		if self.prune: self.subprocs['build-sublm.pl']['args'].append('--prune-singletons')
		if self.boundaries: self.subprocs['build-sublm.pl']['args'].append('--cross-sentence')
		if self.verbose: self.subprocs['build-sublm.pl']['args'].append('--verbose')

		# run ngt and build-sublm.pl per loop
		for i in range(self.parts):

			self.subprocs['ngt']['msg'] = 'ngt - Extracting n-gram statistics for word list #%d'%(i)
			args = list(self.subprocs['ngt']['args'])
			args.append('-fd=%s%sdict.%03d'%(self.tmpdir,os.sep,i))
			args.append('-iknstat=%s%siknstat.%03d'%(self.tmpdir,os.sep,i))
			args.append('-o=%s%sngram.%03d'%(self.tmpdir,os.sep,i))
			returncode = self.__runcmd('ngt',args)
			if os.path.exists('%s%sdict.%03d'%(self.tmpdir,os.sep,i)): os.unlink('%s%sdict.%03d'%(self.tmpdir,os.sep,i))
			if not returncode == 0 or not os.path.exists('%s%sngram.%03d'%(self.tmpdir,os.sep,i)):
				self.errors.append([__name__,'failed',self.subprocs['dict']['msg']])
				logger.error('%s\t%s',*self.errors[-1][1:])
				return

			self.subprocs['build-sublm.pl']['msg'] = 'build-sublm.pl - Estimating language models for word list #%d'%(i)
			args = list(self.subprocs['build-sublm.pl']['args'])
			args.extend(['--ngrams','%s%sngram.%03d'%(self.tmpdir,os.sep,i)])
			args.extend(['--sublm','%s%ssublm.%03d'%(self.tmpdir,os.sep,i)])
			args.extend(['--%s'%(self.smoothing),'cat %s%siknstat.%03d'%(self.tmpdir,os.sep,i) if self.smoothing in ('kneser-ney','improved-kneser-ney') else '',])
			returncode = self.__runcmd('build-sublm.pl',args)
			if os.path.exists('%s%siknstat.%03d'%(self.tmpdir,os.sep,i)): os.unlink('%s%siknstat.%03d'%(self.tmpdir,os.sep,i))
			if os.path.exists('%s%sngram.%03d'%(self.tmpdir,os.sep,i)): os.unlink('%s%sngram.%03d'%(self.tmpdir,os.sep,i))
			if not returncode == 0 or not self.lmgrams == len(glob(os.sep.join([self.tmpdir,'sublm.%03d.*gr.gz'%(i)]))):
				self.errors.append([__name__,'failed',self.subprocs['dict']['msg']])
				logger.error('%s\t%s',*self.errors[-1][1:])
				return

		# run merge-sublm.pl to merge sub-lm's created in loop
		try:
			os.makedirs(os.path.dirname(self.outputs[0]))
		except OSError,e:
			if not e.errno == 17:
				logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
				raise OSError(e)

		self.subprocs['merge-sublm.pl']['msg'] = 'merge-sublm.pl - Merging language model splits'
		self.subprocs['merge-sublm.pl']['kw']['executable'] = self.perl
		self.subprocs['merge-sublm.pl']['args'] = [
			'',
			self.subprocs['merge-sublm.pl']['path'],
			'--size',self.lmgrams,
			'--sublm','%s%ssublm.'%(self.tmpdir,os.sep),
			'--lm', self.outputs[0],
			]
		returncode = self.__runcmd('merge-sublm.pl',self.subprocs['merge-sublm.pl']['args'])
		for i in range(self.parts):
			for tmpfile in glob(os.sep.join([self.tmpdir,'sublm.%03d.*gr.gz'%(i)])):
				os.unlink(tmpfile)
		if not returncode == 0 or not os.path.exists(self.outputs[0]):
			self.errors.append([__name__,'failed',self.subprocs['dict']['msg']])
			logger.error('%s\t%s',*self.errors[-1][1:])
			return

		# create ARPA format language model file
		self.subprocs['compile-lm']['msg'] = 'compile-lm - Compiling language model to ARPA'
		self.subprocs['compile-lm']['kw']['executable'] = self.subprocs['compile-lm']['path']
		self.subprocs['compile-lm']['args'] = [
			'',
			'--text','yes',
			self.outputs[0],
			self.outputs[1].replace('irstlm_arpa.%s.gz'%(self.e),'irstlm_arpa.%s'%(self.e)),
			]
		returncode = self.__runcmd('compile-lm',self.subprocs['compile-lm']['args'])
		for tmpfile in glob(os.sep.join(['','tmp','dskbuff--*'])):
			os.unlink(tmpfile)
		if not returncode == 0 or not os.path.exists(self.outputs[1].replace('irstlm_arpa.%s.gz'%(self.e),'irstlm_arpa.%s'%(self.e))):
			self.errors.append([__name__,'failed',self.subprocs['dict']['msg']])
			logger.error('%s\t%s',*self.errors[-1][1:])
			return

		sys.stderr.write('    (%s %s) compress IRST - arpa\n'%(self.e,'recaser' if self.recaser else 'lm'))
		logger.info('%s\t%s',*['run','compress IRST - arpa'])
		_compressarpa(self.outputs[1].replace('irstlm_arpa.%s.gz'%(self.e),'irstlm_arpa.%s'%(self.e)),self.outputs[1])
		if not os.path.exists(self.outputs[1]):
			self.errors.append([__name__,'failed',self.subprocs['dict']['msg']])
			logger.error('%s\t%s',*self.errors[-1][1:])
			return

		# create IRSTLM binary format language model file
		self.subprocs['compile-lm']['msg'] = 'compile-lm - Compiling language model to binary'
		self.subprocs['compile-lm']['kw']['executable'] = self.subprocs['compile-lm']['path']
		self.subprocs['compile-lm']['args'] = [
			'',
			'--text','no',
			self.outputs[0],
			self.outputs[2],
			]
		returncode = self.__runcmd('compile-lm',self.subprocs['compile-lm']['args'])
		for tmpfile in glob(os.sep.join(['','tmp','dskbuff--*'])):
			os.unlink(tmpfile)
		if not returncode == 0 or not os.path.exists(self.outputs[2]):
			self.errors.append([__name__,'failed',self.subprocs['dict']['msg']])
			logger.error('%s\t%s',*self.errors[-1][1:])
			return

	def __runcmd(self,prog,args):

		args = [str(arg) for arg in args]

		try:
			self.subprocs[prog]['popen'] = subprocess.Popen(args,**self.subprocs[prog]['kw'])
		except Exception,e:
			self.errors.append([__name__,'failed','%s %s - %s'%(os.path.basename(self.subprocs[prog]['kw']['executable'],' '.join(args)),str(e))])
			logger.error('%s\t%s',*self.errors[-1][1:])
			return

		logger.info('%s\t%d - %s %s',*['run',self.subprocs[prog]['popen'].pid,os.path.basename(self.subprocs[prog]['kw']['executable']),' '.join(args)])
		sys.stderr.write('run -%s %s\n'%(' (recaser)' if self.recaser else '',self.subprocs[prog]['msg']))

		try:
			for line in iter(self.subprocs[prog]['popen'].stdout.readline, ''):
				if '' in line or not line.strip(): continue
				try:
					logger.info('%s\t%s',*[self.subprocs[prog]['popen'].pid,line.rstrip().encode(self.encoding),])
					if not 'Comput' in line and not ' n:' in line and not ' n1:' in line and not 'level' in line and not 'Level' in line and not '-grams' in line: continue
					sys.stderr.write('    (%s %s) %s\n'%(self.e,'recaser' if self.recaser else 'lm',line.rstrip().encode(self.encoding)))
				except KeyboardInterrupt:
					raise KeyboardInterrupt()
				except:
					pass
		except KeyboardInterrupt:
			# trap KeyboardInterrupt to delete dskbuff temp files
			for tmpfile in glob(os.sep.join(['','tmp','dskbuff--*'])):
				os.unlink(tmpfile)
			for i in range(self.parts):
				for tmpfile in glob(os.sep.join([self.tmpdir,'sublm.%03d.*gr.gz'%(i)])):
					os.unlink(tmpfile)
			while self.subprocs[prog]['popen'].poll() is None:
				self.subprocs[prog]['popen'].terminate()
				time.sleep(0.2)
			raise KeyboardInterrupt()

		while self.subprocs[prog]['popen'].poll() is None:
			time.sleep(0.2)

#		self.status.append([__name__,'complete','%s %s - %s'%(os.path.basename(self.subprocs[prog]['kw']['executable']),' '.join(args),self.subprocs[prog]['popen'].returncode)])
		logger.info('%s\t%s',*['complete','%s %s - %s'%(os.path.basename(self.subprocs[prog]['kw']['executable']),' '.join(args),self.subprocs[prog]['popen'].returncode)])

		return self.subprocs[prog]['popen'].returncode

	def ngram_count(self,inp):

		# SRILM output folder
		try:
			os.makedirs(os.path.dirname(self.outputs[0]))
		except OSError,e:
			if not e.errno == 17:
				logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
				raise OSError(e)

		# create command line
		args = [
			'',
			'-text',inp,
			'-lm',self.outputs[0],
			'-order',self.lmgrams,
			'-debug', '1',
			]
		if self.interpolate: args.append('-interpolate')
		if self.memuse: args.append('-memuse')
		if not self.boundaries: args.extend(['-no-sos','-no-eos'])
		args = [str(item) for item in args]

		self.kw['executable'] = self.path_ngram_count

		# run subprocess
		try:
			subproc = subprocess.Popen(args,**self.kw)
		except Exception,e:
			self.errors.append([__name__,'failed','%s %s - %s'%(os.path.basename(self.kw['executable']),' '.join(args),str(e))])
			logger.error('%s\t%s',*self.errors[-1][1:])
			lock.release()
			return

		logger.info('%s\t%d - %s %s',*['run',subproc.pid,os.path.basename(self.kw['executable']),' '.join(args)])
		sys.stderr.write('run -%s ngram-count - %s\n'%(' (recaser)' if self.recaser else '',self.lmtype.upper()))

		for line in iter(subproc.stdout.readline, ''):
			if not line.rstrip(): continue
			try:
				logger.info('%s\t%s',*[subproc.pid,line.rstrip()])
				if line.startswith('warning: '): continue
				sys.stderr.write('    (%s %s) %s\n'%(self.e,'recaser' if self.recaser else 'lm',line.rstrip().encode(self.encoding)))
			except KeyboardInterrupt:
				raise KeyboardInterrupt()
			except:
				pass

		while subproc.poll() is None:
			time.sleep(0.2)

	def buildlm(self,inp):

		# RANDLM output folder
		try:
			os.makedirs(os.path.dirname(self.outputs[0]))
		except OSError,e:
			if not e.errno == 17:
				logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
				raise OSError(e)

		# create command line
		args = [
			'',
			'-order',self.lmgrams,
			'-struct',self.lmstructure,
			'-falsepos',self.falsepos,
			'-values',self.values,
			'-tmp-dir',self.tmpdir,
			'-input-path',inp,
			'-output-dir',os.path.dirname(self.outputs[0]),
			'-output-prefix', os.path.basename(self.outputs[0]).replace(os.extsep+self.lmstructure,''),
			]
		args = [str(item) for item in args]

		self.kw['executable'] = self.path_buildlm

		# run subprocess
		try:
			subproc = subprocess.Popen(args,**self.kw)
		except Exception,e:
			self.errors.append([__name__,'failed','%s %s - %s'%(os.path.basename(self.kw['executable']),' '.join(args),str(e))])
			logger.error('%s\t%s',*self.errors[-1][1:])
			lock.release()
			return

		logger.info('%s\t%d - %s %s',*['run',subproc.pid,os.path.basename(self.kw['executable']),' '.join(args)])
		sys.stderr.write('run -%s buildlm - %s\n'%(' (recaser)' if self.recaser else '',self.lmtype.upper()))

		for line in iter(subproc.stdout.readline, ''):
			if not line.rstrip(): continue
			try:
				logger.info('%s\t%s',*[subproc.pid,line.rstrip()])
				sys.stderr.write('    (%s %s) %s\n'%(self.e,'recaser' if self.recaser else 'lm',line.rstrip().encode(self.encoding)))
			except KeyboardInterrupt:
				raise KeyboardInterrupt()
			except:
				pass

		while subproc.poll() is None:
			time.sleep(0.2)






















































def _compressarpa(inp,out):
	import gzip
	f_out = gzip.open(out,'wb')
	f_in = open(inp,'rb')
	try:
		while True:
			data = f_in.read(32768)
			if not data: break
			f_out.write(data)
	except:
		shutil.move(inp,out)
	else:
		f_out.close()
		f_in.close()
		os.unlink(inp)

def _teststartend(filename,encoding,tempdir):
	if os.read(os.open(filename, os.O_RDONLY), 4) == '<s> ': return filename
	import codecs
	tmpfile = os.sep.join([tempdir,os.path.basename(filename)])
	g = codecs.open(tmpfile,'w',encoding)
	with codecs.open(filename,'r',encoding) as f:
		for line in f:
			g.write('<s> %s </s>\x0a'%(line.rstrip('\r\n')))
	g.close()
	return tmpfile

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import filter\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0]
	)

licensetxt=u'''DoMY™ CE v2.5
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stdout.write(usage().encode('utf8')+'\n')
