#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#2.5.321 - bugfixes in path calculations for self.recaser

import os
import sys
import subprocess
import time
import shutil
import logging
import common as cf
from flock import flock
from glob import glob
from progress_bar import ProgressBar

logger = logging.getLogger('.'.join([os.path.splitext(os.path.basename(sys.argv[0]))[0],'manager','filtergraph',__name__]))
skipclose = True
lmbytype = {
	'srilm': '0',
	'irstlm': '1',
	'randlm': '5',


	}

class filter(object):

	cfg = {
		'args': {},
		'exeoptions': '-w',
		'kw': {
			'bufsize': 0,
			'executable': '4,perl',
			'stdin': 'PIPE',
			'stdout': 'PIPE',
			'stderr': 'PIPE',
			'preexec_fn': None,
			'close_fds': False,
			'shell': False,
			'cwd': None,
			'env': None,
			'universal_newlines': False,
			'startupinfo': None,
			'creationflags': 0,
			},
		'script': '4,mert-moses.pl',
		'encoding': 'utf8',
		'recaser': '%(recaser)s',
		'forceinfoupdate': '%(forceinfoupdate)s',
		'hierarchical': '%(hierarchical)s',
		'lmlang': '',
		'f': '',
		'e': '',
		'tmbuildname': '',
		'alignertype': 'giza',
		'tmgrams': '7',
		'lmbuildname': '',
		'lmtype': 'irstlm',
		'lmgrams': '5',
		'lmstructure': 'BloomMap',
		'lmodel-files': '',
		'version': '2.5.321',

		'mertroot': '%(mertroot)s',

		'working-dir': '',					# output where all the files are created
		'input': '',						# REQUIRED, input text to decode (dev set, mert set)
		'refs': '',							# REQUIRED, basename of files with references (dev set, mert set)
		'decoder': '4,moses',				# REQUIRED, pathname to the decoder executable
		'config': '',						# REQUIRED, pathname to startup ini file
		'inputtype': 0,						# Handle different input types: (0 for text, 1 for confusion network, 2 for lattices, default is 0)
		'nbest': 100,						# how big nbestlist to generate
		'lattice-samples': 0,				# how many lattice samples (Chatterjee & Cancedda, emnlp 2010)
		'queue-flags': '-hard',				# extra parameters for parallelizer, anything you with to pass to qsub. The default '-hard'. To reset the parameters, please use --queue-flags=' '
		'jobs': 0,							# if parallel, number of jobs to use (undef or 0 -> serial)
		'decoder-flags': '',				# additional parameters to pass to the decoder
		'continue': False,					# try to continue from the last saved run?
		'skip-decoder': False,				# skip the decoder run for the first time, assuming that we got interrupted during optimization
		'shortest': True,					# Use "--shortest" to use shortest reference length when computing BLEU score
		'average': False,					# Use "--average" to use average reference length when computing BLEU score
		'closest': False,					# Use "--closest" to use closest reference length when computing BLEU score
		'nocase': False,					# Use "--nocase" to compute case-insensitive scores
		'nonorm': False,					# Use "--nonorm" to non normalize translation before computing scores
		'help': False,
		'verbose': False,
		'mertdir': '4,mert',				# REQUIRED, path to new mert directory
		'mertargs': '',						# args to pass through to mert & extractor
		'mertmertargs': '',					# args to pass through to mert only
		'filtercmd': '',					# path to filter-model-given-input.pl, allow to override the default location
		'filterfile': '',					# path to alternative to input-text for filtering model. useful for lattice decoding
		'qsubwrapper': '',					# allow to override the default qsub-wrapper.pl script
		'mosesparallelcmd': '',				# allow to override the default location, use a different script instead of moses-parallel
		'old-sge': False,					# passed to moses-parallel parallelizers, assume Grid Engine < 6.0
		'filter-phrase-table': True,		# filter phrase table, (dis)allow of phrase tables
		'no-filter-phrase-table': False,	# disallow filtering of phrase tables (useful if binary phrase tables are available)
		'predictable-seeds': False,			# make random restarts deterministic
		'historic-bests': False,			# use best settings from all previous iterations as starting points [Foster&Kuhn,2009]
		'random-directions': False,			# search only in random directions
		'number-of-random-directions': 0,	# number of random directions (50), also works with default optimizer [Cer&al.,2008]
		'random-restarts': 20,				# number of random restarts
		'activate-features': '',			# comma-separated (or blank-separated) list of features to work on. if undef work on all features (others are fixed to the starting values)
		'range': 'tm:0..1,-1..1',			# specify min and max value for some features
											# --range can be repeated as needed.
											# The order of the various --range specifications
											# is important only within a feature name.
											# E.g.:
											# --range=tm:0..1,-1..1 --range=tm:0..2
											# is identical to:
											# --range=tm:0..1,-1..1,0..2
											# but not to:
											# --range=tm:0..2 --range=tm:0..1,-1..1 
		'prev-aggregate-nbestlist': -1,		# number of previous run to consider when loading data (default =-1, i.e. all previous)
											# -1 means all previous, i.e. from iteration 1
											# 0 means no previous data, i.e. from actual iteration
											# 1 means 1 previous data , i.e. from the actual iteration and from the previous one
											# and so on
		'maximum-iterations': '%(maximum-iterations)s',		# Maximum number of iterations. Default: 25
		'pairwise-ranked': False,			# use Hopkins&May[2011]
		'pro-starting-point': False,		# get a starting point from pairwise ranked optimizer
		'historic-interpolation': False,	# Interpolate optimized weights with prior iterations' weight (parameter sets factor [0;1] given to current weights)
#		'sc-config': 'METRIC1:WEIGHT1,METRIC2:WEIGHT2', # extra option to specify tuning with multiple metrics.
		'threads': 0,						# Use multi-threaded moses and mert.
		}
	args = []
	script = '4,mert-moses.pl'
	kw = {}
	subproc = {}
	encoding = 'utf8'
	recaser = False
	forceinfoupdate = False
	hierarchical = False
	lmlang = ''
	f = ''
	e = ''
	tmbuildname = ''
	alignertype = 'giza'
	tmgrams = '7'
	lmbuildname = ''
	lmstructure = ''
	lmgrams = '5'
	lmtype = 'irstlm'
	lmcode = '1'
	mertroot = 'no_mert'
	threads = 0			# Use multi-threaded moses
	inputs = []
	outputs = []
	lmodel_files = []
	mosesconfig = ''
	config = {
		'working-dir': '',
		'input': '',
		'inputtype': 0,
		'refs': '',
		'decoder': '4,moses',
		'config': '',
		'nbest': 100,
		'lattice-samples': 0,
		'queue-flags': '-hard',
		'jobs': 0,
		'decoder-flags': '',
		'continue': False,
		'skip-decoder': False,
		'shortest': True,
		'average': False,
		'closest': False,
		'nocase': False,
		'nonorm': False,
		'help': False,
		'verbose': False,
		'mertdir': '4,mert',
		'mertargs': '',
		'mertmertargs': '',
		'filtercmd': '',
		'filterfile': '',
		'qsubwrapper': '',
		'mosesparallelcmd': '',
		'old-sge': False,
		'filter-phrase-table': False,
		'no-filter-phrase-table': True,
		'predictable-seeds': False,
		'historic-bests': False,
		'random-directions': False,
		'number-of-random-directions': 0,
		'random-restarts': 20,
		'activate-features': '',
		'range': 'tm:0..1,-1..1',
		'prev-aggregate-nbestlist': -1,
		'maximum-iterations': 25,
		'pairwise-ranked': False,
		'pro-starting-point': False,
		'historic-interpolation': False,
#		'sc-config': '\"METRIC1:WEIGHT1,METRIC2:WEIGHT2\"',
		'threads': 1,
		}
	isopen = False
	p = object
	errors = []

	def open(self,parent,cfg):
		self.args = dict(cfg['args'])
		self.kw = dict(cfg['kw'])
		if cfg['script']:
			self.script = self.p.decodepath(','.join([__name__.split(',')[1]]+cfg['script'].split(',')),True)
			if not self.script:
				self.errors.append([__name__,'missing',self.script])
				logger.warn('%s\t%s',*self.errors[-1][1:])
			self.args[0] = self.script
			self.kw['cwd'] = os.path.dirname(self.script)
		else:
			self.script = None
		# convert args dict to args list
		self.args = [str(self.args[k]) for k in sorted(self.args.keys())]
		if cfg['kw']['executable']:
			self.kw['executable'] = self.p.decodepath(','.join([__name__.split(',')[1]]+cfg['kw']['executable'].split(',')),True)
			if not self.kw['executable']:
				self.errors.append([__name__,'missing',self.kw['executable']])
				logger.warn('%s\t%s',*self.errors[-1][1:])
			self.args.insert(0,cfg['exeoptions'] if cfg['exeoptions'] else '')
		else:
			self.kw['executable'] = None

		self.kw['stdin'] = subprocess.PIPE if cfg['kw']['stdin'] == 'PIPE' else cfg['kw']['stdin']
		self.kw['stdout'] = subprocess.PIPE if cfg['kw']['stdout'] == 'PIPE' else cfg['kw']['stdout']
		self.kw['stderr'] = subprocess.STDOUT if cfg['kw']['stderr'] == 'PIPE' else cfg['kw']['stderr']
		self.encoding = 'utf8' if 'utf8' in cfg['encoding'].lower().replace('-','') else cfg['encoding']

		# other values
		self.recaser = self.recaser if not cfg['recaser'] or (cfg['recaser'].startswith('%(') and cfg['recaser'].endswith(')s')) else cf.zbool(cfg['recaser'])
		self.forceinfoupdate = self.forceinfoupdate if not cfg['forceinfoupdate'] or (cfg['forceinfoupdate'].startswith('%(') and cfg['forceinfoupdate'].endswith(')s')) else cf.zbool(cfg['forceinfoupdate'])
		self.hierarchical = self.hierarchical if not cfg['hierarchical'] or (cfg['hierarchical'].startswith('%(') and cfg['hierarchical'].endswith(')s')) else cf.zbool(cfg['hierarchical'])

		# Languages
		self.e = cfg['e'] if cfg['e'] and not cfg['e'].startswith('%(') and not cfg['e'].endswith(')s') else ''
		self.f = cfg['f'] if cfg['f'] and not cfg['f'].startswith('%(') and not cfg['f'].endswith(')s') else ''
		# LM values
		self.lmbuildname = cfg['lmbuildname']
		self.lmtype = cfg['lmtype'] if cfg['lmtype'] and not cfg['lmtype'].startswith('%(') and not cfg['lmtype'].endswith(')s') else self.lmtype
		if not self.lmtype in lmbytype.keys():
			self.errors.append([__name__,'invalid','\"[%s]\" lmtype=%s must be set to \'irstlmken\', \'irstlm\', \'randlm\', \'srilm\', \'srilmken\''%(__name__,self.lmtype)])
			logger.error('%s\t%s',*self.errors[-1][1:])
		self.lmgrams = str(int(cfg['lmgrams'])) if cf.isinteger(cfg['lmgrams']) else self.lmgrams
		self.lmstructure = cfg['lmstructure'] if cfg['lmstructure'] in ['BloomMap','LogFreqBloomFilter'] else 'BloomMap'
		# TM values
		self.tmbuildname = cfg['tmbuildname'] if cfg['tmbuildname'] and not cfg['tmbuildname'].startswith('%(') and not cfg['tmbuildname'].endswith(')s') else ''
		self.alignertype = cfg['alignertype'].lower() if cfg['alignertype'] and not cfg['alignertype'].startswith('%(') and not cfg['alignertype'].endswith(')s') else self.alignertype
		if 'giza' in self.alignertype:
			self.alignertype = 'giza'
		elif 'recaser' in self.alignertype:
			self.alignertype = 'recaser'
		elif 'berk' in self.alignertype:
			self.alignertype = 'berk'
		else:
			self.alignertype = ''
		self.tmgrams = str(int(cfg['tmgrams'])) if cf.isinteger(cfg['tmgrams']) else self.tmgrams
		self.mertroot = cfg['mertroot'] if cfg['mertroot'] and not cfg['mertroot'].startswith('%(') and not cfg['mertroot'].endswith(')s') else self.mertroot

		# inputs & refs
		if cfg['input']:
			self.input = cfg['input']
		else:
			if not self.f:
				self.errors.append([__name__,'missing','\"[%s]\" f= must be set for \"input\"'%(__name__,)])
				logger.error('%s\t%s',*self.errors[-1][1:])
			if self.recaser:
				if not self.lmbuildname:
					self.errors.append([__name__,'missing','\"[%s]\" lmbuildname= must be set for \"input\"'%(__name__,)])
					logger.error('%s\t%s',*self.errors[-1][1:])
				self.input = os.sep.join([self.p.rootfolder,'BUILDS','lm',self.lmbuildname,'%s.%s'%(self.mertroot,self.f)])
			else:
				if self.mertroot == 'no_mert':
					self.errors.append([__name__,'missing','\"[%s]\" mertroot= must be set'%(__name__,)])
					logger.error('%s\t%s',*self.errors[-1][1:])
				if not self.tmbuildname:
					self.errors.append([__name__,'missing','\"[%s]\" tmbuildname= must be set for \"input\"'%(__name__,)])
					logger.error('%s\t%s',*self.errors[-1][1:])
				self.input = os.sep.join([self.p.rootfolder,'BUILDS','tm',self.tmbuildname,'%s.%s'%(self.mertroot,self.f)])

		if cfg['refs']:
			self.refs = cfg['refs']
		else:
			if not self.e:
				self.errors.append([__name__,'missing','\"[%s]\" e= must be set for \"refs\"'%(__name__,)])
				logger.error('%s\t%s',*self.errors[-1][1:])
			if self.recaser:
				if not self.lmbuildname:
					self.errors.append([__name__,'missing','\"[%s]\" lmbuildname= must be set for \"refs\"'%(__name__,)])
					logger.error('%s\t%s',*self.errors[-1][1:])
				self.refs = os.sep.join([self.p.rootfolder,'BUILDS','lm',self.lmbuildname,'%s.%s'%(self.mertroot,self.e)])
			else:
				if self.mertroot == 'no_mert':
					self.errors.append([__name__,'missing','\"[%s]\" mertroot= must be set'%(__name__,)])
					logger.error('%s\t%s',*self.errors[-1][1:])
				if not self.tmbuildname:
					self.errors.append([__name__,'missing','\"[%s]\" tmbuildname= must be set for \"refs\"'%(__name__,)])
					logger.error('%s\t%s',*self.errors[-1][1:])
				self.refs = os.sep.join([self.p.rootfolder,'BUILDS','tm',self.tmbuildname,'%s.%s'%(self.mertroot,self.e)])

		# test if working-dir is valid path string
		if cfg['working-dir']:
			cfg['working-dir'] = cfg['working-dir']
			try:
				os.makedirs(os.sep.join(['','tmp','tst',cfg['working-dir']]))
			except OSError:
				self.errors.append([__name__,'invalid','\"[%s]\" working-dir=%s'%(__name__,cfg['working-dir'])])
				logger.error('%s\t%s',*self.errors[-1][1:])
				cfg['working-dir'] = ''
			else:
				shutil.rmtree(os.sep.join(['','tmp','tst']))
		else:
			if not self.e:
				self.errors.append([__name__,'missing','\"[%s]\" e= must be set for \"working-dir\"'%(__name__,)])
				logger.error('%s\t%s',*self.errors[-1][1:])
			if not self.alignertype:
				self.errors.append([__name__,'invalid','\"[%s]\" alignertype=%s, but must be set to \'giza\', \'berk\' or \'recaser\' for \"working-dir\"'%(__name__,cfg['alignertype'])])
				logger.error('%s\t%s',*self.errors[-1][1:])
			if not self.lmbuildname:
				self.errors.append([__name__,'missing','\"[%s]\" lmbuildname= must be set for \"working-dir\"'%(__name__,)])
				logger.error('%s\t%s',*self.errors[-1][1:])

			if self.recaser:
				cfg['working-dir'] = os.sep.join([self.p.rootfolder,'ENGINES','recasers','%s-t=%s-l=%s-T=%s-n=%s-a=%s-g=%s'%('recaser',self.e,self.lmbuildname,self.lmtype,self.lmgrams,self.alignertype,self.tmgrams,),'working',])
			else:
				if not self.f:
					self.errors.append([__name__,'missing','\"[%s]\" f= must be set for \"working-dir\"'%(__name__,)])
					logger.error('%s\t%s',*self.errors[-1][1:])
				if not self.tmbuildname:
					self.errors.append([__name__,'missing','\"[%s]\" tmbuildname= must be set for \"working-dir\"'%(__name__,)])
					logger.error('%s\t%s',*self.errors[-1][1:])
				cfg['working-dir'] = os.sep.join([self.p.rootfolder,'TRAININGS','merts','mert-%s-s=%s-t=%s-p=%s-a=%s-g=%s-l=%s-T=%s-n=%s'%(self.mertroot,self.f,self.e,self.tmbuildname,self.alignertype,self.tmgrams,self.lmbuildname,self.lmtype,self.lmgrams,)])

		if cfg['config'] and not os.path.exists(cfg['config']):
			# error, config defined but doesn't exist
			self.errors.append([__name__,'invalid','\"[%s]\" config=%s moses configuration file does not exist'%(__name__,cfg['config'])])
			logger.error('%s\t%s',*self.errors[-1][1:])
		elif cfg['config']:
			self.mosesconfig = cfg['config']
		else:
			self.mosesconfig = os.sep.join([cfg['working-dir'],'run0.moses.ini'])

		cfg['config'] = os.sep.join([cfg['working-dir'],'run0.moses.ini'])
		cfg['input'] = os.sep.join([cfg['working-dir'],os.path.basename(self.input)])
		cfg['refs'] = os.sep.join([cfg['working-dir'],os.path.basename(self.refs)])

		ABBR_FULL_MAP = (('d','weight-d'),('lm','weight-l'),('tm','weight-t'),('w','weight-w'),('g','weight-generation'),('lex','weight-lex'),('I','weight-i'))

		# copy config, verify values and test command line values
		for k in [k for k in self.config.keys() if k in cfg.keys()]:
			if type(cfg[k]) is str or type(cfg[k]) is unicode:
				self.config[k] = cfg[k]
			else:
				self.config[k] = cfg[k]

		# Set other mert-moses.pl settings
		if not self.config['inputtype'] in [0,1,2,]:
			self.errors.append([__name__,'invalid','\"[%s]\" inputtype= must be \'0\' for text, \'1\' for confusion network or \'2\' for lattices'%(__name__)])
			logger.error('%s\t%s',*self.errors[-1][1:])

		self.config['decoder'] = self.config['decoder'] if not self.config['decoder'] == self.cfg['decoder'] else self.p.decodepath(self.config['decoder'],True)
		if not self.config['decoder']:
			self.errors.append([__name__,'invalid','\"[%s]\" decoder= must be the full path to the moses binary'%(__name__)])
			logger.error('%s\t%s',*self.errors[-1][1:])

		self.config['nbest'] = 100 if not cf.isinteger(self.config['nbest']) else abs(self.config['nbest'])
		self.config['lattice-samples'] = 0 if not cf.isinteger(self.config['lattice-samples']) else abs(self.config['lattice-samples'])
		self.config['jobs'] = 0 if not cf.isinteger(self.config['jobs']) else abs(self.config['jobs'])

		if sum([self.config['average'],self.config['closest']]) > 1:
			self.errors.append([__name__,'invalid','\"[%s]\" You can specify just ONE reference length strategy (closest= or shortest= or average=) not more'%(__name__)])
			logger.error('%s\t%s',*self.errors[-1][1:])
		elif sum([self.config['average'],self.config['closest']]) == 1:
			self.config['shortest'] = False
		elif self.config['shortest'] + self.config['average'] + self.config['closest'] == 0:
			self.config['shortest'] = True

		self.config['mertdir'] = self.config['mertdir'] if not self.config['mertdir'] == self.cfg['mertdir'] else os.path.dirname(self.p.decodepath(self.config['mertdir'],True))
		if not self.config['mertdir']:
			self.errors.append([__name__,'invalid','\"[%s]\" mertdir= must be the parent path to the mertdir binary'%(__name__)])
			logger.error('%s\t%s',*self.errors[-1][1:])

		self.config['filtercmd'] = os.path.abspath(os.path.expanduser(self.config['filtercmd'].replace('/',os.sep).replace('\\',os.sep))) if self.config['filtercmd'] else ''

		if self.config['filterfile']:
			self.config['filterfile'] = os.path.abspath(os.path.expanduser(self.config['filterfile']))
			if not os.path.exists(self.config['filterfile']):
				self.errors.append([__name__,'invalid','\"[%s]\" filterfile=%s'%(__name__,self.config['filterfile'])])
				logger.error('%s\t%s',*self.errors[-1][1:])

		self.config['qsubwrapper'] = os.path.abspath(os.path.expanduser(self.config['qsubwrapper'])) if self.config['qsubwrapper'] and not os.path.abspath(os.path.expanduser(self.config['qsubwrapper'])) == os.sep.join([os.path.dirname(self.script),'qsub-wrapper.pl']) else ''
		if self.config['qsubwrapper'] and not os.path.exists(self.config['qsubwrapper']):
			self.errors.append([__name__,'invalid','\"[%s]\" qsubwrapper=%s'%(__name__,self.config['qsubwrapper'])])
			logger.error('%s\t%s',*self.errors[-1][1:])

		if self.config['mosesparallelcmd'] and not os.path.exists(self.config['mosesparallelcmd']):
			self.errors.append([__name__,'invalid','\"[%s]\" mosesparallelcmd=%s'%(__name__,self.config['mosesparallelcmd'])])
			logger.error('%s\t%s',*self.errors[-1][1:])

		if not self.config['no-filter-phrase-table'] == self.cfg['no-filter-phrase-table']:
			self.config['filter-phrase-table'] = not self.config['no-filter-phrase-table']
		if not self.config['filter-phrase-table'] == self.cfg['filter-phrase-table']:
			self.config['no-filter-phrase-table'] = not self.config['filter-phrase-table']

		if self.config['filter-phrase-table']:
			if not self.config['filtercmd']:
				self.errors.append([__name__,'invalid','\"[%s]\" filtercmd= required, option not found'%(__name__)])
				logger.error('%s\t%s',*self.errors[-1][1:])
			elif not os.path.exists(self.config['filtercmd']):
				self.errors.append([__name__,'missing','\"[%s]\" filtercmd=%s, filtering command not found'%(__name__,self.config['filtercmd'])])
				logger.error('%s\t%s',*self.errors[-1][1:])

		if self.config['random-directions'] and not self.config['number-of-random-directions']:
			self.config['number-of-random-directions'] = 50

		self.config['prev-aggregate-nbestlist'] = abs(self.config['prev-aggregate-nbestlist']) if cf.isinteger(self.config['prev-aggregate-nbestlist']) and not self.config['prev-aggregate-nbestlist'] == -1 else -1
		self.config['maximum-iterations'] = int(self.config['maximum-iterations']) if cf.isinteger(self.config['maximum-iterations']) else 25

		self.config['threads'] = int(self.config['threads']) if cf.isinteger(self.config['threads']) and 0 < int(self.config['threads']) <= cf.cpu_count() else 1

		for opt in ['-config','-f','-ttable-file','-t','-distortion-file','-generation-file','-lmodel-file','-global-lexical-file',]:
			if opt in self.config['decoder-flags']:
				self.errors.append([__name__,'invalid','\"[%s]\" decoder-flags= includes forbidden parameter \"%s\"'%(__name__,opt)])
				logger.error('%s\t%s',*self.errors[-1][1:])
		if not '-v' in self.config['decoder-flags']:
			self.config['decoder-flags'] = '%s%s-v 0'%(self.config['decoder-flags'],' ' if self.config['decoder-flags'] else '')
		if not '-threads' in self.config['decoder-flags']:
			self.config['decoder-flags'] = '%s%s-threads %s'%(self.config['decoder-flags'],' ' if self.config['decoder-flags'] else '',1 if self.lmtype == 'irstlm' else self.config['threads'])

#		self.config['skip-decoder']
#		self.config['nocase']
#		self.config['nonorm']
#		self.config['help']
#		self.config['verbose']
#		self.config['mertargs']
#		self.config['mertmertargs']
#		self.config['old-sge']
#		self.config['predictable-seeds']
#		self.config['historic-bests']
#		self.config['random-restarts']
#		self.config['activate-features']
#		self.config['range']
#		self.config['pairwise-ranked']
#		self.config['pro-starting-point']
#		self.config['historic-interpolation']
#		self.config['sc-config']

		# set input file
		self.inputs.append(self.input) # self.config['input']
		self.inputs.append(self.refs) # self.config['refs']
		self.inputs.append(self.mosesconfig) # self.config['config']

		# set output file
		if self.recaser:
			self.outputs = [os.sep.join([os.path.dirname(self.config['working-dir']),'moses-%s.ini'%(self.mertroot),])]
		else:
			self.outputs = [os.sep.join([self.config['working-dir'],'moses-%s.ini'%(self.mertroot)])]

	def run(self,k):
		global skipclose
		skipclose = False

	def flush(self,k):
		return

	def close(self):
		if skipclose: return

		lockfile = os.sep.join(['','var','tmp','domy','train.lock'])
		if not os.path.exists(os.path.dirname(lockfile)): os.makedirs(os.path.dirname(lockfile))
		lock = flock(lockfile,False).acquire()
		if not lock or not lock.ownlock():
			self.errors.append([__name__,'failed','train lock, another training session is running'])
			logger.error('%s\t%s',*self.errors[-1][1:])
			return
		with open(lockfile) as p:
			lockcontent = p.readline().rstrip()

		self.config['continue'] = self.__setcontinue()

		# test input and output files
		if self.recaser:
			for inp in self.inputs:
				if not os.path.exists(inp):
					self.errors.append([__name__,'missing','[%s] %s'%(__name__,inp)])
					logger.error('%s\t%s',*self.errors[-1][1:])
				if self.errors: return
			if self.finishedstep >=self.config['maximum-iterations']:
				self.config['continue'] = False
				self.finishedstep = 0
				if os.path.exists(os.sep.join([self.config['working-dir'],'finished_step.txt'])):
					os.unlink(os.sep.join([self.config['working-dir'],'finished_step.txt']))
		else:
			infodata = cf.updatefingerprint(self.inputs,self.outputs,self.forceinfoupdate)
			if 'bypass' in infodata and self.config['continue']:
				infodata = {'start':'%s'%(cf.timestamp())}
			elif 'bypass' in infodata and (not self.config['continue'] or self.finishedstep >=self.config['maximum-iterations']):
				# output file(s) exists.
				logger.info('%s\t%s',*['complete','tuning succeeded after %s runs.'%(self.finishedstep,)])
				sys.stderr.write('%s - Skipping. [%s] Outputs exist and verified\n'%(cf.timestamp(),__name__))
				lock.release()
				return
			elif 'errors' in infodata:
				# report/return self.errors
				for error in infodata['errors']:
					self.errors.append([__name__,'error',error.strip()])
					logger.error('%s\t%s',*self.errors[-1][1:])
				lock.release()
				return
			sys.stderr.flush()

		# make output folder
		for out in self.outputs+[os.sep.join([self.config['working-dir'],'dummy'])]:
			try:
				os.makedirs(os.path.dirname(out))
			except OSError,e:
				if not e.errno == 17:
					logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
					raise OSError(e)

		if not self.config['continue']:

			# copy mert files to working-dir
			shutil.copyfile(self.input,self.config['input'])
			shutil.copyfile(self.refs,self.config['refs'])
			if not self.mosesconfig == self.config['config']:
				shutil.copyfile(self.mosesconfig,self.config['config'])


		ttl_lines = cf.file_length(self.config['input'])
		progress = ProgressBar()
		progress.reset(0,ttl_lines,72,mode='fixed',char='#')

		# build command line
		misc = {}
		args = list(self.args)
		for k in [k for k in sorted(self.config.keys()) if self.config[k] and not self.config[k] == self.cfg[k]]:
			if type(self.config[k]) is bool:
				if self.config[k]: args.extend(['--%s'%(k)])
			else:
				args.extend(['--%s'%(k), str(self.config[k])])
			misc[k] = self.config[k]

		# make log file for mert-moses.pl output
		with open('%s%s%s.log'%(self.config['working-dir'],os.sep,os.path.basename(self.script)),'a' if self.config['continue'] else 'w') as l:
			logger.info('%s\t%s',*['restart' if self.config['continue'] else 'start','mert-moses.pl'])
			l.write('\n***** %sTARTING AT STEP %s %s\n'%( 'RE-S' if self.config['continue'] else 'S',self.finishedstep+1,'{0:*<48}'.format('')))
			l.write('COMMAND LINE:\n%s %s\n\n'%(self.kw['executable'],' '.join(args).replace(' --',' \\\n   --')))
			l.write('%s - Inputs:\n'%(cf.timestamp()))
			l.write('- %s\n'%('\n- '.join([x for x in self.inputs]) if self.inputs else '<No input files>'))
			l.write('%s - Outputs:\n'%(cf.timestamp()))
			l.write('- %s\n'%('\n- '.join([x for x in self.outputs]) if self.outputs else '<No output files>'))
			l.write('\n')
			l.write('COMMAND LINE:\n%s %s\n'%(self.kw['executable'],' '.join(args).replace(' --',' \\\n --')))

			sys.stderr.write('\n%start -%s mert-moses.pl [%s]\nat run (%s) with a maximum %s runs.\n'%('Re-s' if self.config['continue'] else 'S',' (recaser)' if self.recaser else '',__name__,self.finishedstep+1,self.config['maximum-iterations']))

			# run mert-moses.pl
			try:
				self.subproc['popen'] = subprocess.Popen(args,**self.kw)
			except Exception,e:
				self.errors.append([__name__,'failed','%s %s - %s'%(os.path.basename(self.kw['executable']),' '.join(args),str(e))])
				logger.error('%s\t%s',*self.errors[-1][1:])
				l.write('failed\t%s'%(str(e)))
				l.close()
				lock.release()
				return

			logger.info('%s\t%d - %s %s',*['run',self.subproc['popen'].pid,os.path.basename(self.kw['executable']),' '.join(args)])

			# monitor mert-moses.pl output
			SUCCESS = False
			startbuff = []
			for line in iter(self.subproc['popen'].stdout.readline,''):

				# normalize line format
				line = '(%s) %s'%(self.finishedstep+1,line.rstrip('\r\n')) if line.startswith('run ') or line.startswith('Scoring the nbestlist') or line.startswith('Maximum number of iterations exceeded') or line.startswith('None of the weights changed more than')  else line.rstrip('\r\n')
				if not line: continue
				end = '\n'

				if type(startbuff) is list:
					if line.startswith('Usage:'): break
					if line.startswith('Translating line'):
						startbuff = None
					else:
						startbuff.append(line)

				if line.startswith('Translating line '): # move progress bar
					progress.increment_amount()
					sys.stderr.write(progress.bar)

				elif line.startswith('(%s) run %s start at '%(self.finishedstep+1,self.finishedstep+1)): # start new run, reset progres.bar
					sys.stderr.write('%s%s%s'%(end,line.replace(' run %s '%(self.finishedstep+1),' run %s of %s - '%(self.finishedstep+1,self.config['maximum-iterations'])),end))
					with open(lockfile,'w') as p:
						p.write('%s%s%s%s'%(lockcontent,end,line,end))

				elif line.startswith('(%s) run decoder to produce n-best lists'%(self.finishedstep+1)): # start new run, show progres.bar
					sys.stderr.write('   produce %s-best list for %s phrases\n'%(self.config['nbest'],ttl_lines))
					progress.reset(0,ttl_lines,72,mode='fixed',char='#')
					sys.stderr.write(progress.bar)
					with open(lockfile,'w') as p:
						p.write('%s%s%s%s'%(lockcontent,end,line,end))

				elif line.startswith('(%s) Scoring the nbestlist.'%(self.finishedstep+1)): # end progress.bar display
					progress.update_amount(ttl_lines)
					sys.stderr.write(progress.bar)
					sys.stderr.write('%s   Scoring the list.%s'%(end,end))
					with open(lockfile,'w') as p:
						p.write('%s\n%s%s'%(lockcontent,line,end))
					self.finishedstep += 1
					end = '\n\n'

				elif line.startswith('(%s) run %s end at '%(self.finishedstep+1,self.finishedstep+1)):
					sys.stderr.write('%s%s'%(line,end))
					with open(lockfile,'w') as p:
						p.write('%s%s%s%s'%(lockcontent,end,line,end))

				elif line.startswith('(%s) BEST at %s:'%(self.finishedstep+1,self.finishedstep+1)): # end of run. advance counter & double-space log output, continue loop.
					with open(lockfile,'w') as p:
						p.write('%s%s%s%s'%(lockcontent,end,line,end))

				elif line.startswith('(%s) Maximum number of iterations exceeded'%(self.finishedstep+1)) and line.endswith('stopping'): # loop interrupted by self.config['maximum-iterations']
					sys.stderr.write('%s%s%s'%(end,line,end))
					with open(lockfile,'w') as p:
						p.write('%s%s%s%s'%(lockcontent,end,line,end))
					end = '\n\n'

				elif line.startswith('(%s) None of the weights changed more than'%(self.finishedstep+1)) and line.endswith('Stopping.'): # found optimal settings. complete
					sys.stderr.write('%s%s%s'%(end,line,end))
					with open(lockfile,'w') as p:
						p.write('%s%s%s%s'%(lockcontent,end,line,end))
					end = '\n\n'
					SUCCESS = True

				elif line.startswith('Training finished at'): # final cleanup
					sys.stderr.write('%s%s%s'%(end,line.replace('Training','Tuning'),end))
					with open(lockfile,'w') as p:
						p.write('%s%s%s%s'%(lockcontent,end,line,end))
					end = '\n\n'

				l.write('%s%s'%(line,end))
				l.flush()
				sys.stderr.flush()

		with open(lockfile,'w') as p:
			p.write('%s\n'%(lockcontent))
		sys.stderr.write('\n')
		sys.stderr.flush()

		while self.subproc['popen'].poll() is None:
			time.sleep(0.2)
		if self.subproc['popen'].returncode:
			self.errors.append([__name__,'error','mert-moses.pl - return code %s'%(self.subproc['popen'].returncode)])
			logger.error('%s\t%s',*self.errors[-1][1:])

		if type(startbuff) is list:
			for line in startbuff:
				if line.lower().startswith('error'):
					self.errors.append([__name__,'error',line])
					logger.error('%s\t%s',*self.errors[-1][1:])
			lock.release()
			return

		if os.path.exists(os.sep.join([self.config['working-dir'],'moses.ini'])):
			shutil.copyfile(os.sep.join([self.config['working-dir'],'moses.ini']),self.outputs[0])
			# clean up moses' config.ini
			with open(self.outputs[0],'r') as t:
				contents = t.read().splitlines()
			contents.insert(0,contents.pop(contents.index('### MOSES CONFIG FILE ###')))
			try:
				x = contents.index('[threads]')
				del(contents[x+1])
				del(contents[x])
				del(contents[x-1])
			except:
				pass
			try:
				x = contents.index('[v]')
				del(contents[x+1])
				del(contents[x])
				del(contents[x-1])
			except:
				pass
			with open(self.outputs[0],'w') as t:
				t.write('%s\x0a'%('\x0a'.join(contents)))
		else:
			self.errors.append([__name__,'missing','mert-moses.pl %s'%(os.sep.join([self.config['working-dir'],'moses.ini']))])
			logger.error('%s\t%s',*self.errors[-1][1:])

		finished_run = os.sep.join([self.config['working-dir'],'finished_step.txt'])
		# update infodata and save
		if self.recaser:
			for j,fname in enumerate(self.outputs):
				if not os.path.exists(fname):
					self.errors.append([__name__,'missing','[%s] %s'%(__name__,fname)])
					logger.error('%s\t%s',*self.errors[-1][1:])
			if not self.errors:
				logger.info('%s\t%s',*['success','mert-moses.pl'])
				sys.stderr.write('\nSuccess - mert-moses.pl\n')
		elif self.errors:
			self.errors.append([__name__,'error','mert-moses.pl'])
			logger.error('%s\t%s',*self.errors[-1][1:])
			with open(l.name,'a') as l:
				l.write('%s\t%s'%tuple(self.errors[-1][1:]))
			if os.path.exists(finished_run): os.unlink(finished_run)
		else:
			infodata['misc'] = misc
			infodata = cf.updatefingerprint([],self.outputs,True,infodata)
			if 'errors' in infodata:
				# report/return self.errors
				with open(l.name,'a') as l:
					for error in infodata['errors']:
						self.errors.append([__name__,'error',error.strip()])
						logger.error('%s\t%s',*self.errors[-1][1:])
						l.write('%s\n'%(error.strip()))
				if os.path.exists(finished_run): os.unlink(finished_run)
			else:
				logger.info('%s\t%s',*['success','mert-moses.pl'])
				sys.stderr.write('Success - mert-moses.pl\n')
				with open(finished_run,'w') as t:
					t.write('%s\x0a'%(self.finishedstep))
#				if self.recaser and os.path.dirname(self.inputs[0]) == os.path.dirname(self.config['working-dir']):
#					if os.path.exists(self.config['working-dir']): shutil.rmtree(self.config['working-dir'])

		sys.stderr.flush()
		lock.release()

	def __setcontinue(self):
		'''set to continue automatically after interruption'''

		# find how many steps finished
		finished_run = os.sep.join([self.config['working-dir'],'finished_step.txt'])
		try:
			with open(finished_run,'r') as f:
				self.finishedstep = int(f.readline().strip())
		except:
			self.finishedstep = 0

		return (self.finishedstep < self.config['maximum-iterations']) and os.path.exists(finished_run) and os.path.exists(self.config['input']) and os.path.exists(self.config['refs']) and os.path.exists(self.config['config'])

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import filter\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0]
	)

licensetxt=u'''DoMY™ CE v2.5
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stderr.write(usage().encode('utf8')+'\n')
