#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#2.5.329 - display update to match changes in moses' train-model.perl, step 6.x "creating table half"

import os
import sys
import subprocess
import time
import shutil
import logging
import common as cf
from flock import flock
from glob import glob
from progress_bar import ProgressBar

logger = logging.getLogger('.'.join([os.path.splitext(os.path.basename(sys.argv[0]))[0],'manager','filtergraph',__name__]))
skipclose = True
stages = {
	1: 'preparing corpus',
	2: 'running aligner',
	3: 'generate word alignment',
	4: 'generate lexical translation table',
	5: 'extract phrases',
	6: 'score phrases',
	7: 'learn reordering model',
	8: 'learn generation model',
	9: 'create moses.ini',
	}

class filter(object):

	cfg = {
		'args': {},
		'exeoptions': '-w',
		'kw': {
			'bufsize': 0,
			'executable': '4,perl',
			'stdin': 'PIPE',
			'stdout': 'PIPE',
			'stderr': 'PIPE',
			'preexec_fn': None,
			'close_fds': False,
			'shell': False,
			'cwd': None,
			'env': None,
			'universal_newlines': False,
			'startupinfo': None,
			'creationflags': 0,
			},
		'script': '4,train-model.perl',
		'encoding': 'utf8',
		'recaser': '%(recaser)s',
		'forceinfoupdate': '%(forceinfoupdate)s',
		'threads': 0,
		'tmbuildname': None,
		'alignertype': 'giza',
		'tmgrams': 7,
		'tmroot': '%(tmroot)s',
		'defaultfolders': False,
		'version': '2.5.329',

		'root-dir': '',
		'external-bin-dir': '4,GIZA++',
		'corpus-dir': '',
		'corpus': '',
		'f': '',
		'e': '',
		'giza-e2f': '',
		'giza-f2e': '',
		'max-phrase-length': 7,
		'lexical-file': '',
		'no-lexical-weighting': False,
		'model-dir': '',
		'temp-dir': '',
		'sort-buffer-size': 0,
		'extract-file': '',
		'alignment': 'grow-diag-final',
		'alignment-file': '',
		'alignment-stem': '',
		'verbose': False,
		'first-step': 1,
		'last-step': 9,
		'giza-option': '',
		'giza-extension': 'A3.final',
		'parallel': False,
		'lm': '',
		'help': False,
		'mgiza': False,
		'mgiza-cpus': 0,
		'hmm-align': False,
		'final-alignment-model': '',
		'debug': False,
		'dont-zip': False,
		'parts': 1,
		'direction': 0,
		'only-print-giza': False,
		'reordering': 'distance',
		'reordering-smooth': 0.5,
		'input-factor-max': 0,
		'alignment-factors': '',
		'translation-factors': '',
		'reordering-factors': '',
		'generation-factors': '',
		'decoding-steps': '',
		'decoding-graph-backoff': '',
		'factor-delimiter': '',
		'phrase-translation-table': '',
		'generation-corpus': '',
		'generation-table': '',
		'reordering-table': '',
		'generation-type': '',
		'continue': False,
		'hierarchical': False,
		'glue-grammar': False,
		'glue-grammar-file': '',
		'unknown-word-label-file': '',
		'ghkm': False,
		'extract-options': '',
		'score-options': '',
		'source-syntax': False,
		'target-syntax': False,
		'xml': False,
		'phrase-word-alignment': False,
		'config': '',
		'max-lexical-reordering': False,
		'do-steps': '',
		'memscore': False,
		'force-factored-filenames': False,
		'dictionary': '',
		'eppex': '',
		'additional-ini': '',
		'cores': 0,
		}
	args = []
	script = '4,train-model.perl'
	kw = {}
	encoding = 'utf8'
	subproc = {}
	recaser = False
	forceinfoupdate = False
	threads = 1
	buildset = dict(cf.buildset)
	tmroot = ''
	defaultfolders = False
	inputs = {}
	outputs = {}
	deltemp = True
	config = {
		'root-dir': '',
		'external-bin-dir': '',
		'corpus-dir': '',
		'corpus': '',
		'f': '',
		'e': '',
		'giza-e2f': '',
		'giza-f2e': '',
		'max-phrase-length': 7,
		'lexical-file': '',
		'no-lexical-weighting': False,
		'model-dir': '',
		'temp-dir': '',
		'sort-buffer-size': 0,
		'extract-file': '',
		'alignment': 'grow-diag-final',
		'alignment-file': '',
		'alignment-stem': '',
		'verbose': False,
		'first-step': 1,
		'last-step': 9,
		'giza-option': '',
		'giza-extension': 'A3.final',
		'parallel': False,
		'lm': '',
		'help': False,
		'mgiza': False,
		'mgiza-cpus': 1,
		'hmm-align': False,
		'final-alignment-model': '',
		'debug': False,
		'dont-zip': False,
		'parts': 1,
		'direction': 0,
		'only-print-giza': False,
		'reordering': 'distance',
		'reordering-smooth': 0.5,
		'input-factor-max': 0,
		'alignment-factors': '',
		'translation-factors': '',
		'reordering-factors': '',
		'generation-factors': '',
		'decoding-steps': '',
		'decoding-graph-backoff': '',
		'factor-delimiter': '',
		'phrase-translation-table': '',
		'generation-corpus': '',
		'generation-table': '',
		'reordering-table': '',
		'generation-type': '',
		'continue': False,
		'hierarchical': False,
		'glue-grammar': False,
		'glue-grammar-file': '',
		'unknown-word-label-file': '',
		'ghkm': False,
		'extract-options': '',
		'score-options': '',
		'source-syntax': False,
		'target-syntax': False,
		'xml': False,
		'phrase-word-alignment': False,
		'config': '',
		'max-lexical-reordering': False,
		'do-steps': '',
		'memscore': False,
		'force-factored-filenames': False,
		'dictionary': '',
		'eppex': '',
		'additional-ini': '',
		'cores': 1,
		}
	isopen = False
	p = object
	errors = []

	def __del__(self):
		if not 'popen' in self.subproc: return
		while self.subproc['popen'].poll() is None:
			self.subproc['popen'].terminate()
			time.sleep(0.2)
		if self.deltemp and os.path.exists(self.temp_dir):
			shutil.rmtree(self.temp_dir)

	def open(self,parent,cfg):
		self.args = dict(cfg['args'])
		self.kw = dict(cfg['kw'])
		if cfg['script']:
			self.script = self.p.decodepath(','.join([__name__.split(',')[1]]+cfg['script'].split(',')),True)
			if not self.script:
				self.errors.append([__name__,'missing',self.script])
				logger.warn('%s\t%s',*self.errors[-1][1:])
			self.args[0] = self.script
			self.kw['cwd'] = os.path.dirname(self.script)
		else:
			self.script = None
		# convert args dict to args list
		self.args = [str(self.args[k]) for k in sorted(self.args.keys())]
		if cfg['kw']['executable']:
			self.kw['executable'] = self.p.decodepath(','.join([__name__.split(',')[1]]+cfg['kw']['executable'].split(',')),True)
			if not self.kw['executable']:
				self.errors.append([__name__,'missing',self.kw['executable']])
				logger.warn('%s\t%s',*self.errors[-1][1:])
			self.args.insert(0,cfg['exeoptions'] if cfg['exeoptions'] else '')
		else:
			self.kw['executable'] = None

		self.kw['stdin'] = subprocess.PIPE if cfg['kw']['stdin'] == 'PIPE' else cfg['kw']['stdin']
		self.kw['stdout'] = subprocess.PIPE if cfg['kw']['stdout'] == 'PIPE' else cfg['kw']['stdout']
		self.kw['stderr'] = subprocess.STDOUT if cfg['kw']['stderr'] == 'PIPE' else cfg['kw']['stderr']
		self.encoding = 'utf8' if 'utf8' in cfg['encoding'].lower().replace('-','') else cfg['encoding']

		self.recaser = cf.zbool(cfg['recaser']) if cfg['recaser'] and not cfg['recaser'].startswith('%(') and not cfg['recaser'].endswith(')s') else self.recaser
		self.forceinfoupdate = self.forceinfoupdate if not cfg['forceinfoupdate'] or (cfg['forceinfoupdate'].startswith('%(') and cfg['forceinfoupdate'].endswith(')s')) else cf.zbool(cfg['forceinfoupdate'])
		self.threads = int(cfg['threads']) if cf.isinteger(cfg['threads']) and 0 < int(cfg['threads']) <= cf.cpu_count() else 1

		# test utilities exist
		if not self.p.decodepath('4,gzip',True):
			self.errors.append([__name__,'missing','\"gzip\"'%(__name__,)])
			logger.error('%s\t%s',*self.errors[-1][1:])

		if not self.p.decodepath('4,bzcat',True):
			self.errors.append([__name__,'missing','\"bzcat\"'%(__name__,)])
			logger.error('%s\t%s',*self.errors[-1][1:])

		# test required values
		self.buildset['f'] = '' if not cfg['f'] or cfg['f'] == '%(f)s' or cfg['f'] == '%(srclang)s' else cfg['f']
		if not self.buildset['f']:
			self.errors.append([__name__,'missing','\"[%s]\" f='%(__name__,)])
			logger.error('%s\t%s',*self.errors[-1][1:])
		self.buildset['e'] = '' if not cfg['e'] or cfg['e'] == '%(e)s' or cfg['e'] == '%(tgtlang)s' else cfg['e']
		if not self.buildset['e']:
			self.errors.append([__name__,'missing','\"[%s]\" e='%(__name__,)])
			logger.error('%s\t%s',*self.errors[-1][1:])

		self.buildset['tmbuildname'] = '' if not cfg['tmbuildname'] or (cfg['tmbuildname'].startswith('%(') and cfg['tmbuildname'].endswith(')s')) else cfg['tmbuildname']
		if not self.buildset['tmbuildname']:
			self.errors.append([__name__,'missing','\"[%s]\" tmbuildname must be set'%(__name__,)])
			logger.warn('%s\t%s',*self.errors[-1][1:])

		if 'giza' in cfg['alignertype'].lower():
			self.buildset['alignertype'] = 'giza'
			self.alignment = cfg['alignment']
			cfg['alignment'] = '' if self.alignment == self.cfg['alignment'] else self.alignment
			if not self.alignment in ['intersect','union','grow','grow-final','grow-diag','grow-diag-final','grow-diag-final-and','srctotgt','tgttosrc',]:
				self.errors.append([__name__,'invalid','\"[%s]\"with alignertype=giza, alignment= must be set to \'intersect\', \'union\', \'grow\', \'grow-final\', \'grow-diag\', \'grow-diag-final\' (default), \'grow-diag-final-and\', \'srctotgt\', or \'tgttosrc\''%(__name__,)])
				logger.error('%s\t%s',*self.errors[-1][1:])
			if not self.p.decodepath('4,mgizapp',True):
				self.errors.append([__name__,'missing','\"mgizapp\"'])
				logger.error('%s\t%s',*self.errors[-1][1:])
			if not self.p.decodepath('4,snt2cooc.out',True):
				self.errors.append([__name__,'missing','\"snt2cooc.out\"'])
				logger.error('%s\t%s',*self.errors[-1][1:])
			if not self.p.decodepath('4,mkcls',True):
				self.errors.append([__name__,'missing','\"mkcls\"'])
				logger.error('%s\t%s',*self.errors[-1][1:])
		elif 'berk' in cfg['alignertype'].lower():
			self.errors.append([__name__,'error','\"berk\" unsupported in DoMY CE version'])
			logger.error('%s\t%s',*self.errors[-1][1:])
























		elif 'recaser' in cfg['alignertype'].lower():
			self.buildset['alignertype'] = 'recaser'
			self.alignment = 'word-to-word' #if cfg['alignment'] == 'grow-diag-final' else cfg['alignment']
			cfg['alignment'] = '' if self.alignment == self.cfg['alignment'] else self.alignment
			self.recaser = True
			if not self.alignment in ['word-to-word',]:
				self.errors.append([__name__,'invalid','\"[%s]\" with alignertype=recaser, alignment= must be set to \'word-to-word\''%(__name__,)])
				logger.error('%s\t%s',*self.errors[-1][1:])
		else:
			self.errors.append([__name__,'invalid','\"[%s]\" alignertype must be set to \'giza\', \'recaser\' or \'berk\''%(__name__,)])
			logger.error('%s\t%s',*self.errors[-1][1:])

		cfg['max-phrase-length'] = int(cfg['tmgrams']) if cf.isinteger(cfg['tmgrams']) else self.config['max-phrase-length']
		if cfg['max-phrase-length'] > 9:
			logger.warn('%s\t%s',*['warning','\"[%s]\" max-phrase-length=%s is very large and will consume significant resources'%(__name__,cfg['max-phrase-length'])])

		self.tmroot = cfg['tmroot'] if cfg['tmroot'] and not cfg['tmroot'].startswith('%(') and not cfg['tmroot'].endswith(')s') else 'recasetext' if self.recaser else 'bitext'
		self.defaultfolders = cfg['defaultfolders'] if cfg['defaultfolders'] else self.recaser if self.recaser else (self.buildset['alignertype'] == 'recaser')

		# copy config and verify values
		for k in [k for k in self.config.keys() if k in cfg.keys()]:
			if type(cfg[k]) is str or type(cfg[k]) is unicode:
				self.config[k] = cfg[k].replace('%(rootfolder)s',self.p.rootfolder)
			else:
				self.config[k] = cfg[k]

		external_bin_dir = self.p.decodepath(','.join([__name__.split(',')[1]]+cfg['external-bin-dir'].split(',')),False)
		if not external_bin_dir:
			self.errors.append([__name__,'missing',external_bin_dir])
			logger.warn('%s\t%s',*self.errors[-1][1:])
		self.config['external-bin-dir'] = self.config['external-bin-dir'] if not self.config['external-bin-dir'] == cfg['external-bin-dir'] else external_bin_dir

		self.config['mgiza'] = self.buildset['alignertype'] == 'giza'
		self.config['mgiza-cpus'] = int(self.config['mgiza-cpus']) if cf.isinteger(self.config['mgiza-cpus']) and 0 < int(self.config['mgiza-cpus']) <= cf.cpu_count() else self.threads
		self.config['cores'] = int(self.config['cores']) if cf.isinteger(self.config['cores']) and 0 < int(self.config['cores']) <= cf.cpu_count() else self.threads

		if not self.config['lm']:
			self.config['lm'] = '0:0:/tmp/placeholder.lm:0'
		elif len(self.config['lm'].split(':')) == 4:
			if not self.config['lm'] == '0:0:/tmp/placeholder.lm:0' and not os.path.exists(self.config['lm'].split(':')[2]):
				self.errors.append([__name__,'missing','\"[%s]\" language model, lm=%s'%(__name__,self.config['lm'])])
				logger.error('%s\t%s',*self.errors[-1][1:])
		else:
			self.errors.append([__name__,'invalid','\"[%s]\" lm value must be formatted \'0:0:/path/lmfile:0\''%(__name__,)])
			logger.error('%s\t%s',*self.errors[-1][1:])

		# verify final alignment
		self.config['final-alignment-model'] = 'hmm' if self.config['hmm-align'] else self.config['final-alignment-model']
		if not self.config['final-alignment-model'] in ['',1,2,'hmm',3,4,5]:
			self.errors.append([__name__,'invalid','\"[%s]\" final-alignment-model can be set to \'1\', \'2\', \'hmm\', \'3\', \'4\' or \'5\''%(__name__,)])
			logger.error('%s\t%s',*self.errors[-1][1:])

		giza_extension = 'A3.final'
		if self.config['final-alignment-model'] == 1 and self.buildset['alignertype'] == 'giza':
			giza_extension = 'A1.5'
		elif self.config['final-alignment-model'] == 2 and self.buildset['alignertype'] == 'giza':
			giza_extension = 'A2.5'
		elif self.config['final-alignment-model'] == 'hmm' and self.buildset['alignertype'] == 'giza':
			giza_extension = 'Ahmm.5'
		self.config['giza-extension'] = self.config['giza-extension'] if self.config['giza-extension'] else giza_extension

		if not self.config['direction'] in [0,1,2]:
			self.errors.append([__name__,'invalid','\"[%s]\" direction= can be set to \'0\', \'1\', or \'2\''%(__name__,)])
			logger.error('%s\t%s',*self.errors[-1][1:])

		if not self.config['alignment-factors'] in ['','0-0','0,1,2-0,1']:
			self.errors.append([__name__,'invalid','\"[%s]\" alignment-factors= format for alignment factors is \"0-0\" or \"0,1,2-0,1\", you provided %s'%(__name__,self.config['alignment-factors'])])
			logger.error('%s\t%s',*self.errors[-1][1:])
		if not self.config['translation-factors'] in ['','0-0','0-0+1-1','0-0+0,1-0,1']:
			self.errors.append([__name__,'invalid','\"[%s]\" translation-factors= format for translation factors is \"0-0\" or \"0-0+1-1\" or \"0-0+0,1-0,1\", you provided %s'%(__name__,self.config['translation-factors'])])
			logger.error('%s\t%s',*self.errors[-1][1:])
		if not self.config['reordering-factors'] in ['','0-0','0-0+1-1','0-0+0,1-0,1']:
			self.errors.append([__name__,'invalid','\"[%s]\" reordering-factors= format for reordering factors is \"0-0\" or \"0-0+1-1\" or \"0-0+0,1-0,1\", you provided %s'%(__name__,self.config['reordering-factors'])])
			logger.error('%s\t%s',*self.errors[-1][1:])
		if not self.config['generation-factors'] in ['','0-0','0-0+1-1','0-0+0,1-0,1']:
			self.errors.append([__name__,'invalid','\"[%s]\" generation-factors= format for generation factors is \"0-1\" or \"0-1+0-2\" or \"0-1+0,1-1,2\", you provided %s'%(__name__,self.config['generation-factors'])])
			logger.error('%s\t%s',*self.errors[-1][1:])
		if not self.config['decoding-steps'] in ['','0-0','0-0+1-1','0-0+0,1-0,1']:
			self.errors.append([__name__,'invalid','\"[%s]\" decoding-steps= format for decoding steps is \"t0,g0,t1,g1:t2\", you provided %s'%(__name__,self.config['decoding-steps'])])
			logger.error('%s\t%s',*self.errors[-1][1:])

		self.config['reordering'] = '' if self.config['reordering'] == self.cfg['reordering'] else self.config['reordering']
		for reorder in self.config['reordering'].split(','):
			for aspect in reorder.split('-'):
				if not aspect in ['','wbe','phrase','hier','mslr','msd','monotonicity','leftright','backward','forward','bidirectional','fe','f','allff','collapseff']:
					self.errors.append([__name__,'invalid','\"[%s]\" reorder= aspects can be set to \'wbe\', \'phrase\', \'hier\', \'mslr\', \'msd\', \'monotonicity\', \'leftright\', \'backward\', \'forward\', \'bidirectional\', \'fe\', \'f\', \'allff\', \'collapseff\''%(__name__,)])
					logger.error('%s\t%s',*self.errors[-1][1:])

		# verify steps
		self.config['do-steps'] = '' if self.config['do-steps'] == '%(do-steps)s' else self.config['do-steps']
		dosteps = self.config['do-steps'].split(',') if self.config['do-steps'] else ['%s-%s'%(self.config['first-step'],self.config['last-step'])]
		self.STEPS = []
		for step in dosteps:
			if cf.isinteger(step) and 0 < int(step) < 10:
				f = int(step)
				l = int(step)
			elif '-' in step and len(step.split('-')) == 2:
				f = int(step.split('-')[0])
				l = int(step.split('-')[1])
			else:
				self.errors.append([__name__,'invalid','\"[%s]\" Malformed argument to do-steps='%(__name__,)])
				logger.error('%s\t%s',*self.errors[-1][1:])
				continue
			if f < 1 or l > 9:
				self.errors.append([__name__,'invalid','\"[%s]\" Only steps between 1 and 9 can be used'%(__name__,)])
				logger.error('%s\t%s',*self.errors[-1][1:])
				continue
			if f > l:
				self.errors.append([__name__,'invalid','\"[%s]\" The first step must be smaller than the last step'%(__name__,)])
				logger.error('%s\t%s',*self.errors[-1][1:])
				continue
			self.STEPS.extend(range(f,l+1))

		self.__setios()

	def run(self,k):
		global skipclose
		skipclose = False

	def flush(self,k):
		return

	def close(self):
		if skipclose: return

		lockfile = os.sep.join(['','var','tmp','domy','train.lock'])
		if not os.path.exists(os.path.dirname(lockfile)): os.makedirs(os.path.dirname(lockfile))
		lock = flock(lockfile,False).acquire()
		if not lock or not lock.ownlock():
			self.errors.append([__name__,'failed','train lock, another training session is running'])
			logger.error('%s\t%s',*self.errors[-1][1:])
			return
		with open(lockfile) as f:
			lockcontent = f.readline().rstrip()

		try:
			os.makedirs(self.model_dir)
		except OSError,e:
			if not e.errno == 17:
				logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
				raise OSError(e)

		f = open('%s%s%s.log'%(self.model_dir,os.sep,os.path.basename(self.script)),'w')
		f.write('%s - Inputs:\n'%(cf.timestamp()))
		f.write('- %s\n'%('\n- '.join([x for x in self.inputs[1]]) if self.inputs[1] else '<No input files>'))
		f.write('%s - Outputs:\n'%(cf.timestamp()))
		f.write('- %s\n'%('\n- '.join([x for x in self.outputs[9]]) if self.outputs[9] else '<No output files>'))

		# test input and output files
		if self.recaser:
			for inp in self.inputs[1]:
				if not os.path.exists(inp):
					self.errors.append([__name__,'missing','[%s] %s'%(__name__,inp)])
					logger.error('%s\t%s',*self.errors[-1][1:])
				if self.errors: return
		else:
			infodata = cf.updatefingerprint(self.inputs[1],self.outputs[9],self.forceinfoupdate)
			if 'bypass' in infodata:
				# output file(s) exists.
				logger.info('%s\t%s',*['skipping','outputs exist and verified'])
				sys.stderr.write('%s - Skipping. [%s] Outputs exist and verified\n'%(cf.timestamp(),__name__));
				f.write('%s - Skipping. Outputs exist and verified\n'%(cf.timestamp()))
				lock.release()
				return
			elif 'errors' in infodata:
				# report/return self.errors
				for error in infodata['errors']:
					self.errors.append([__name__,'error',error.strip()])
					logger.error('%s\t%s',*self.errors[-1][1:])
					f.write('%s\n'%(error.strip()))
				lock.release()
				return
			sys.stderr.flush()

		logger.info('%s\t%s',*['start','[%s]\nsteps %s'%(__name__,', '.join([str(step) for step in self.STEPS]))])
		sys.stderr.write('\nStart -%s train-model.perl [%s]\nsteps %s\n'%(' (recaser)' if self.recaser else '',__name__,', '.join([str(step) for step in self.STEPS])))

		# make tmpdir
		try:
			os.makedirs(self.temp_dir)
		except OSError,e:
			if e.errno == 17:
				self.deltemp = False
			else:
				logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
				raise OSError(e)

		parts = int(os.path.getsize(os.extsep.join([self.config['corpus'],self.config['f']])) / 734003200) + 1
		self.config['parts'] = '' if parts == self.cfg['parts'] else parts

		for step in self.STEPS:

			run_aligner = True

			# build command line for train-model.perl
			kw = dict(self.kw)
			args = self.args + ['--do-steps',str(step)]
			for k in [k for k in sorted(self.config.keys()) if not k == 'do-steps' and self.config[k] and not self.config[k] == self.cfg[k]]:
				if type(self.config[k]) is bool:
					if self.config[k]: args.extend(['--%s'%(k)])
				else:
					args.extend(['--%s'%(k), str(self.config[k])])

			ttl_lines = 0

			f.write('***** STEP %s %s\n'%(step,'{0:*<62}'.format('')))

			if (step == 1 or step == 2) and self.buildset['alignertype'] == 'recaser' and self.alignment == 'word-to-word':

				sys.stderr.write('\n')
				msg = '(%s) %s @ %s\n'%(step,stages[step],cf.timestamp())
				f.write(msg)
				sys.stderr.write(msg)
				msg = 'Skipping step %s with alignment=%s'%(step,self.alignment)
				f.write('%s\n'%(msg))
				f.write('\n')
				f.flush()
#				sys.stderr.write('   %s\n'%(msg))
#				sys.stderr.flush()

				run_aligner = False

			elif step == 3 and self.buildset['alignertype'] == 'recaser' and self.alignment == 'word-to-word':

				sys.stderr.write('\n')
				msg = '(%s) %s @ %s\n'%(step,stages[step],cf.timestamp())
				f.write(msg)
				f.write('\n')
				f.flush()
				sys.stderr.write(msg)
				sys.stderr.flush()

				if not os.path.exists(os.path.dirname(self.outputs[step][0])):
					os.makedirs(os.path.dirname(self.outputs[step][0]))

				with open(self.outputs[step][0],'w') as o:
					with open(self.inputs[step][0],'r') as i:
						for line in i:
							o.write('%s\n'%(' '.join(['%s-%s'%(i,i) for i,j in enumerate(line.split())])))

				run_aligner = False
















































































































































































































































			elif step == 2 and self.buildset['alignertype'] == 'giza':

				ttl_lines = cf.file_length(self.inputs[1][0])

			elif step == 4:
				for out in glob('%s.?2?'%(self.lexical_file)):
					os.unlink(out)
				if not os.path.exists(os.path.dirname(self.lexical_file)): os.makedirs(os.path.dirname(self.lexical_file))

			elif step == 5:
				for out in glob('%s*'%(self.extract_file)):
					os.unlink(out)
				if not os.path.exists(os.path.dirname(self.extract_file)): os.makedirs(os.path.dirname(self.extract_file))

			elif step == 6:
				for out in glob(os.sep.join([self.model_dir,'%s-table*.gz'%('rule' if self.config['hierarchical'] else 'phrase')])):
					os.unlink(out)

			elif step == 7:
				old_files = glob(os.sep.join([self.model_dir,'reordering-table*.gz']))

			elif step == 8:
				old_files = glob(os.sep.join([self.model_dir,'generation*.gz']))

			elif step == 9:
				if self.config['lm'] == '0:0:/tmp/placeholder.lm:0':
					with open('/tmp/placeholder.lm','w') as p:
						p.write('%s\n'%(cf.timestamp()))



			# run aligner step set in preparation above
			if run_aligner: self.__run_aligner(kw,args,ttl_lines,f,step,lockfile,lockcontent)



			# evaluate step's results

















			elif step == 6 and not self.errors:
				for out in glob(os.sep.join([self.model_dir,'*-table*.gz'])):
					if os.path.getsize(out) > 35:
						self.outputs[step].append(out)
					else:
						self.errors.append([__name__,'invalid','empty %s table, %s'%('rule' if self.config['hierarchical'] else 'phrase',out)])
						logger.error('%s\t%s',*self.errors[-1][1:])
#						os.unlink(out)

			elif step == 7 and not self.errors:
				for out in [out for out in glob(os.sep.join([self.model_dir,'reordering-table*.gz'])) if not out in old_files and not out in self.outputs[step]]:
					if os.path.getsize(out) > 35:
						self.outputs[step].append(out)
					else:
						self.errors.append([__name__,'invalid','empty reordering table, %s'%(out)])
						logger.error('%s\t%s',*self.errors[-1][1:])
#						os.unlink(out)

			elif step == 8 and not self.errors:
				for out in [out for out in glob(os.sep.join([self.model_dir,'generation*.gz'])) if not out in old_files and not out in self.outputs[step]]:
					if os.path.getsize(out) > 35:
						self.outputs[step].append(out)
					else:
						self.errors.append([__name__,'invalid','empty generation table, %s'%(out)])
						logger.error('%s\t%s',*self.errors[-1][1:])
						os.unlink(out)

			elif step == 9:
				if os.path.exists('/tmp/placeholder.lm'): os.unlink('/tmp/placeholder.lm')
				if self.recaser and not self.errors:
					for out in glob('%s.%s'%(self.alignment_file,self.alignment)):
						os.unlink(out)
					for out in glob('%s.?2?'%(self.lexical_file)):
						os.unlink(out)
					for out in glob('%s*'%(self.extract_file)):
						os.unlink(out)
					if os.path.exists(self.corpus_dir): shutil.rmtree(self.corpus_dir)
					if os.path.exists(self.giza_f2e): shutil.rmtree(self.giza_f2e)
					if os.path.exists(self.giza_e2f): shutil.rmtree(self.giza_e2f)

			# update infodata and save
			if self.recaser:
				for j,fname in enumerate(self.outputs[step]):
					if not os.path.exists(fname):
						self.errors.append([__name__,'missing','train-model.perl, step %s, %s'%(__name__,step,fname)])
						logger.error('%s\t%s',*self.errors[-1][1:])
				if self.errors: break
				logger.info('%s\t%s',*['success','train-model.perl, step %s'%(step)])
				sys.stderr.write('(%s) %s Success\n'%(step,stages[step]))
			elif self.errors:
				self.errors.append([__name__,'fail','train-model.perl, step %s'%(step)])
				logger.error('%s\t%s',*self.errors[-1][1:])
				f.write('%s\t%s'%tuple(self.errors[-1][1:]))
				break
			else:
				infodata = cf.updatefingerprint([],self.outputs[step],True,infodata)
				if 'errors' in infodata:
					# report/return self.errors
					sys.stderr.write('\n')
					for error in infodata['errors']:
						self.errors.append([__name__,'error',error.strip()])
						logger.error('%s\t%s',*self.errors[-1][1:])
						f.write('%s\n'%(error.strip()))
					break
				else:
					logger.info('%s\t%s',*['success','train-model.perl, step %s'%(step)])
					sys.stderr.write('(%s) %s Success\n'%(step,stages[step]))

			sys.stderr.flush()
			f.flush()
			f.write('\n')

		if self.deltemp and os.path.exists(self.temp_dir):
			shutil.rmtree(self.temp_dir)
		f.close()
		lock.release()

	def __setios(self):

		abs_pair = sorted([self.config['f'],self.config['e']])
		alignments_dir = os.sep.join([self.p.rootfolder,'TRAININGS','alignments','align-%s-%s-%s'%(self.buildset['tmbuildname'],abs_pair[0],abs_pair[1])])

		root_dir = os.sep.join([self.p.rootfolder,'ENGINES','recasers' if self.recaser else 'tables'])
		self.config['root-dir'] = self.config['root-dir'] if self.config['root-dir'] else root_dir

		default = os.sep.join([self.config['root-dir'],'corpus'])
		if self.defaultfolders:
			self.corpus_dir = default
		else:
			self.corpus_dir = os.sep.join([alignments_dir,'%s.%s'%(self.buildset['alignertype'],'classes')])
		self.corpus_dir = self.config['corpus-dir'] if self.config['corpus-dir'] else self.corpus_dir
		self.config['corpus-dir'] = '' if self.corpus_dir == default else self.corpus_dir

		default = os.sep.join([self.config['root-dir'],'model'])
		if self.defaultfolders:
			self.model_dir = default
		else:
			self.model_dir = os.sep.join([root_dir,'%s-s=%s-t=%s-p=%s-a=%s-g=%s'%('rule' if self.config['hierarchical'] else 'phrase',self.config['f'],self.config['e'],self.buildset['tmbuildname'],self.buildset['alignertype'],self.config['max-phrase-length'])])
		self.model_dir = self.config['model-dir'] if self.config['model-dir'] else self.model_dir
		self.config['model-dir'] = '' if self.model_dir == default else self.model_dir

		default = os.sep.join([self.model_dir,'aligned'])
		if self.defaultfolders:
			self.alignment_file = default
		else:
			self.alignment_file = os.sep.join([alignments_dir,'%s.%s.%s.%s-%s'%(self.buildset['alignertype'],'aligned',self.config['max-phrase-length'],self.config['f'],self.config['e'])])
		self.alignment_file = self.config['alignment-file'] if self.config['alignment-file'] else self.alignment_file
		self.config['alignment-file'] = '' if self.alignment_file == default else self.alignment_file

		default = os.sep.join([self.model_dir,'extract'])
		if self.defaultfolders:
			self.extract_file = default
		else:
			self.extract_file = os.sep.join([alignments_dir,'%s.%s.%s-%s'%(self.buildset['alignertype'],'extract',self.config['f'],self.config['e']),'ext.%s-gram'%(self.config['max-phrase-length'])])
		self.extract_file = self.config['extract-file'] if self.config['extract-file'] else self.extract_file
		self.config['extract-file'] = '' if self.extract_file == default else self.extract_file

		default = os.sep.join([self.model_dir,'glue-grammar'])
		if self.defaultfolders:
			self.glue_grammar_file = default
		else:
			self.glue_grammar_file = os.sep.join([alignments_dir,'glue-grammar'])
		self.glue_grammar_file = self.config['glue-grammar-file'] if self.config['glue-grammar-file'] else self.glue_grammar_file
		self.config['glue-grammar-file'] = '' if self.glue_grammar_file == default else self.glue_grammar_file

		default = os.sep.join([self.model_dir,'giza.%s-%s'%(self.config['f'],self.config['e'])])
		if self.defaultfolders:
			self.giza_f2e = default
		else:
			self.giza_f2e = os.sep.join([alignments_dir,'%s.%s-%s'%(self.buildset['alignertype'],self.config['f'],self.config['e'])])
		self.giza_f2e = self.config['giza-f2e'] if self.config['giza-f2e'] else self.giza_f2e
		self.config['giza-f2e'] = '' if self.giza_f2e == default else self.giza_f2e

		default = os.sep.join([self.model_dir,'giza.%s-%s'%(self.config['e'],self.config['f'])])
		if self.defaultfolders:
			self.giza_e2f = default
		else:
			self.giza_e2f = os.sep.join([alignments_dir,'%s.%s-%s'%(self.buildset['alignertype'],self.config['e'],self.config['f'])])
		self.giza_e2f = self.config['giza-e2f'] if self.config['giza-e2f'] else self.giza_e2f
		self.config['giza-e2f'] = '' if self.giza_e2f == default else self.giza_e2f

		default = os.sep.join([self.model_dir,'lex'])
		if self.defaultfolders:
			self.lexical_file = default
		else:
			self.lexical_file = os.sep.join([alignments_dir,'%s.%s.%s-%s'%(self.buildset['alignertype'],'lexicon',self.config['f'],self.config['e']),'lex.%s-gram'%(self.config['max-phrase-length'])])
		self.lexical_file = self.config['lexical-file'] if self.config['lexical-file'] else self.lexical_file
		self.config['lexical-file'] = '' if self.lexical_file == default else self.lexical_file

		default = self.model_dir
		if self.defaultfolders:
			self.temp_dir = default
		else:
			self.temp_dir = '%s.tmp'%(os.sep.join([self.p.tempdir,__name__]))
		self.temp_dir = self.config['temp-dir'] if self.config['temp-dir'] else self.temp_dir
		self.config['temp-dir'] = '' if self.temp_dir == default else self.temp_dir

		corpus = os.sep.join([self.p.rootfolder,'BUILDS','lm' if self.recaser else 'tm',self.buildset['tmbuildname'],self.tmroot,])
		self.config['corpus'] = self.config['corpus'] if self.config['corpus'] else corpus

		self.inputs = {}
		self.outputs = {}

		for step in range(1,10):
			self.inputs[step] = []
			self.outputs[step] = []

		self.inputs[1].append('%s.%s'%(self.config['corpus'],self.config['f']))
		self.inputs[1].append('%s.%s'%(self.config['corpus'],self.config['e']))

























		if self.buildset['alignertype'] == 'recaser' and self.alignment == 'word-to-word':
			self.inputs[3].append('%s.%s'%(self.config['corpus'],self.config['f']))
			self.outputs[3].append('%s.%s'%(self.alignment_file,self.alignment))

		self.outputs[9].append('%s%s%s'%(self.model_dir,os.sep,'moses.ini'))

	def __run_aligner(self,kw,args,ttl_lines,f,step,lockfile,lockcontent):

#		print kw['executable'],' '.join(args).replace(' -',' \\\n -')
		f.write('COMMAND LINE:\n%s %s\n'%(kw['executable'],' '.join(args).replace(' -',' \\\n -')))
		try:
			self.subproc['popen'] = subprocess.Popen(args,**kw)
		except Exception,e:
			self.errors.append([__name__,'failed','%s %s - %s'%(os.path.basename(kw['executable']),' '.join(args),str(e))])
			logger.error('%s\t%s',*self.errors[-1][1:])
			f.write('failed\t%s'%(str(e)))
			return

		progress = ProgressBar()

		section = ''
		end = '\n'
		if kw['executable'] == self.kw['executable']:

			for line in iter(self.subproc['popen'].stdout.readline, ''):

				splitline = line.split()
				line = ' '.join(splitline)
				if not splitline or line.lstrip('!').startswith('Use of uninitialized value'): continue
				f.write('%s%s'%(line,end))
				f.flush()

				if (splitline[0].startswith('(') and not splitline[0].startswith('(/')):

					with open(lockfile,'w') as p:
						p.write('%s%s%s%s'%(lockcontent,end,line,end))

					if splitline[0] == '(2.1a)':
						section = splitline[0]
						sys.stderr.write('%s   %s'%(end,line))
						progress.reset(0,ttl_lines,72,mode='fixed',char='#')
						sys.stderr.write('%s'%(end))
						sys.stderr.write(progress.bar)

					elif splitline[0] == '(2.1b)':
						section = splitline[0]
						sys.stderr.write('%s   %s'%(end,line))
						progress.reset(0,100,72,mode='fixed',char='#')
						sys.stderr.write('%s'%(end))
						sys.stderr.write(progress.bar)
						ttl_jobs = 0
						hmmiter,model1iter,model2iter,model3iter,model4iter,model5iter,model6iter = -1,-1,-1,-1,-1,-1,-1

					elif splitline[0] == '(1.2)':
						sys.stderr.write('%s   %s'%(end,' '.join(splitline[0:4]+splitline[5:])))

					elif splitline[0] == '(1.3)':
						sys.stderr.write('%s   %s'%(end,' '.join(splitline[0:3]+splitline[4:])))

					elif splitline[0].startswith('(6.') and ' '.join(splitline[1:4]) == 'creating table half':
						sys.stderr.write('%s   %s'%(end,' '.join(splitline[0:2]+[os.path.basename(os.path.basename(splitline[4]))]+splitline[5:])))

					else:
						sys.stderr.write('%s%s'%(end,'   %s'%(line) if splitline[0].startswith('(') and '.' in splitline[0] else '%s %s'%(os.path.basename(self.model_dir),line)))

				else:

					if section == '(2.1a)':

						tmpline = line.split()

						if tmpline[0] == 'line' and cf.isinteger(tmpline[1]):
							progress.update_amount(int(tmpline[1])+ttl_lines%1000)
							sys.stderr.write(progress.bar)

					elif section == '(2.1b)':

						if not ttl_jobs:

							if 'finished, reusing.' in line:
								progress.update_amount(100)
								sys.stderr.write(progress.bar)

							elif splitline[0] == 'hmmiterations' and hmmiter < 0:
								hmmiter = int(splitline[2])

							elif splitline[0] == 'model1iterations' and model1iter < 0:
								model1iter = int(splitline[2])

							elif splitline[0] == 'model2iterations' and model2iter < 0:
								model2iter = int(splitline[2])

							elif splitline[0] == 'model3iterations' and model3iter < 0:
								model3iter = int(splitline[2])

							elif splitline[0] == 'model4iterations' and model4iter < 0:
								model4iter = int(splitline[2])

							elif splitline[0] == 'model5iterations' and model5iter < 0:
								model5iter = int(splitline[2])

							elif splitline[0] == 'model6iterations' and model6iter < 0:
								model6iter = int(splitline[2])

							if not -1 in [hmmiter,model1iter,model2iter,model3iter,model4iter,model5iter,model6iter]:
								ttl_jobs = self.config['mgiza-cpus'] * sum([hmmiter,model1iter,model2iter,model3iter,model4iter,model5iter,model6iter])
								progress.reset(0,ttl_jobs,72,mode='fixed',char='#')
								sys.stderr.write(progress.bar)

						else:

							if splitline[0] == 'Thread':
								# progress indicator for all iterations of child threads
								progress.increment_amount()
								sys.stderr.write(progress.bar)

							elif line.startswith('Main thread done'):
								# progress indicator for all iterations of master thread of Model 1
								progress.increment_amount()
								sys.stderr.write(progress.bar)

							elif line.startswith('Dump files 0 it'):
								# progress indicator for all iterations of master thread of HMM
								progress.increment_amount()
								sys.stderr.write(progress.bar)

							elif line.startswith('p0_count'):
								# progress indicator for all iterations of master thread of Models 3 & 4
								progress.increment_amount()
								sys.stderr.write(progress.bar)

							elif line.startswith('Combined %s files, totally '%(self.config['mgiza-cpus'])):
								progress.update_amount(ttl_jobs)
								sys.stderr.write(progress.bar)
								sys.stderr.write('%s   %s'%(end,line.replace('sents','sentences')))

					elif splitline[0] in ['Using','WARNING:','ERROR:'] \
						or splitline[0].startswith('1e-'):
						continue

					else:
						sys.stderr.write('.')

				sys.stderr.flush()

		with open(lockfile,'w') as p:
			p.write('%s\n'%(lockcontent))
		sys.stderr.write('\n')
		sys.stderr.flush()

		while self.subproc['popen'].poll() is None:
			time.sleep(0.2)

		if self.subproc['popen'].returncode:
			self.errors.append([__name__,'error','%s. returncode = %s'%(stages[step],self.subproc['popen'].returncode)])
			logger.error('%s\t%s',*self.errors[-1][1:])

'''
CONFIGURATION INDEX

;### HOUSEKEEPING
;help=                              # print help usage
;debug=                             # print verbose output
;verbose=                           # print additional word alignment information
;first-step=1                       # value of first step in the training process (default 1)
;last-step=9                        # value of last step in the training process (default 9)
;do-steps=1,2,3,4,5,6,7,8,9         # value of what steps to run. can be dash-defined range or comma-separted list (overrides first-step/last-step)
;do-steps=1-9                       # value of what steps to run. can be dash-defined range or comma-separted list (overrides first-step/last-step)
;temp-dir=                          # Path to temp directory
;external-bin-dir=                  # Path to set path to GIZA++ & other external scripts
;bin-dir=                           # Path to override default bindir path
;root-dir=                          # Path to root directory, where output files are stored
;factor-delimiter=|                 # value of default factor delimitor character. Default is vertical bar |
;dont-zip=                          # BROKEN. should save uncompressed outputs.
;additional-ini=                    # new
;sort-buffer-size=                  # new
;ghkm=                              # new

;### (1) PREPARE CORPUS
;corpus-dir=                        # Path to corpus directory (default $ROOT/corpus) to store word class files
;corpus=                            # Path to corpus file name (full pathname), excluding extension (also steps 4 & 5)
;e=%(target)s                       # Path extension of the English (target) corpus file (also steps 4 & 5)
;f=%(source)s                       # Path extension of the foreign (source) corpus file (also steps 4 & 5)
;no-lexical-weighting=              # flag - only use conditional probabilities for the t-table, not lexical weighting
;force-factored-filenames=          # flag - use factored file names during factored training, overridden by many other flags
;xml=                               # flag - use xml files for hierarchical models

;### (2) CREATE WORD ALIGNMENTS WITH GIZA (OR BERKELEY in Pro)
;giza-f2e=                          # Path to GIZA++ directory (default $ROOT/giza.$F-$E)
;giza-e2f=                          # path to inverse GIZA++ directory (default $ROOT/giza.$E-$F)
;giza-extension=                    # Path extension of GIZA++ output (less .gz). Not always honored with MGIZA++
;dictionary=                        # Path to create GIZA++ dictionary file outputs
;only-print-giza=                   # flag - display GIZA++ options and stop processing
;parallel=                          # flag - run forward and inverse GIZA++ in parallel
;mgiza=                             # flag - use # multi-threaded mgizapp
;giza-option=                       # value of additional options for GIZA++ training
;parts=1                            # value of how many parts to split/prepare data for GIZA++ by running snt2cooc, break up corpus in smaller parts before GIZA++ training
;mgiza-cpus=                        # value of how many threads during multi-threaded MGIZA++
;direction=                         # value to run training step 2 only in direction 1 or 2 (for parallelization)

;### (3) CREATE WORD ALIGNMENT FILE FROM ALIGNMENTS
;alignment-file=                    # Path to alignment file, excluding extension. alignment value used as extension
;alignment-stem=                    # Path to alignment file, excluding extension. used in step 4: BUILDING LEXICAL TRANSLATION TABLE
;hmm-align=                         # flag - set hmm as final alignment model. Same as final-alignment-model=hmm
;final-alignment-model=             # value of use word alignment model 1/2/hmm/3/4/5 as final (default is 4); value 'hmm' equivalent to the --hmm-align switch
;alignment-factors=                 # value of alignment factors to use. format for alignment factors is "0-0" or "0,1,2-0,1"
;alignment=grow-diag-final          # value of heuristic used for word alignment: intersect, union, grow, grow-final, grow-diag, grow-diag-final (default), grow-diag-final-and, srctotgt, tgttosrc

;### (4) BUILDING LEXICAL TRANSLATION TABLE
;corpus=                            # Path to corpus file name (full pathname), excluding extension
;e=%(target)s                       # Path extension of the English (target) corpus file
;f=%(source)s                       # Path extension of the foreign (source) corpus file
;lexical-file=                      # Path to lexical translation probability file excluding extension (default $ROOT/model)
;max-lexical-reordering=            # flag - sets lexical reordering during step 5 PHRASE EXTRACTION
;force-factored-filenames=          # flag - use factored file names during factored training, overridden by many other flags
;xml=                               # flag - use xml files for hierarchical models
;reordering=distance                # value specifies which reordering models to train using a comma-separated list of config-strings, see FactoredTraining.BuildReorderingModel. (default distance)
;translation-factors=               # value of translation factors to use. format for translation factors is "0-0" or "0-0+1-1" or "0-0+0,1-0,1"

;### (5) PHRASE EXTRACTION
;corpus=                            # Path to corpus file name (full pathname), excluding extension
;e=%(target)s                       # Path extension of the English (target) corpus file
;f=%(source)s                       # Path extension of the foreign (source) corpus file
;extract-file=                      # Path to extraction file, excluding extension (default $ROOT/model/extract)
;eppex=                             # Path to alternate "eppex" application to extract phrases in step 5 PHRASE EXTRACTION. sets max_phrase_length itself (as the maximum phrase length for which any Lossy Counter is defined)
;force-factored-filenames=          # flag - use factored file names during factored training, overridden by many other flags
;max-phrase-length=                 # value of maximum length of phrases (n-grams) entered into t-table (default 7-grams)
;reordering-factors=                # value of reordering factors to use. format for reordering factors is "0-0" or "0-0+1-1" or "0-0+0,1-0,1"
;hierarchical=                      # flag - build hierarchical t-tables
;translation-factors=               # value of translation factors to use. format for translation factors is "0-0" or "0-0+1-1" or "0-0+0,1-0,1"
;glue-grammar=                      # flag - use glue grammar file
;glue-grammar-file=                 # Path to GlueGrammar file for hierarchical training

;### (6) PHRASE SCORING (t-table)
;model-dir=                         # Path to model directory (default $ROOT/model)
;phrase-translation-table=          # Path to t-tables to build... comma-separated list. for the [ttable-file] ini file section
;phrase-word-alignment=             # flag - use WordAlignment
;memscore=                          # flag - use memscore as the phrase scorer, otherwise defaults to use "phrase-extract"
;continue=                          # flag - step 6 to resume PHRASE SCORING
;force-factored-filenames=          # flag - use factored file names during factored training, overridden by many other flags
;score-options=                     # values of score options. values can be OnlyDirect, NoPhraseCount
;hierarchical=                      # flag - build hierarchical t-tables
;unknown-word-label-file=           # Path to UnknownWordLabel file for hierarchical training
;translation-factors=               # value of translation factors to use. format for translation factors is "0-0" or "0-0+1-1" or "0-0+0,1-0,1"
;reordering=distance                # value specifies which reordering models to train using a comma-separated list of config-strings, see FactoredTraining.BuildReorderingModel. (default distance)
;source-syntax=                     # Path to source language syntax file for hierarchical models
;target-syntax=                     # Path to target language syntax file for hierarchical models
;extract-options=                   # values of extract options for hierarchical training

;### (7) LEARN REORDERING MODEL
;model-dir=                         # Path to model directory (default $ROOT/model)
;force-factored-filenames=          # flag - use factored file names during factored training, overridden by many other flags
;reordering-factors=                # value of reordering factors to use. format for reordering factors is "0-0" or "0-0+1-1" or "0-0+0,1-0,1"
;reordering-smooth=0.5              # value specifies the smoothing constant to be used for training lexicalized reordering models. If the letter "u" follows the constant, smoothing is based on actual counts. (default 0.5)
;reordering=distance                # value specifies which reordering models to train using a comma-separated list of config-strings, see FactoredTraining.BuildReorderingModel. (default distance)

;### (8) LEARN GENERATION MODEL
;model-dir=                         # Path to model directory (default $ROOT/model)
;reordering-table=                  # Path to reordering tables to build... comma-separated list. for the [distortion-file] ini file section
;generation-corpus=                 # Path to generation corpus for generation table
;generation-table=                  # Path to generation tables to build... comma-separated list. for the [generation-file] ini file section
;generation-factors=                # value of generation factors to use. format for generation factors is "0-1" or "0-1+0-2" or "0-1+0,1-1,2"
;generation-type=                   # value of the generation type. no further information

;### (9) CREATE CONFIGURATION FILE
;config=                            # Path to save final moses configuration file
;lm=                                # value of language model: <factor>:<order>:<full path filename> (option can be repeated), non-zero length file must exist. comma-separated list. for the [lmodel-file] ini file section
;decoding-steps=                    # value of decoding steps to use. format for decoding steps is "t0,g0,t1,g1:t2"
;force-factored-filenames=          # flag - use factored file names during factored training, overridden by many other flags
;input-factor-max=                  # value of maximum input factors for factored training
;reordering-factors=                # value of reordering factors to use. format for reordering factors is "0-0" or "0-0+1-1" or "0-0+0,1-0,1"
;hierarchical=                      # flag - build hierarchical t-tables
;translation-factors=               # value of translation factors to use. format for translation factors is "0-0" or "0-0+1-1" or "0-0+0,1-0,1"
;reordering=distance                # value specifies which reordering models to train using a comma-separated list of config-strings, see FactoredTraining.BuildReorderingModel. (default distance)
;decoding-graph-backoff=            # value to insert in moses configuration file as [decoding-graph-backoff]
;glue-grammar=                      # flag - use glue grammar file
;score-options=                     # values of score options. values can be OnlyDirect, NoPhraseCount
'''

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import filter\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0]
	)

licensetxt=u'''DoMY™ CE v2.5
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stderr.write(usage().encode('utf8')+'\n')
